@@ -2,7 +2,7 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, Rafal Bobrowski
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #ifndef SOCI_FIREBIRD_COMMON_H_INCLUDED
@@ -10,6 +10,8 @@
 
 #include "soci/firebird/soci-firebird.h"
 #include "soci-compiler.h"
+#include "soci-ssize.h"
+#include <cstdint>
 #include <cstdlib>
 #include <cstring>
 #include <ctime>
@@ -135,20 +137,20 @@ void to_isc(void * val, XSQLVAR * var, short x_scale = 0)
     {
     case SQL_SHORT:
         {
-            short tmp = static_cast<short>(round_for_isc(value*multiplier)/divisor);
-            std::memcpy(var->sqldata, &tmp, sizeof(short));
+            int16_t tmp = static_cast<int16_t>(round_for_isc(value*multiplier)/divisor);
+            std::memcpy(var->sqldata, &tmp, sizeof(int16_t));
         }
         break;
     case SQL_LONG:
         {
-            int tmp = static_cast<int>(round_for_isc(value*multiplier)/divisor);
-            std::memcpy(var->sqldata, &tmp, sizeof(int));
+            int32_t tmp = static_cast<int32_t>(round_for_isc(value*multiplier)/divisor);
+            std::memcpy(var->sqldata, &tmp, sizeof(int32_t));
         }
         break;
     case SQL_INT64:
         {
-            long long tmp = static_cast<long long>(round_for_isc(value*multiplier)/divisor);
-            std::memcpy(var->sqldata, &tmp, sizeof(long long));
+            int64_t tmp = static_cast<int64_t>(round_for_isc(value*multiplier)/divisor);
+            std::memcpy(var->sqldata, &tmp, sizeof(int64_t));
         }
         break;
     case SQL_FLOAT:
@@ -192,7 +194,7 @@ std::string format_decimal(const void *sqldata, int sqlscale)
     std::string r = out.str();
     if (sqlscale < 0)
     {
-        if (static_cast<int>(r.size()) - (x < 0) <= -sqlscale)
+        if (ssize(r) - (x < 0) <= -sqlscale)
         {
             r = std::string(size_t(x < 0), '-') +
                 std::string(-sqlscale - (r.size() - (x < 0)) + 1, '0') +
@@ -235,16 +237,16 @@ T1 from_isc(XSQLVAR * var)
         }
     }
 
-    GCC_WARNING_SUPPRESS(cast-align)
+    SOCI_GCC_WARNING_SUPPRESS(cast-align)
 
     switch (var->sqltype & ~1)
     {
     case SQL_SHORT:
-        return static_cast<T1>(*reinterpret_cast<short*>(var->sqldata)/tens);
+        return static_cast<T1>(*reinterpret_cast<int16_t*>(var->sqldata)/tens);
     case SQL_LONG:
-        return static_cast<T1>(*reinterpret_cast<int*>(var->sqldata)/tens);
+        return static_cast<T1>(*reinterpret_cast<int32_t*>(var->sqldata)/tens);
     case SQL_INT64:
-        return static_cast<T1>(*reinterpret_cast<long long*>(var->sqldata)/tens);
+        return static_cast<T1>(*reinterpret_cast<int64_t*>(var->sqldata)/tens);
     case SQL_FLOAT:
         return static_cast<T1>(*reinterpret_cast<float*>(var->sqldata));
     case SQL_DOUBLE:
@@ -253,7 +255,7 @@ T1 from_isc(XSQLVAR * var)
         throw soci_error("Incorrect data type for numeric conversion");
     }
 
-    GCC_WARNING_RESTORE(cast-align)
+    SOCI_GCC_WARNING_RESTORE(cast-align)
 }
 
 } // namespace firebird
