@@ -2,7 +2,7 @@
 // Copyright (C) 2015 Vadim Zeitlin.
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #ifndef SOCI_PRIVATE_SOCI_MKTIME_H_INCLUDED
@@ -21,6 +21,20 @@ namespace soci
 namespace details
 {
 
+SOCI_DECL time_t timegm_impl_soci ( struct tm* tb );
+
+template <typename T>
+auto timegm_impl(T* t) -> decltype(timegm(t))
+{
+    return timegm(t);    
+}
+
+template <typename T>
+auto timegm_impl(T t) -> time_t
+{
+    return timegm_impl_soci(t);
+}
+
 // Fill the provided struct tm with the values corresponding to the given date
 // in UTC.
 //
@@ -40,7 +54,7 @@ mktime_from_ymdhms(tm& t,
     t.tm_min  = minute;
     t.tm_sec  = second;
 
-    timegm(&t);
+    timegm_impl(&t);
 }
 
 // Helper function for parsing datetime values.
@@ -48,6 +62,17 @@ mktime_from_ymdhms(tm& t,
 // Throws if the string in buf couldn't be parsed as a date or a time string.
 SOCI_DECL void parse_std_tm(char const *buf, std::tm &t);
 
+// Reverse function for formatting datetime values.
+//
+// The string returned in the buffer uses YYYY-MM-DD HH:MM:SS format.
+inline int format_std_tm(std::tm const& t, char* buf, std::size_t len)
+{
+    return snprintf(buf, len,
+        "%d-%02d-%02d %02d:%02d:%02d",
+        t.tm_year + 1900, t.tm_mon + 1, t.tm_mday,
+        t.tm_hour, t.tm_min, t.tm_sec);
+}
+
 } // namespace details
 
 } // namespace soci
