@@ -2,7 +2,7 @@
 // Copyright (C) 2008 Maciej Sobczak
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #ifndef SOCI_BOOST_GREGORIAN_DATE_H_INCLUDED
@@ -34,6 +34,9 @@ struct type_conversion<boost::gregorian::date>
         out = boost::gregorian::date_from_tm(in);
     }
 
+    struct move_from_base_check :
+        std::integral_constant<bool, false> {};
+
     static void to_base(
         boost::gregorian::date const & in, base_type & out, indicator & ind)
     {
