@@ -3,7 +3,7 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #ifndef SOCI_DB2_H_INCLUDED
@@ -57,11 +57,16 @@ namespace soci
 class SOCI_DB2_DECL db2_soci_error : public soci_error {
 public:
     db2_soci_error(std::string const & msg, SQLRETURN rc) : soci_error(msg),errorCode(rc) {};
-    ~db2_soci_error() SOCI_NOEXCEPT SOCI_OVERRIDE { };
+    ~db2_soci_error() noexcept override { };
 
     //We have to extract error information before exception throwing, cause CLI handles could be broken at the construction time
     static const std::string sqlState(std::string const & msg,const SQLSMALLINT htype,const SQLHANDLE hndl);
 
+    std::string get_backend_name() const override { return "db2"; }
+    int get_backend_error_code() const override { return errorCode; }
+
+    // This member variable is only public for compatibility, don't use it
+    // directly, call get_backend_error_code() instead.
     SQLRETURN errorCode;
 };
 
@@ -79,12 +84,12 @@ struct SOCI_DB2_DECL db2_standard_into_type_backend : details::standard_into_typ
         : statement_(st),buf(NULL)
     {}
 
-    void define_by_pos(int& position, void* data, details::exchange_type type) SOCI_OVERRIDE;
+    void define_by_pos(int& position, void* data, details::exchange_type type) override;
 
-    void pre_fetch() SOCI_OVERRIDE;
-    void post_fetch(bool gotData, bool calledFromFetch, indicator* ind) SOCI_OVERRIDE;
+    void pre_fetch() override;
+    void post_fetch(bool gotData, bool calledFromFetch, indicator* ind) override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     db2_statement_backend& statement_;
 
@@ -102,15 +107,15 @@ struct SOCI_DB2_DECL db2_vector_into_type_backend : details::vector_into_type_ba
         : statement_(st),buf(NULL)
     {}
 
-    void define_by_pos(int& position, void* data, details::exchange_type type) SOCI_OVERRIDE;
+    void define_by_pos(int& position, void* data, details::exchange_type type) override;
 
-    void pre_fetch() SOCI_OVERRIDE;
-    void post_fetch(bool gotData, indicator* ind) SOCI_OVERRIDE;
+    void pre_fetch() override;
+    void post_fetch(bool gotData, indicator* ind) override;
 
-    void resize(std::size_t sz) SOCI_OVERRIDE;
-    std::size_t size() SOCI_OVERRIDE;
+    void resize(std::size_t sz) override;
+    std::size_t size() const override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     db2_statement_backend& statement_;
 
@@ -131,13 +136,13 @@ struct SOCI_DB2_DECL db2_standard_use_type_backend : details::standard_use_type_
         : statement_(st),buf(NULL),ind(0)
     {}
 
-    void bind_by_pos(int& position, void* data, details::exchange_type type, bool readOnly) SOCI_OVERRIDE;
-    void bind_by_name(std::string const& name, void* data, details::exchange_type type, bool readOnly) SOCI_OVERRIDE;
+    void bind_by_pos(int& position, void* data, details::exchange_type type, bool readOnly) override;
+    void bind_by_name(std::string const& name, void* data, details::exchange_type type, bool readOnly) override;
 
-    void pre_use(indicator const* ind) SOCI_OVERRIDE;
-    void post_use(bool gotData, indicator* ind) SOCI_OVERRIDE;
+    void pre_use(indicator const* ind) override;
+    void post_use(bool gotData, indicator* ind) override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     db2_statement_backend& statement_;
 
@@ -156,14 +161,14 @@ struct SOCI_DB2_DECL db2_vector_use_type_backend : details::vector_use_type_back
     db2_vector_use_type_backend(db2_statement_backend &st)
         : statement_(st),buf(NULL) {}
 
-    void bind_by_pos(int& position, void* data, details::exchange_type type) SOCI_OVERRIDE;
-    void bind_by_name(std::string const& name, void* data, details::exchange_type type) SOCI_OVERRIDE;
+    void bind_by_pos(int& position, void* data, details::exchange_type type) override;
+    void bind_by_name(std::string const& name, void* data, details::exchange_type type) override;
 
-    void pre_use(indicator const* ind) SOCI_OVERRIDE;
+    void pre_use(indicator const* ind) override;
 
-    std::size_t size() SOCI_OVERRIDE;
+    std::size_t size() const override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     db2_statement_backend& statement_;
 
@@ -183,27 +188,27 @@ struct SOCI_DB2_DECL db2_statement_backend : details::statement_backend
 {
     db2_statement_backend(db2_session_backend &session);
 
-    void alloc() SOCI_OVERRIDE;
-    void clean_up() SOCI_OVERRIDE;
-    void prepare(std::string const& query, details::statement_type eType) SOCI_OVERRIDE;
+    void alloc() override;
+    void clean_up() override;
+    void prepare(std::string const& query, details::statement_type eType) override;
 
-    exec_fetch_result execute(int number) SOCI_OVERRIDE;
-    exec_fetch_result fetch(int number) SOCI_OVERRIDE;
+    exec_fetch_result execute(int number) override;
+    exec_fetch_result fetch(int number) override;
 
-    long long get_affected_rows() SOCI_OVERRIDE;
-    int get_number_of_rows() SOCI_OVERRIDE;
-    std::string get_parameter_name(int index) const SOCI_OVERRIDE;
+    long long get_affected_rows() override;
+    int get_number_of_rows() override;
+    std::string get_parameter_name(int index) const override;
 
-    std::string rewrite_for_procedure_call(std::string const& query) SOCI_OVERRIDE;
+    std::string rewrite_for_procedure_call(std::string const& query) override;
 
-    int prepare_for_describe() SOCI_OVERRIDE;
-    void describe_column(int colNum, data_type& dtype, std::string& columnName) SOCI_OVERRIDE;
+    int prepare_for_describe() override;
+    void describe_column(int colNum, db_type& dbtype, std::string& columnName) override;
     size_t column_size(int col);
 
-    db2_standard_into_type_backend* make_into_type_backend() SOCI_OVERRIDE;
-    db2_standard_use_type_backend* make_use_type_backend() SOCI_OVERRIDE;
-    db2_vector_into_type_backend* make_vector_into_type_backend() SOCI_OVERRIDE;
-    db2_vector_use_type_backend* make_vector_use_type_backend() SOCI_OVERRIDE;
+    db2_standard_into_type_backend* make_into_type_backend() override;
+    db2_standard_use_type_backend* make_use_type_backend() override;
+    db2_vector_into_type_backend* make_vector_into_type_backend() override;
+    db2_vector_use_type_backend* make_vector_use_type_backend() override;
 
     db2_session_backend& session_;
 
@@ -215,49 +220,50 @@ struct SOCI_DB2_DECL db2_statement_backend : details::statement_backend
     details::db2::binding_method use_binding_method_;
 };
 
-struct db2_rowid_backend : details::rowid_backend
+struct SOCI_DB2_DECL db2_rowid_backend : details::rowid_backend
 {
     db2_rowid_backend(db2_session_backend &session);
 
-    ~db2_rowid_backend() SOCI_OVERRIDE;
+    ~db2_rowid_backend() override;
 };
 
-struct db2_blob_backend : details::blob_backend
+struct SOCI_DB2_DECL db2_blob_backend : details::blob_backend
 {
     db2_blob_backend(db2_session_backend& session);
 
-    ~db2_blob_backend() SOCI_OVERRIDE;
+    ~db2_blob_backend() override;
 
-    std::size_t get_len() SOCI_OVERRIDE;
-    std::size_t read(std::size_t offset, char* buf, std::size_t toRead) SOCI_OVERRIDE;
-    std::size_t write(std::size_t offset, char const* buf, std::size_t toWrite) SOCI_OVERRIDE;
-    std::size_t append(char const* buf, std::size_t toWrite) SOCI_OVERRIDE;
-    void trim(std::size_t newLen) SOCI_OVERRIDE;
+    std::size_t get_len() override;
+    std::size_t read_from_start(void* buf, std::size_t toRead, std::size_t offset = 0) override;
+    std::size_t write_from_start(const void* buf, std::size_t toWrite, std::size_t offset = 0) override;
+    std::size_t append(const void* buf, std::size_t toWrite) override;
+    void trim(std::size_t newLen) override;
+    details::session_backend &get_session_backend() override;
 
     db2_session_backend& session_;
 };
 
-struct db2_session_backend : details::session_backend
+struct SOCI_DB2_DECL db2_session_backend : details::session_backend
 {
     db2_session_backend(connection_parameters const& parameters);
 
-    ~db2_session_backend() SOCI_OVERRIDE;
+    ~db2_session_backend() override;
 
-    bool is_connected() SOCI_OVERRIDE;
+    bool is_connected() override;
 
-    void begin() SOCI_OVERRIDE;
-    void commit() SOCI_OVERRIDE;
-    void rollback() SOCI_OVERRIDE;
+    void begin() override;
+    void commit() override;
+    void rollback() override;
 
-    std::string get_dummy_from_table() const SOCI_OVERRIDE { return "sysibm.sysdummy1"; }
+    std::string get_dummy_from_table() const override { return "sysibm.sysdummy1"; }
 
-    std::string get_backend_name() const SOCI_OVERRIDE { return "DB2"; }
+    std::string get_backend_name() const override { return "DB2"; }
 
     void clean_up();
 
-    db2_statement_backend* make_statement_backend() SOCI_OVERRIDE;
-    db2_rowid_backend* make_rowid_backend() SOCI_OVERRIDE;
-    db2_blob_backend* make_blob_backend() SOCI_OVERRIDE;
+    db2_statement_backend* make_statement_backend() override;
+    db2_rowid_backend* make_rowid_backend() override;
+    db2_blob_backend* make_blob_backend() override;
 
     void parseConnectString(std::string const &);
     void parseKeyVal(std::string const &);
@@ -274,7 +280,7 @@ struct SOCI_DB2_DECL db2_backend_factory : backend_factory
 {
     db2_backend_factory() {}
     db2_session_backend* make_session(
-        connection_parameters const & parameters) const SOCI_OVERRIDE;
+        connection_parameters const & parameters) const override;
 };
 
 extern SOCI_DB2_DECL db2_backend_factory const db2;
