@@ -2,7 +2,7 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, Rafal Bobrowski
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 //
 
@@ -25,23 +25,21 @@
 #include <cstdlib>
 #include <vector>
 #include <string>
+#include <cstdint>
 
 namespace soci
 {
 
-std::size_t const stat_size = 20;
-
-// size of buffer for error messages. All examples use this value.
-// Anyone knows, where it is stated that 512 bytes is enough ?
-std::size_t const SOCI_FIREBIRD_ERRMSG = 512;
-
 class SOCI_FIREBIRD_DECL firebird_soci_error : public soci_error
 {
 public:
     firebird_soci_error(std::string const & msg,
         ISC_STATUS const * status = 0);
 
-    ~firebird_soci_error() SOCI_NOEXCEPT SOCI_OVERRIDE {};
+    error_category get_error_category() const override;
+
+    std::string get_backend_name() const override { return "firebird"; }
+    int get_backend_error_code() const override;
 
     std::vector<ISC_STATUS> status_;
 };
@@ -60,13 +58,13 @@ struct firebird_standard_into_type_backend : details::standard_into_type_backend
     {}
 
     void define_by_pos(int &position,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
 
-    void pre_fetch() SOCI_OVERRIDE;
+    void pre_fetch() override;
     void post_fetch(bool gotData, bool calledFromFetch,
-        indicator *ind) SOCI_OVERRIDE;
+        indicator *ind) override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     firebird_statement_backend &statement_;
     virtual void exchangeData();
@@ -86,15 +84,15 @@ struct firebird_vector_into_type_backend : details::vector_into_type_backend
     {}
 
     void define_by_pos(int &position,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
 
-    void pre_fetch() SOCI_OVERRIDE;
-    void post_fetch(bool gotData, indicator *ind) SOCI_OVERRIDE;
+    void pre_fetch() override;
+    void post_fetch(bool gotData, indicator *ind) override;
 
-    void resize(std::size_t sz) SOCI_OVERRIDE;
-    std::size_t size() SOCI_OVERRIDE;
+    void resize(std::size_t sz) override;
+    std::size_t size() const override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     firebird_statement_backend &statement_;
     virtual void exchangeData(std::size_t row);
@@ -115,14 +113,14 @@ struct firebird_standard_use_type_backend : details::standard_use_type_backend
     {}
 
     void bind_by_pos(int &position,
-        void *data, details::exchange_type type, bool readOnly) SOCI_OVERRIDE;
+        void *data, details::exchange_type type, bool readOnly) override;
     void bind_by_name(std::string const &name,
-        void *data, details::exchange_type type, bool readOnly) SOCI_OVERRIDE;
+        void *data, details::exchange_type type, bool readOnly) override;
 
-    void pre_use(indicator const *ind) SOCI_OVERRIDE;
-    void post_use(bool gotData, indicator *ind) SOCI_OVERRIDE;
+    void pre_use(indicator const *ind) override;
+    void post_use(bool gotData, indicator *ind) override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     firebird_statement_backend &statement_;
     virtual void exchangeData();
@@ -151,15 +149,15 @@ struct firebird_vector_use_type_backend : details::vector_use_type_backend
     {}
 
     void bind_by_pos(int &position,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
     void bind_by_name(std::string const &name,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
 
-    void pre_use(indicator const *ind) SOCI_OVERRIDE;
+    void pre_use(indicator const *ind) override;
 
-    std::size_t size() SOCI_OVERRIDE;
+    std::size_t size() const override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     firebird_statement_backend &statement_;
     virtual void exchangeData(std::size_t row);
@@ -182,32 +180,34 @@ private:
 };
 
 struct firebird_session_backend;
-struct firebird_statement_backend : details::statement_backend
+struct SOCI_FIREBIRD_DECL firebird_statement_backend : details::statement_backend
 {
     firebird_statement_backend(firebird_session_backend &session);
 
-    void alloc() SOCI_OVERRIDE;
-    void clean_up() SOCI_OVERRIDE;
+    void alloc() override;
+    void clean_up() override;
     void prepare(std::string const &query,
-        details::statement_type eType) SOCI_OVERRIDE;
+        details::statement_type eType) override;
 
-    exec_fetch_result execute(int number) SOCI_OVERRIDE;
-    exec_fetch_result fetch(int number) SOCI_OVERRIDE;
+    exec_fetch_result execute(int number) override;
+    exec_fetch_result fetch(int number) override;
 
-    long long get_affected_rows() SOCI_OVERRIDE;
-    int get_number_of_rows() SOCI_OVERRIDE;
-    std::string get_parameter_name(int index) const SOCI_OVERRIDE;
+    long long get_affected_rows() override;
+    int get_number_of_rows() override;
+    std::string get_parameter_name(int index) const override;
+    int get_row_to_dump() const override { return current_row_; }
 
-    std::string rewrite_for_procedure_call(std::string const &query) SOCI_OVERRIDE;
+    std::string rewrite_for_procedure_call(std::string const &query) override;
 
-    int prepare_for_describe() SOCI_OVERRIDE;
-    void describe_column(int colNum, data_type &dtype,
-        std::string &columnName) SOCI_OVERRIDE;
+    int prepare_for_describe() override;
+    void describe_column(int colNum,
+        db_type &dbtype,
+        std::string &columnName) override;
 
-    firebird_standard_into_type_backend * make_into_type_backend() SOCI_OVERRIDE;
-    firebird_standard_use_type_backend * make_use_type_backend() SOCI_OVERRIDE;
-    firebird_vector_into_type_backend * make_vector_into_type_backend() SOCI_OVERRIDE;
-    firebird_vector_use_type_backend * make_vector_use_type_backend() SOCI_OVERRIDE;
+    firebird_standard_into_type_backend * make_into_type_backend() override;
+    firebird_standard_use_type_backend * make_use_type_backend() override;
+    firebird_vector_into_type_backend * make_vector_into_type_backend() override;
+    firebird_vector_use_type_backend * make_vector_use_type_backend() override;
 
     firebird_session_backend &session_;
 
@@ -229,6 +229,9 @@ protected:
 
     long long rowsAffectedBulk_; // number of rows affected by the last bulk operation
 
+    // Return the number of rows affected by last operation.
+    long long get_last_row_count();
+
     virtual void exchangeData(bool gotData, int row);
     virtual void prepareSQLDA(XSQLDA ** sqldap, short size = 10);
     virtual void rewriteQuery(std::string const & query,
@@ -247,83 +250,81 @@ protected:
     std::map <std::string, int> names_;
 
     bool procedure_;
+
+private:
+    // Used during bulk operations to keep track of the row which potentially
+    // resulted in an error.
+    int current_row_ = -1;
 };
 
-struct firebird_blob_backend : details::blob_backend
+struct SOCI_FIREBIRD_DECL firebird_blob_backend : details::blob_backend
 {
     firebird_blob_backend(firebird_session_backend &session);
 
-    ~firebird_blob_backend() SOCI_OVERRIDE;
+    ~firebird_blob_backend() override;
 
-    std::size_t get_len() SOCI_OVERRIDE;
-    std::size_t read(std::size_t offset, char *buf,
-        std::size_t toRead) SOCI_OVERRIDE;
-    std::size_t write(std::size_t offset, char const *buf,
-        std::size_t toWrite) SOCI_OVERRIDE;
-    std::size_t append(char const *buf, std::size_t toWrite) SOCI_OVERRIDE;
-    void trim(std::size_t newLen) SOCI_OVERRIDE;
+    std::size_t get_len() override;
+
+    std::size_t read_from_start(void * buf, std::size_t toRead, std::size_t offset = 0) override;
+
+    std::size_t write_from_start(const void * buf, std::size_t toWrite, std::size_t offset = 0) override;
+
+    std::size_t append(const void *buf, std::size_t toWrite) override;
+    void trim(std::size_t newLen) override;
+
+    details::session_backend &get_session_backend() override;
+
+    // Writes the current data into the database by allocating a new BLOB
+    // object for it.
+    //
+    // Returns The ID of the newly created BLOB object
+    ISC_QUAD save_to_db();
+    void assign(ISC_QUAD const & bid);
+
+private:
+    void open();
+    long getBLOBInfo();
+    void load();
+    void writeBuffer(std::size_t offset, void const * buf,
+        std::size_t toWrite);
+    void closeBlob();
 
     firebird_session_backend &session_;
-
-    virtual void save();
-    virtual void assign(ISC_QUAD const & bid)
-    {
-        cleanUp();
-
-        bid_ = bid;
-        from_db_ = true;
-    }
-
-    // BLOB id from in database
-    ISC_QUAD bid_;
-
+    ISC_QUAD blob_id_;
     // BLOB id was fetched from database (true)
     // or this is new BLOB
     bool from_db_;
-
-    // BLOB handle
-    isc_blob_handle bhp_;
-
-protected:
-
-    virtual void open();
-    virtual long getBLOBInfo();
-    virtual void load();
-    virtual void writeBuffer(std::size_t offset, char const * buf,
-        std::size_t toWrite);
-    virtual void cleanUp();
-
+    isc_blob_handle blob_handle_;
     // buffer for BLOB data
-    std::vector<char> data_;
-
+    std::vector<std::uint8_t> data_;
     bool loaded_;
     long max_seg_size_;
 };
 
-struct firebird_session_backend : details::session_backend
+struct SOCI_FIREBIRD_DECL firebird_session_backend : details::session_backend
 {
     firebird_session_backend(connection_parameters const & parameters);
 
-    ~firebird_session_backend() SOCI_OVERRIDE;
+    ~firebird_session_backend() override;
 
-    bool is_connected() SOCI_OVERRIDE;
+    bool is_connected() override;
 
-    void begin() SOCI_OVERRIDE;
-    void commit() SOCI_OVERRIDE;
-    void rollback() SOCI_OVERRIDE;
+    void begin() override;
+    void commit() override;
+    void rollback() override;
 
     bool get_next_sequence_value(session & s,
-        std::string const & sequence, long long & value) SOCI_OVERRIDE;
+        std::string const & sequence, long long & value) override;
 
-    std::string get_dummy_from_table() const SOCI_OVERRIDE { return "rdb$database"; }
+    std::string get_dummy_from_table() const override { return "rdb$database"; }
 
-    std::string get_backend_name() const SOCI_OVERRIDE { return "firebird"; }
+    std::string get_backend_name() const override { return "firebird"; }
 
     void cleanUp();
 
-    firebird_statement_backend * make_statement_backend() SOCI_OVERRIDE;
-    details::rowid_backend* make_rowid_backend() SOCI_OVERRIDE;
-    firebird_blob_backend * make_blob_backend() SOCI_OVERRIDE;
+    firebird_statement_backend * make_statement_backend() override;
+    details::rowid_backend* make_rowid_backend() override;
+    firebird_blob_backend * make_blob_backend() override;
 
     bool get_option_decimals_as_strings() { return decimals_as_strings_; }
 
@@ -344,7 +345,7 @@ struct firebird_backend_factory : backend_factory
 {
     firebird_backend_factory() {}
     firebird_session_backend * make_session(
-        connection_parameters const & parameters) const SOCI_OVERRIDE;
+        connection_parameters const & parameters) const override;
 };
 
 extern SOCI_FIREBIRD_DECL firebird_backend_factory const firebird;
