@@ -3,7 +3,7 @@
 // MySQL backend copyright (C) 2006 Pawel Aleksander Fedorynski
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #ifndef SOCI_MYSQL_H_INCLUDED
@@ -18,11 +18,12 @@
 #endif
 
 #include <soci/soci-backend.h>
-#ifdef _WIN32
-#include <winsock.h> // SOCKET
-#endif // _WIN32
+#include <soci/trivial-blob-backend.h>
+
 #include <mysql.h> // MySQL Client
 #include <errmsg.h> // MySQL Error codes
+
+
 #include <vector>
 
 
@@ -32,21 +33,16 @@ namespace soci
 class SOCI_MYSQL_DECL mysql_soci_error : public soci_error
 {
 public:
-    mysql_soci_error(std::string const & msg, int errNum)
-        : soci_error(msg), err_num_(errNum), cat_(unknown) {
-            if(errNum == CR_CONNECTION_ERROR ||
-               errNum == CR_CONN_HOST_ERROR ||
-               errNum == CR_SERVER_GONE_ERROR ||
-               errNum == CR_SERVER_LOST ||
-               errNum == 1927) { // Lost connection to backend server
-                cat_ = connection_error;
-            }
-        }
+    mysql_soci_error(std::string const & msg, int errNum);
 
-    error_category get_error_category() const SOCI_OVERRIDE { return cat_; }
+    error_category get_error_category() const override;
 
+    std::string get_backend_name() const override { return "mysql"; }
+    int get_backend_error_code() const override { return err_num_; }
+
+    // This member variable is only public for compatibility, don't use it
+    // directly, call get_backend_error_code() instead.
     unsigned int err_num_;
-    error_category cat_;
 };
 
 struct mysql_statement_backend;
@@ -56,13 +52,13 @@ struct mysql_standard_into_type_backend : details::standard_into_type_backend
         : statement_(st) {}
 
     void define_by_pos(int &position,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
 
-    void pre_fetch() SOCI_OVERRIDE;
+    void pre_fetch() override;
     void post_fetch(bool gotData, bool calledFromFetch,
-        indicator *ind) SOCI_OVERRIDE;
+        indicator *ind) override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     mysql_statement_backend &statement_;
 
@@ -77,15 +73,15 @@ struct mysql_vector_into_type_backend : details::vector_into_type_backend
         : statement_(st) {}
 
     void define_by_pos(int &position,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
 
-    void pre_fetch() SOCI_OVERRIDE;
-    void post_fetch(bool gotData, indicator *ind) SOCI_OVERRIDE;
+    void pre_fetch() override;
+    void post_fetch(bool gotData, indicator *ind) override;
 
-    void resize(std::size_t sz) SOCI_OVERRIDE;
-    std::size_t size() SOCI_OVERRIDE;
+    void resize(std::size_t sz) override;
+    std::size_t size() const override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     mysql_statement_backend &statement_;
 
@@ -100,14 +96,14 @@ struct mysql_standard_use_type_backend : details::standard_use_type_backend
         : statement_(st), position_(0), buf_(NULL) {}
 
     void bind_by_pos(int &position,
-        void *data, details::exchange_type type, bool readOnly) SOCI_OVERRIDE;
+        void *data, details::exchange_type type, bool readOnly) override;
     void bind_by_name(std::string const &name,
-        void *data, details::exchange_type type, bool readOnly) SOCI_OVERRIDE;
+        void *data, details::exchange_type type, bool readOnly) override;
 
-    void pre_use(indicator const *ind) SOCI_OVERRIDE;
-    void post_use(bool gotData, indicator *ind) SOCI_OVERRIDE;
+    void pre_use(indicator const *ind) override;
+    void post_use(bool gotData, indicator *ind) override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     mysql_statement_backend &statement_;
 
@@ -124,15 +120,15 @@ struct mysql_vector_use_type_backend : details::vector_use_type_backend
         : statement_(st), position_(0) {}
 
     void bind_by_pos(int &position,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
     void bind_by_name(std::string const &name,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
 
-    void pre_use(indicator const *ind) SOCI_OVERRIDE;
+    void pre_use(indicator const *ind) override;
 
-    std::size_t size() SOCI_OVERRIDE;
+    std::size_t size() const override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     mysql_statement_backend &statement_;
 
@@ -144,32 +140,34 @@ struct mysql_vector_use_type_backend : details::vector_use_type_backend
 };
 
 struct mysql_session_backend;
-struct mysql_statement_backend : details::statement_backend
+struct SOCI_MYSQL_DECL mysql_statement_backend : details::statement_backend
 {
     mysql_statement_backend(mysql_session_backend &session);
 
-    void alloc() SOCI_OVERRIDE;
-    void clean_up() SOCI_OVERRIDE;
+    void alloc() override;
+    void clean_up() override;
     void prepare(std::string const &query,
-        details::statement_type eType) SOCI_OVERRIDE;
+        details::statement_type eType) override;
 
-    exec_fetch_result execute(int number) SOCI_OVERRIDE;
-    exec_fetch_result fetch(int number) SOCI_OVERRIDE;
+    exec_fetch_result execute(int number) override;
+    exec_fetch_result fetch(int number) override;
 
-    long long get_affected_rows() SOCI_OVERRIDE;
-    int get_number_of_rows() SOCI_OVERRIDE;
-    std::string get_parameter_name(int index) const SOCI_OVERRIDE;
+    long long get_affected_rows() override;
+    int get_number_of_rows() override;
+    std::string get_parameter_name(int index) const override;
 
-    std::string rewrite_for_procedure_call(std::string const &query) SOCI_OVERRIDE;
+    std::string rewrite_for_procedure_call(std::string const &query) override;
 
-    int prepare_for_describe() SOCI_OVERRIDE;
-    void describe_column(int colNum, data_type &dtype,
-        std::string &columnName) SOCI_OVERRIDE;
+    int prepare_for_describe() override;
+    void describe_column(int colNum,
+        db_type &dbtype,
+        std::string &columnName) override;
+    data_type to_data_type(db_type dbt) const override;
 
-    mysql_standard_into_type_backend * make_into_type_backend() SOCI_OVERRIDE;
-    mysql_standard_use_type_backend * make_use_type_backend() SOCI_OVERRIDE;
-    mysql_vector_into_type_backend * make_vector_into_type_backend() SOCI_OVERRIDE;
-    mysql_vector_use_type_backend * make_vector_use_type_backend() SOCI_OVERRIDE;
+    mysql_standard_into_type_backend * make_into_type_backend() override;
+    mysql_standard_use_type_backend * make_use_type_backend() override;
+    mysql_vector_into_type_backend * make_vector_into_type_backend() override;
+    mysql_vector_use_type_backend * make_vector_use_type_backend() override;
 
     mysql_session_backend &session_;
 
@@ -191,15 +189,14 @@ struct mysql_statement_backend : details::statement_backend
     bool justDescribed_; // to optimize row description with immediately
                          // following actual statement execution
 
+    // Set to true if the last column passed to describe_column() was a
+    // MEDIUMINT UNSIGNED one, see to_data_type().
+    bool lastDescribedUnsignedMediumInt_ = false;
+
     // Prefetch the row offsets in order to use mysql_row_seek() for
     // random access to rows, since mysql_data_seek() is expensive.
     std::vector<MYSQL_ROW_OFFSET> resultRowOffsets_;
 
-    bool hasIntoElements_;
-    bool hasVectorIntoElements_;
-    bool hasUseElements_;
-    bool hasVectorUseElements_;
-
     // the following maps are used for finding data buffers according to
     // use elements specified by the user
 
@@ -210,59 +207,73 @@ struct mysql_statement_backend : details::statement_backend
     UseByNameBuffersMap useByNameBuffers_;
 };
 
-struct mysql_rowid_backend : details::rowid_backend
+struct SOCI_MYSQL_DECL mysql_rowid_backend : details::rowid_backend
 {
     mysql_rowid_backend(mysql_session_backend &session);
 
-    ~mysql_rowid_backend() SOCI_OVERRIDE;
+    ~mysql_rowid_backend() override;
 };
 
-struct mysql_blob_backend : details::blob_backend
+class SOCI_MYSQL_DECL mysql_blob_backend : public details::trivial_blob_backend
 {
+public:
     mysql_blob_backend(mysql_session_backend &session);
 
-    ~mysql_blob_backend() SOCI_OVERRIDE;
+    ~mysql_blob_backend() override;
 
-    std::size_t get_len() SOCI_OVERRIDE;
-    std::size_t read(std::size_t offset, char *buf,
-        std::size_t toRead) SOCI_OVERRIDE;
-    std::size_t write(std::size_t offset, char const *buf,
-        std::size_t toWrite) SOCI_OVERRIDE;
-    std::size_t append(char const *buf, std::size_t toWrite) SOCI_OVERRIDE;
-    void trim(std::size_t newLen) SOCI_OVERRIDE;
+    std::size_t hex_str_size() const;
+    void write_hex_str(char *buf, std::size_t size) const;
+    std::string as_hex_str() const;
 
-    mysql_session_backend &session_;
+    void load_from_hex_str(const char* str, std::size_t length);
 };
 
-struct mysql_session_backend : details::session_backend
+struct SOCI_MYSQL_DECL mysql_session_backend : details::session_backend
 {
     mysql_session_backend(connection_parameters const & parameters);
 
-    ~mysql_session_backend() SOCI_OVERRIDE;
+    ~mysql_session_backend() override;
 
-    bool is_connected() SOCI_OVERRIDE;
+    bool is_connected() override;
 
-    void begin() SOCI_OVERRIDE;
-    void commit() SOCI_OVERRIDE;
-    void rollback() SOCI_OVERRIDE;
+    void begin() override;
+    void commit() override;
+    void rollback() override;
 
-    bool get_last_insert_id(session&, std::string const&, long long&) SOCI_OVERRIDE;
+    bool get_last_insert_id(session&, std::string const&, long long&) override;
 
     // Note that MySQL supports both "SELECT 2+2" and "SELECT 2+2 FROM DUAL"
     // syntaxes, but there doesn't seem to be any reason to use the longer one.
-    std::string get_dummy_from_table() const SOCI_OVERRIDE { return std::string(); }
+    std::string get_dummy_from_table() const override { return std::string(); }
 
-    std::string get_backend_name() const SOCI_OVERRIDE { return "mysql"; }
+    std::string get_backend_name() const override { return "mysql"; }
 
     void clean_up();
 
-    mysql_statement_backend * make_statement_backend() SOCI_OVERRIDE;
-    mysql_rowid_backend * make_rowid_backend() SOCI_OVERRIDE;
-    mysql_blob_backend * make_blob_backend() SOCI_OVERRIDE;
+    mysql_statement_backend * make_statement_backend() override;
+    mysql_rowid_backend * make_rowid_backend() override;
+    mysql_blob_backend * make_blob_backend() override;
 
-    std::string get_table_names_query() const SOCI_OVERRIDE
+    std::string get_table_names_query() const override
     {
-        return "SELECT table_name AS 'TABLE_NAME' FROM information_schema.tables WHERE table_schema = DATABASE()";
+        return R"delim(SELECT LOWER(table_name) AS 'TABLE_NAME' FROM information_schema.tables WHERE table_schema = DATABASE())delim";
+    }
+
+    std::string get_column_descriptions_query() const override
+    {
+        return R"delim(SELECT column_name as "COLUMN_NAME",
+            data_type as "DATA_TYPE",
+            character_maximum_length as "CHARACTER_MAXIMUM_LENGTH",
+            numeric_precision as "NUMERIC_PRECISION",
+            numeric_scale as "NUMERIC_SCALE",
+            is_nullable as "IS_NULLABLE"
+            from information_schema.columns
+            where
+            case
+            when :s is not NULL THEN table_schema = :s
+            else table_schema = DATABASE()
+            end
+            and UPPER(table_name) = UPPER(:t))delim";
     }
 
     MYSQL *conn_;
@@ -273,7 +284,7 @@ struct mysql_backend_factory : backend_factory
 {
     mysql_backend_factory() {}
     mysql_session_backend * make_session(
-        connection_parameters const & parameters) const SOCI_OVERRIDE;
+        connection_parameters const & parameters) const override;
 };
 
 extern SOCI_MYSQL_DECL mysql_backend_factory const mysql;
