@@ -2,28 +2,20 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, David Courtney
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #ifndef SOCI_ODBC_H_INCLUDED
 #define SOCI_ODBC_H_INCLUDED
 
-#ifdef _WIN32
-# ifdef SOCI_DLL
-#  ifdef SOCI_ODBC_SOURCE
-#   define SOCI_ODBC_DECL __declspec(dllexport)
-#  else
-#   define SOCI_ODBC_DECL __declspec(dllimport)
-#  endif // SOCI_ODBC_SOURCE
-# endif // SOCI_DLL
-#endif // _WIN32
-//
-// If SOCI_ODBC_DECL isn't defined yet define it now
-#ifndef SOCI_ODBC_DECL
-# define SOCI_ODBC_DECL
+#include <soci/soci-platform.h>
+
+#ifdef SOCI_ODBC_SOURCE
+# define SOCI_ODBC_DECL SOCI_DECL_EXPORT
+#else
+# define SOCI_ODBC_DECL SOCI_DECL_IMPORT
 #endif
 
-#include "soci/soci-platform.h"
 #include <vector>
 #include <soci/soci-backend.h>
 #include <sstream>
@@ -51,6 +43,16 @@ namespace details
     {
       return reinterpret_cast<SQLCHAR*>(const_cast<char*>(s.c_str()));
     }
+
+    inline SQLWCHAR* sqlchar_cast(std::wstring const& s)
+    {
+      return reinterpret_cast<SQLWCHAR*>(const_cast<wchar_t*>(s.c_str()));
+    }
+
+    inline SQLWCHAR* sqlchar_cast(std::u16string const& s)
+    {
+      return reinterpret_cast<SQLWCHAR*>(const_cast<char16_t*>(s.c_str()));
+    }
 }
 
 // Option allowing to specify the "driver completion" parameter of
@@ -86,6 +88,8 @@ protected:
     inline SQLLEN get_sqllen_from_value(const SQLLEN val) const;
     inline void set_sqllen_from_value(SQLLEN &target, const SQLLEN val) const;
 
+    inline bool supports_negative_tinyint() const;
+    inline bool can_convert_to_unsigned_sql_type() const;
 
     odbc_statement_backend &statement_;
 private:
@@ -100,13 +104,13 @@ struct odbc_standard_into_type_backend : details::standard_into_type_backend,
     {}
 
     void define_by_pos(int &position,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
 
-    void pre_fetch() SOCI_OVERRIDE;
+    void pre_fetch() override;
     void post_fetch(bool gotData, bool calledFromFetch,
-        indicator *ind) SOCI_OVERRIDE;
+        indicator *ind) override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     char *buf_;        // generic buffer
     void *data_;
@@ -122,35 +126,41 @@ struct odbc_vector_into_type_backend : details::vector_into_type_backend,
                                        private odbc_standard_type_backend_base
 {
     odbc_vector_into_type_backend(odbc_statement_backend &st)
-        : odbc_standard_type_backend_base(st), indHolders_(NULL),
-          data_(NULL), buf_(NULL) {}
+        : odbc_standard_type_backend_base(st),
+          data_(NULL), buf_(NULL), position_(0) {}
 
     void define_by_pos(int &position,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
 
-    void pre_fetch() SOCI_OVERRIDE;
-    void post_fetch(bool gotData, indicator *ind) SOCI_OVERRIDE;
+    void pre_fetch() override;
+    void post_fetch(bool gotData, indicator *ind) override;
 
-    void resize(std::size_t sz) SOCI_OVERRIDE;
-    std::size_t size() SOCI_OVERRIDE;
+    void resize(std::size_t sz) override;
+    std::size_t size() const override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
-    // helper function for preparing indicators
-    // (as part of the define_by_pos)
-    void prepare_indicators(std::size_t size);
+    // Normally data retrieved from the database is handled in post_fetch(),
+    // however we may need to call SQLFetch() multiple times, so we call this
+    // function instead after each call to it to retrieve the given range of
+    // rows.
+    void do_post_fetch_rows(std::size_t beginRow, std::size_t endRow);
 
     // IBM DB2 driver is not compliant to ODBC spec for indicators in 64bit
     // SQLLEN is still defined 32bit (int) but spec requires 64bit (long)
     inline SQLLEN get_sqllen_from_vector_at(std::size_t idx) const;
 
-    SQLLEN *indHolders_;
+    // Rebind the single vector value at the given index to the first row.
+    // Used when vector values are fetched by single row.
+    void rebind_row(std::size_t rowInd);
+
     std::vector<SQLLEN> indHolderVec_;
     void *data_;
     char *buf_;              // generic buffer
     details::exchange_type type_;
     std::size_t colSize_;    // size of the string column (used for strings)
     SQLSMALLINT odbcType_;
+    int position_;
 };
 
 struct odbc_standard_use_type_backend : details::standard_use_type_backend,
@@ -161,14 +171,14 @@ struct odbc_standard_use_type_backend : details::standard_use_type_backend,
           position_(-1), data_(0), buf_(0), indHolder_(0) {}
 
     void bind_by_pos(int &position,
-        void *data, details::exchange_type type, bool readOnly) SOCI_OVERRIDE;
+        void *data, details::exchange_type type, bool readOnly) override;
     void bind_by_name(std::string const &name,
-        void *data, details::exchange_type type, bool readOnly) SOCI_OVERRIDE;
+        void *data, details::exchange_type type, bool readOnly) override;
 
-    void pre_use(indicator const *ind) SOCI_OVERRIDE;
-    void post_use(bool gotData, indicator *ind) SOCI_OVERRIDE;
+    void pre_use(indicator const *ind) override;
+    void post_use(bool gotData, indicator *ind) override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     // Return the pointer to the buffer containing data to be used by ODBC.
     // This can be either data_ itself or buf_, that is allocated by this
@@ -192,43 +202,45 @@ private:
                           SQLSMALLINT& sqlType,
                           SQLSMALLINT& cType);
 
+    void copy_from_string(const std::wstring& s,
+                          SQLLEN& size,
+                          SQLSMALLINT& sqlType,
+                          SQLSMALLINT& cType);
 };
 
 struct odbc_vector_use_type_backend : details::vector_use_type_backend,
                                       private odbc_standard_type_backend_base
 {
     odbc_vector_use_type_backend(odbc_statement_backend &st)
-        : odbc_standard_type_backend_base(st), indHolders_(NULL),
+        : odbc_standard_type_backend_base(st),
           data_(NULL), buf_(NULL) {}
 
     // helper function for preparing indicators
     // (as part of the define_by_pos)
     void prepare_indicators(std::size_t size);
 
-    // common part for bind_by_pos and bind_by_name
-    void prepare_for_bind(void *&data, SQLUINTEGER &size, SQLSMALLINT &sqlType, SQLSMALLINT &cType);
-    void bind_helper(int &position,
-        void *data, details::exchange_type type);
+    // helper of pre_use(), return the pointer to the data to be used by ODBC.
+    void* prepare_for_bind(SQLUINTEGER &size, SQLSMALLINT &sqlType, SQLSMALLINT &cType);
 
     void bind_by_pos(int &position,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
     void bind_by_name(std::string const &name,
-        void *data, details::exchange_type type) SOCI_OVERRIDE;
+        void *data, details::exchange_type type) override;
 
-    void pre_use(indicator const *ind) SOCI_OVERRIDE;
+    void pre_use(indicator const *ind) override;
 
-    std::size_t size() SOCI_OVERRIDE;
+    std::size_t size() const override;
 
-    void clean_up() SOCI_OVERRIDE;
+    void clean_up() override;
 
     // IBM DB2 driver is not compliant to ODBC spec for indicators in 64bit
     // SQLLEN is still defined 32bit (int) but spec requires 64bit (long)
     inline void set_sqllen_from_vector_at(const std::size_t idx, const SQLLEN val);
 
-    SQLLEN *indHolders_;
     std::vector<SQLLEN> indHolderVec_;
     void *data_;
     details::exchange_type type_;
+    int position_;
     char *buf_;              // generic buffer
     std::size_t colSize_;    // size of the string column (used for strings)
     // used for strings only
@@ -236,40 +248,43 @@ struct odbc_vector_use_type_backend : details::vector_use_type_backend,
 };
 
 struct odbc_session_backend;
-struct odbc_statement_backend : details::statement_backend
+struct SOCI_ODBC_DECL odbc_statement_backend : details::statement_backend
 {
     odbc_statement_backend(odbc_session_backend &session);
 
-    void alloc() SOCI_OVERRIDE;
-    void clean_up() SOCI_OVERRIDE;
+    void alloc() override;
+    void clean_up() override;
     void prepare(std::string const &query,
-        details::statement_type eType) SOCI_OVERRIDE;
+        details::statement_type eType) override;
 
-    exec_fetch_result execute(int number) SOCI_OVERRIDE;
-    exec_fetch_result fetch(int number) SOCI_OVERRIDE;
+    exec_fetch_result execute(int number) override;
+    exec_fetch_result fetch(int number) override;
 
-    long long get_affected_rows() SOCI_OVERRIDE;
-    int get_number_of_rows() SOCI_OVERRIDE;
-    std::string get_parameter_name(int index) const SOCI_OVERRIDE;
+    long long get_affected_rows() override;
+    int get_number_of_rows() override;
+    std::string get_parameter_name(int index) const override;
+    int get_row_to_dump() const override { return error_row_; }
 
-    std::string rewrite_for_procedure_call(std::string const &query) SOCI_OVERRIDE;
+    std::string rewrite_for_procedure_call(std::string const &query) override;
 
-    int prepare_for_describe() SOCI_OVERRIDE;
-    void describe_column(int colNum, data_type &dtype,
-        std::string &columnName) SOCI_OVERRIDE;
+    int prepare_for_describe() override;
+    void describe_column(int colNum,
+        db_type &dbtype,
+        std::string &columnName) override;
+    data_type to_data_type(db_type dbt) const override;
 
     // helper for defining into vector<string>
     std::size_t column_size(int position);
 
-    odbc_standard_into_type_backend * make_into_type_backend() SOCI_OVERRIDE;
-    odbc_standard_use_type_backend * make_use_type_backend() SOCI_OVERRIDE;
-    odbc_vector_into_type_backend * make_vector_into_type_backend() SOCI_OVERRIDE;
-    odbc_vector_use_type_backend * make_vector_use_type_backend() SOCI_OVERRIDE;
+    odbc_standard_into_type_backend * make_into_type_backend() override;
+    odbc_standard_use_type_backend * make_use_type_backend() override;
+    odbc_vector_into_type_backend * make_vector_into_type_backend() override;
+    odbc_vector_use_type_backend * make_vector_use_type_backend() override;
 
     odbc_session_backend &session_;
     SQLHSTMT hstmt_;
     SQLULEN numRowsFetched_;
-    bool hasVectorUseElements_;
+    bool fetchVectorByRows_;
     bool boundByName_;
     bool boundByPos_;
 
@@ -278,61 +293,70 @@ struct odbc_statement_backend : details::statement_backend
     std::string query_;
     std::vector<std::string> names_; // list of names for named binds
 
+    // This vector, containing non-owning non-null pointers, can be empty if
+    // we're not using any vector "intos".
+    std::vector<odbc_vector_into_type_backend*> intos_;
+
+private:
+    // fetch() helper wrapping SQLFetch() call for the given range of rows.
+    exec_fetch_result do_fetch(int beginRow, int endRow);
+
+    // First row with the error for bulk operations or -1.
+    int error_row_ = -1;
 };
 
-struct odbc_rowid_backend : details::rowid_backend
+struct SOCI_ODBC_DECL odbc_rowid_backend : details::rowid_backend
 {
     odbc_rowid_backend(odbc_session_backend &session);
 
-    ~odbc_rowid_backend() SOCI_OVERRIDE;
+    ~odbc_rowid_backend() override;
 };
 
-struct odbc_blob_backend : details::blob_backend
+struct SOCI_ODBC_DECL odbc_blob_backend : details::blob_backend
 {
     odbc_blob_backend(odbc_session_backend &session);
 
-    ~odbc_blob_backend() SOCI_OVERRIDE;
+    ~odbc_blob_backend() override;
 
-    std::size_t get_len() SOCI_OVERRIDE;
-    std::size_t read(std::size_t offset, char *buf,
-        std::size_t toRead) SOCI_OVERRIDE;
-    std::size_t write(std::size_t offset, char const *buf,
-        std::size_t toWrite) SOCI_OVERRIDE;
-    std::size_t append(char const *buf, std::size_t toWrite) SOCI_OVERRIDE;
-    void trim(std::size_t newLen) SOCI_OVERRIDE;
+    std::size_t get_len() override;
+    std::size_t read_from_start(void *buf, std::size_t toRead, std::size_t offset = 0) override;
+    std::size_t write_from_start(const void *buf, std::size_t toWrite, std::size_t offset = 0) override;
+    std::size_t append(const void *buf, std::size_t toWrite) override;
+    void trim(std::size_t newLen) override;
+    details::session_backend &get_session_backend() override;
 
     odbc_session_backend &session_;
 };
 
-struct odbc_session_backend : details::session_backend
+struct SOCI_ODBC_DECL odbc_session_backend : details::session_backend
 {
     odbc_session_backend(connection_parameters const & parameters);
 
-    ~odbc_session_backend() SOCI_OVERRIDE;
+    ~odbc_session_backend() override;
 
-    bool is_connected() SOCI_OVERRIDE;
+    bool is_connected() override;
 
-    void begin() SOCI_OVERRIDE;
-    void commit() SOCI_OVERRIDE;
-    void rollback() SOCI_OVERRIDE;
+    void begin() override;
+    void commit() override;
+    void rollback() override;
 
     bool get_next_sequence_value(session & s,
-        std::string const & sequence, long long & value) SOCI_OVERRIDE;
+        std::string const & sequence, long long & value) override;
     bool get_last_insert_id(session & s,
-        std::string const & table, long long & value) SOCI_OVERRIDE;
+        std::string const & table, long long & value) override;
 
-    std::string get_dummy_from_table() const SOCI_OVERRIDE;
+    std::string get_dummy_from_table() const override;
 
-    std::string get_backend_name() const SOCI_OVERRIDE { return "odbc"; }
+    std::string get_backend_name() const override { return "odbc"; }
 
     void configure_connection();
     void reset_transaction();
 
     void clean_up();
 
-    odbc_statement_backend * make_statement_backend() SOCI_OVERRIDE;
-    odbc_rowid_backend * make_rowid_backend() SOCI_OVERRIDE;
-    odbc_blob_backend * make_blob_backend() SOCI_OVERRIDE;
+    odbc_statement_backend * make_statement_backend() override;
+    odbc_rowid_backend * make_rowid_backend() override;
+    odbc_blob_backend * make_blob_backend() override;
 
     enum database_product
     {
@@ -369,31 +393,13 @@ class SOCI_ODBC_DECL odbc_soci_error : public soci_error
     SQLINTEGER sqlcode_;
 
 public:
-    odbc_soci_error(SQLSMALLINT htype,
-                  SQLHANDLE hndl,
-                  std::string const & msg)
-        : soci_error(interpret_odbc_error(htype, hndl, msg))
-    {
-    }
+    odbc_soci_error(SQLSMALLINT htype, SQLHANDLE hndl, std::string const & msg);
 
-    error_category get_error_category() const SOCI_OVERRIDE
-    {
-        const char* const s = reinterpret_cast<const char*>(sqlstate_);
+    error_category get_error_category() const override;
 
-        if ((s[0] == '0' && s[1] == '8') ||
-            strcmp(s, "HYT01") == 0)
-            return connection_error;
-
-        if (strcmp(s, "23000") == 0 ||
-            strcmp(s, "40002") == 0 ||
-            strcmp(s, "44000") == 0)
-            return constraint_violation;
-
-        if (strcmp(s, "HY014") == 0)
-            return system_error;
-
-        return unknown;
-    }
+    std::string get_backend_name() const override { return "odbc"; }
+    int get_backend_error_code() const override { return sqlcode_; }
+    std::string get_sqlstate() const override { return (char const*)sqlstate_; }
 
     SQLCHAR const * odbc_error_code() const
     {
@@ -407,57 +413,13 @@ public:
     {
         return message_;
     }
+
 private:
-    std::string interpret_odbc_error(SQLSMALLINT htype, SQLHANDLE hndl, std::string const& msg)
-    {
-        const char* socierror = NULL;
-
-        SQLSMALLINT length, i = 1;
-        switch ( SQLGetDiagRecA(htype, hndl, i, sqlstate_, &sqlcode_,
-                               message_, SQL_MAX_MESSAGE_LENGTH + 1,
-                               &length) )
-        {
-          case SQL_SUCCESS:
-            // The error message was successfully retrieved.
-            break;
-
-          case SQL_INVALID_HANDLE:
-            socierror = "[SOCI]: Invalid handle.";
-            break;
-
-          case SQL_ERROR:
-            socierror = "[SOCI]: SQLGetDiagRec() error.";
-            break;
-
-          case SQL_SUCCESS_WITH_INFO:
-            socierror = "[SOCI]: Error message too long.";
-            break;
-
-          case SQL_NO_DATA:
-            socierror = "[SOCI]: No error.";
-            break;
-
-          default:
-            socierror = "[SOCI]: Unexpected SQLGetDiagRec() return value.";
-            break;
-        }
-
-        if (socierror)
-        {
-            // Use our own error message if we failed to retrieve the ODBC one.
-            strcpy(reinterpret_cast<char*>(message_), socierror);
-
-            // Use "General warning" SQLSTATE code.
-            strcpy(reinterpret_cast<char*>(sqlstate_), "01000");
-
-            sqlcode_ = 0;
-        }
-
-        std::ostringstream ss;
-        ss << "Error " << msg << ": " << message_ << " (SQL state " << sqlstate_ << ")";
-
-        return ss.str();
-    }
+    // Initialize the member variables and return the full error message
+    // corresponding to the last error on the given ODBC object (connection or
+    // statement handle depending on htype value).
+    std::string
+    interpret_odbc_error(SQLSMALLINT htype, SQLHANDLE hndl, std::string const& msg);
 };
 
 inline bool is_odbc_error(SQLRETURN rc)
@@ -476,7 +438,7 @@ inline bool odbc_standard_type_backend_base::use_string_for_bigint() const
 {
     // Oracle ODBC driver doesn't support SQL_C_[SU]BIGINT data types
     // (see appendix G.1 of Oracle Database Administrator's reference at
-    // http://docs.oracle.com/cd/B19306_01/server.102/b15658/app_odbc.htm),
+    // https://docs.oracle.com/cd/B19306_01/server.102/b15658/app_odbc.htm),
     // so we need a special workaround for this case and we represent 64
     // bit integers as strings and rely on ODBC driver for transforming
     // them to SQL_NUMERIC.
@@ -543,6 +505,25 @@ inline void odbc_standard_type_backend_base::set_sqllen_from_value(SQLLEN &targe
     }
 }
 
+inline bool odbc_standard_type_backend_base::supports_negative_tinyint() const
+{
+    // MSSQL ODBC driver only supports a range of [0..255] for tinyint.
+    return statement_.session_.get_database_product()
+            != odbc_session_backend::prod_mssql;
+}
+
+inline bool odbc_standard_type_backend_base::can_convert_to_unsigned_sql_type() const
+{
+    // MSSQL ODBC driver seemingly can't handle the conversion of unsigned C
+    // types to their respective unsigned SQL type because they are out of
+    // range for their supported signed types. This results in the error
+    // "Numeric value out of range (SQL state 22003)".
+    // The only place it works is with tinyint values as their range is
+    // [0..255], i.e. they have enough space for unsigned values anyway.
+    return statement_.session_.get_database_product()
+            != odbc_session_backend::prod_mssql;
+}
+
 inline SQLLEN odbc_vector_into_type_backend::get_sqllen_from_vector_at(std::size_t idx) const
 {
     if (requires_noncompliant_32bit_sqllen())
@@ -580,7 +561,7 @@ struct odbc_backend_factory : backend_factory
 {
     odbc_backend_factory() {}
     odbc_session_backend * make_session(
-        connection_parameters const & parameters) const SOCI_OVERRIDE;
+        connection_parameters const & parameters) const override;
 };
 
 extern SOCI_ODBC_DECL odbc_backend_factory const odbc;
