@@ -2,7 +2,7 @@
 // Copyright (C) 2006-2008 Mateusz Loskot
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #ifndef SOCI_PLATFORM_H_INCLUDED
@@ -21,7 +21,7 @@
 #include <ctime>
 #include <memory>
 
-#include "soci/soci-config.h" // for SOCI_HAVE_CXX11
+#include "soci/soci-config.h"
 
 #if defined(_MSC_VER)
 #define LL_FMT_FLAGS "I64"
@@ -105,42 +105,55 @@ namespace std {
 // mode, we just need to check for the minimal compiler version supporting them
 // (see https://msdn.microsoft.com/en-us/library/hh567368.aspx).
 
-#if defined(SOCI_HAVE_CXX11) || (defined(_MSC_VER) && _MSC_VER >= 1800)
-# define SOCI_OVERRIDE override
+#ifdef _MSC_VER
+    #if _MSC_VER < 1900
+        #error This version of SOCI requires MSVS 2015 or later.
+    #endif
+    #if _MSVC_LANG >= 201703L
+        #define SOCI_HAVE_CXX17
+    #endif
 #else
-# define SOCI_OVERRIDE
+    #if __cplusplus < 201402L
+        #error This version of SOCI requires C++14.
+    #endif
+    #if __cplusplus >= 201703L
+        #define SOCI_HAVE_CXX17
+    #endif
 #endif
 
-namespace soci
-{
-
-namespace cxx_details
-{
-
-#if defined(SOCI_HAVE_CXX11) || (defined(_MSC_VER) && _MSC_VER >= 1800)
-    template <typename T>
-    using auto_ptr = std::unique_ptr<T>;
-#else // std::unique_ptr<> not available
-    using std::auto_ptr;
+// Define SOCI_ALLOW_DEPRECATED_BEGIN and SOCI_ALLOW_DEPRECATED_END
+// Ref.: https://www.fluentcpp.com/2019/08/30/how-to-disable-a-warning-in-cpp/
+#if defined(__GNUC__) || defined(__clang__)
+# define SOCI_ALLOW_DEPRECATED_BEGIN \
+    _Pragma("GCC diagnostic push") \
+    _Pragma("GCC diagnostic ignored \"-Wdeprecated\"") \
+    _Pragma("GCC diagnostic ignored \"-Wdeprecated-declarations\"")
+# define SOCI_ALLOW_DEPRECATED_END \
+    _Pragma("GCC diagnostic pop")
+#elif defined(_MSC_VER)
+# define SOCI_ALLOW_DEPRECATED_BEGIN \
+    __pragma(warning(push)) \
+    __pragma(warning(disable: 4973 )) \
+    __pragma(warning(disable: 4974 )) \
+    __pragma(warning(disable: 4995 )) \
+    __pragma(warning(disable: 4996 ))
+# define SOCI_ALLOW_DEPRECATED_END \
+    __pragma(warning(pop))
+# define SOCI_DONT_WARN(statement) statement
+#else
+# pragma message("WARNING: SOCI_ALLOW_DEPRECATED_* not available for this compilet")
+# define SOCI_ALLOW_DEPRECATED_BEGIN
+# define SOCI_ALLOW_DEPRECATED_END
 #endif
 
-} // namespace cxx_details
-
-} // namespace soci
-
-#if defined(SOCI_HAVE_CXX11) || (defined(_MSC_VER) && _MSC_VER >= 1800)
-    #define SOCI_NOT_ASSIGNABLE(classname) \
-        classname& operator=(const classname&) = delete;
-    #define SOCI_NOT_COPYABLE(classname) \
-        classname(const classname&) = delete; \
-        SOCI_NOT_ASSIGNABLE(classname)
-#else // no C++11 deleted members support
-    #define SOCI_NOT_ASSIGNABLE(classname) \
-        classname& operator=(const classname&);
-    #define SOCI_NOT_COPYABLE(classname) \
-        classname(const classname&); \
-        SOCI_NOT_ASSIGNABLE(classname)
-#endif // C++11 deleted members available
+#define SOCI_NOT_ASSIGNABLE(classname) \
+public: \
+    classname(const classname&) = default; \
+private: \
+    classname& operator=(const classname&) = delete;
+#define SOCI_NOT_COPYABLE(classname) \
+    classname(const classname&) = delete; \
+    classname& operator=(const classname&) = delete;
 
 #define SOCI_UNUSED(x) (void)x;
 
@@ -156,24 +169,12 @@ namespace cxx_details
 // Here the problem is that MSVC complains about unreachable code in this case
 // (but only in release builds, where optimizations are enabled), while other
 // compilers complain about missing return statement without it.
+// Note: this macro is no longer used in SOCI's codebase. It is only retained
+// in case downstream code is depending on it.
 #if defined(_MSC_VER) && defined(NDEBUG)
     #define SOCI_DUMMY_RETURN(x)
 #else
     #define SOCI_DUMMY_RETURN(x) return x
 #endif
 
-#if defined(SOCI_HAVE_CXX11) || (defined(_MSC_VER) && _MSC_VER >= 1900)
-    #define SOCI_NOEXCEPT noexcept
-    #define SOCI_NOEXCEPT_FALSE noexcept(false)
-#else
-    #if defined(__cplusplus) && __cplusplus >= 201103L
-        // Otherwise throwing from a dtor not marked with noexcept(false) would
-        // simply result in terminating the program.
-        #error "SOCI must be configured with C++11 support when using C++11"
-    #endif
-
-    #define SOCI_NOEXCEPT throw()
-    #define SOCI_NOEXCEPT_FALSE
-#endif
-
 #endif // SOCI_PLATFORM_H_INCLUDED
