@@ -2,7 +2,7 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_EMPTY_SOURCE
@@ -34,7 +34,7 @@ void empty_vector_into_type_backend::resize(std::size_t /* sz */)
     // ...
 }
 
-std::size_t empty_vector_into_type_backend::size()
+std::size_t empty_vector_into_type_backend::size() const
 {
     // ...
     return 1;
