@@ -2,11 +2,12 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, Rafal Bobrowski
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_FIREBIRD_SOURCE
 #include "soci/firebird/soci-firebird.h"
+#include "soci-ssize.h"
 #include "firebird/error-firebird.h"
 #include <cctype>
 #include <sstream>
@@ -39,7 +40,7 @@ void firebird_statement_backend::prepareSQLDA(XSQLDA ** sqldap, short size)
 
 void firebird_statement_backend::alloc()
 {
-    ISC_STATUS stat[stat_size];
+    ISC_STATUS stat[ISC_STATUS_LENGTH];
 
     if (isc_dsql_allocate_statement(stat, &session_.dbhp_, &stmtp_))
     {
@@ -51,7 +52,7 @@ void firebird_statement_backend::clean_up()
 {
     rowsAffectedBulk_ = -1LL;
 
-    ISC_STATUS stat[stat_size];
+    ISC_STATUS stat[ISC_STATUS_LENGTH];
 
     if (stmtp_ != 0)
     {
@@ -154,7 +155,7 @@ namespace
         char type_item[] = {isc_info_sql_stmt_type};
         char res_buffer[8];
 
-        ISC_STATUS stat[stat_size];
+        ISC_STATUS stat[ISC_STATUS_LENGTH];
 
         if (isc_dsql_sql_info(stat, &stmt, sizeof(type_item),
             type_item, sizeof(res_buffer), res_buffer))
@@ -216,7 +217,7 @@ void firebird_statement_backend::rewriteQuery(
         prepareSQLDA(&sqldap_);
     }
 
-    ISC_STATUS stat[stat_size];
+    ISC_STATUS stat[ISC_STATUS_LENGTH];
     isc_stmt_handle tmpStmtp = 0;
 
     // allocate temporary statement to determine its type
@@ -299,7 +300,7 @@ void firebird_statement_backend::prepare(std::string const & query,
     // firebird's api
     rewriteQuery(query, queryBuffer);
 
-    ISC_STATUS stat[stat_size];
+    ISC_STATUS stat[ISC_STATUS_LENGTH];
 
     // prepare real statement
     if (isc_dsql_prepare(stat, session_.current_transaction(), &stmtp_, 0,
@@ -373,7 +374,7 @@ namespace
 statement_backend::exec_fetch_result
 firebird_statement_backend::execute(int number)
 {
-    ISC_STATUS stat[stat_size];
+    ISC_STATUS stat[ISC_STATUS_LENGTH];
     XSQLDA *t = NULL;
 
     std::size_t usize = uses_.size();
@@ -409,35 +410,36 @@ firebird_statement_backend::execute(int number)
 
     if (useType_ == eVector)
     {
-        long long rowsAffectedBulkTemp = 0;
+        rowsAffectedBulk_ = 0;
 
         // Here we have to explicitly loop to achieve the
         // effect of inserting or updating with vector use elements.
-        std::size_t rows = static_cast<firebird_vector_use_type_backend*>(uses_[0])->size();
-        for (std::size_t row=0; row < rows; ++row)
+        const int rows = ssize(*static_cast<firebird_vector_use_type_backend*>(uses_[0]));
+        for (current_row_ = 0; current_row_ < rows; ++current_row_)
         {
             // first we have to prepare input parameters
             for (std::size_t col=0; col<usize; ++col)
             {
-                static_cast<firebird_vector_use_type_backend*>(uses_[col])->exchangeData(row);
+                static_cast<firebird_vector_use_type_backend*>(uses_[col])->exchangeData(current_row_);
             }
 
             // then execute query
             if (isc_dsql_execute(stat, session_.current_transaction(), &stmtp_, SQL_DIALECT_V6, t))
             {
-                // preserve the number of rows affected so far.
-                rowsAffectedBulk_ = rowsAffectedBulkTemp;
                 throw_iscerror(stat);
             }
             else
             {
-                rowsAffectedBulkTemp += get_affected_rows();
+                // Don't call get_affected_rows() here, because it would return
+                // the current value of rowsAffectedBulk_ itself.
+                rowsAffectedBulk_ += get_last_row_count();
             }
             // soci does not allow bulk insert/update and bulk select operations
             // in same query. So here, we know that into elements are not
             // vectors. So, there is no need to fetch data here.
         }
-        rowsAffectedBulk_ = rowsAffectedBulkTemp;
+
+        current_row_ = -1;
     }
     else
     {
@@ -479,7 +481,7 @@ firebird_statement_backend::fetch(int number)
     if (endOfRowSet_)
         return ef_no_data;
 
-    ISC_STATUS stat[stat_size];
+    ISC_STATUS stat[ISC_STATUS_LENGTH];
 
     for (size_t i = 0; i<static_cast<unsigned int>(sqldap_->sqld); ++i)
     {
@@ -569,6 +571,11 @@ long long firebird_statement_backend::get_affected_rows()
         return rowsAffectedBulk_;
     }
 
+    return get_last_row_count();
+}
+
+long long firebird_statement_backend::get_last_row_count()
+{
     ISC_STATUS_ARRAY stat;
     char type_item[] = { isc_info_sql_records };
     char res_buffer[256];
@@ -658,7 +665,8 @@ int firebird_statement_backend::prepare_for_describe()
 }
 
 void firebird_statement_backend::describe_column(int colNum,
-                                                data_type & type, std::string & columnName)
+                                                db_type & dbtype,
+                                                std::string & columnName)
 {
     XSQLVAR * var = sqldap_->sqlvar+(colNum-1);
 
@@ -668,46 +676,72 @@ void firebird_statement_backend::describe_column(int colNum,
     {
     case SQL_TEXT:
     case SQL_VARYING:
-        type = dt_string;
+        dbtype = db_string;
         break;
     case SQL_TYPE_DATE:
     case SQL_TYPE_TIME:
     case SQL_TIMESTAMP:
-        type = dt_date;
+        dbtype = db_date;
         break;
     case SQL_FLOAT:
     case SQL_DOUBLE:
-        type = dt_double;
+        dbtype = db_double;
         break;
     case SQL_SHORT:
+        if (var->sqlscale < 0)
+        {
+            if (session_.get_option_decimals_as_strings())
+            {
+                dbtype = db_string;
+            }
+            else
+            {
+                dbtype = db_double;
+            }
+        }
+        else
+        {
+            dbtype = db_int16;
+        }
+        break;
     case SQL_LONG:
         if (var->sqlscale < 0)
         {
             if (session_.get_option_decimals_as_strings())
-                type = dt_string;
+            {
+                dbtype = db_string;
+            }
             else
-                type = dt_double;
+            {
+                dbtype = db_double;
+            }
         }
         else
         {
-            type = dt_integer;
+            dbtype = db_int32;
         }
         break;
     case SQL_INT64:
         if (var->sqlscale < 0)
         {
             if (session_.get_option_decimals_as_strings())
-                type = dt_string;
+            {
+                dbtype = db_string;
+            }
             else
-                type = dt_double;
+            {
+                dbtype = db_double;
+            }
         }
         else
         {
-            type = dt_long_long;
+            dbtype = db_int64;
         }
         break;
-        /* case SQL_BLOB:
-        case SQL_ARRAY:*/
+    case SQL_BLOB:
+        dbtype = db_blob;
+        break;
+        /* case SQL_ARRAY:*/
     default:
         std::ostringstream msg;
         msg << "Type of column ["<< colNum << "] \"" << columnName
