@@ -3,17 +3,20 @@
 // MySQL backend copyright (C) 2006 Pawel Aleksander Fedorynski
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
+#include "soci-cstrtoi.h"
 #define SOCI_MYSQL_SOURCE
 #include "soci/mysql/soci-mysql.h"
 #include "soci/soci-platform.h"
 #include "common.h"
 #include "soci-exchange-cast.h"
 #include "soci-mktime.h"
+#include "soci/blob.h"
 // std
 #include <ciso646>
+#include <cstdint>
 #include <cstdlib>
 #include <cstring>
 #include <ctime>
@@ -89,17 +92,47 @@ void mysql_standard_into_type_backend::post_fetch(
                 dest.assign(buf, lengths[pos]);
             }
             break;
-        case x_short:
-            parse_num(buf, exchange_type_cast<x_short>(data_));
+        case x_int8:
+            {
+                int32_t tmp = 0;
+                parse_num(buf, tmp);
+                if (tmp < (std::numeric_limits<int8_t>::min)() ||
+                    tmp > (std::numeric_limits<int8_t>::max)())
+                {
+                    throw soci_error("Cannot convert data.");
+                }
+                exchange_type_cast<x_int8>(data_) = static_cast<int8_t>(tmp);
+            }
             break;
-        case x_integer:
-            parse_num(buf, exchange_type_cast<x_integer>(data_));
+        case x_uint8:
+            {
+                uint32_t tmp = 0;
+                parse_num(buf, tmp);
+                if (tmp < (std::numeric_limits<uint8_t>::min)() ||
+                    tmp > (std::numeric_limits<uint8_t>::max)())
+                {
+                    throw soci_error("Cannot convert data.");
+                }
+                exchange_type_cast<x_uint8>(data_) = static_cast<uint8_t>(tmp);
+            }
             break;
-        case x_long_long:
-            parse_num(buf, exchange_type_cast<x_long_long>(data_));
+        case x_int16:
+            parse_num(buf, exchange_type_cast<x_int16>(data_));
             break;
-        case x_unsigned_long_long:
-            parse_num(buf, exchange_type_cast<x_unsigned_long_long>(data_));
+        case x_uint16:
+            parse_num(buf, exchange_type_cast<x_uint16>(data_));
+            break;
+        case x_int32:
+            parse_num(buf, exchange_type_cast<x_int32>(data_));
+            break;
+        case x_uint32:
+            parse_num(buf, exchange_type_cast<x_uint32>(data_));
+            break;
+        case x_int64:
+            parse_num(buf, exchange_type_cast<x_int64>(data_));
+            break;
+        case x_uint64:
+            parse_num(buf, exchange_type_cast<x_uint64>(data_));
             break;
         case x_double:
             parse_num(buf, exchange_type_cast<x_double>(data_));
@@ -108,6 +141,17 @@ void mysql_standard_into_type_backend::post_fetch(
             // attempt to parse the string and convert to std::tm
             parse_std_tm(buf, exchange_type_cast<x_stdtm>(data_));
             break;
+        case x_blob:
+            {
+                unsigned long * lengths = mysql_fetch_lengths(statement_.result_);
+                std::size_t size = lengths[pos];
+                blob &b = exchange_type_cast<x_blob>(data_);
+
+                mysql_blob_backend *bbe = static_cast<mysql_blob_backend *>(b.get_backend());
+
+                bbe->set_data(buf, size);
+            }
+            break;
         default:
             throw soci_error("Into element used with non-supported type.");
         }
