@@ -3,7 +3,7 @@
 // MySQL backend copyright (C) 2006 Pawel Aleksander Fedorynski
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_MYSQL_SOURCE
@@ -12,8 +12,11 @@
 #include "soci/soci-platform.h"
 #include "soci-dtocstr.h"
 #include "soci-exchange-cast.h"
+#include "soci-mktime.h"
+#include "soci/blob.h"
 // std
 #include <ciso646>
+#include <cstdint>
 #include <cstdio>
 #include <cstring>
 #include <limits>
@@ -64,38 +67,70 @@ void mysql_standard_use_type_backend::pre_use(indicator const *ind)
                              s.c_str(), s.size());
             }
             break;
-        case x_short:
+        case x_int8:
             {
                 std::size_t const bufSize
-                    = std::numeric_limits<short>::digits10 + 3;
+                    = std::numeric_limits<int8_t>::digits10 + 3;
                 buf_ = new char[bufSize];
-                snprintf(buf_, bufSize, "%d",
-                    static_cast<int>(exchange_type_cast<x_short>(data_)));
+                snprintf(buf_, bufSize, "%d", exchange_type_cast<x_int8>(data_));
             }
             break;
-        case x_integer:
+        case x_uint8:
             {
                 std::size_t const bufSize
-                    = std::numeric_limits<int>::digits10 + 3;
+                    = std::numeric_limits<uint8_t>::digits10 + 3;
                 buf_ = new char[bufSize];
-                snprintf(buf_, bufSize, "%d", exchange_type_cast<x_integer>(data_));
+                snprintf(buf_, bufSize, "%u", exchange_type_cast<x_uint8>(data_));
             }
             break;
-        case x_long_long:
+        case x_int16:
             {
                 std::size_t const bufSize
-                    = std::numeric_limits<long long>::digits10 + 3;
+                    = std::numeric_limits<int16_t>::digits10 + 3;
                 buf_ = new char[bufSize];
-                snprintf(buf_, bufSize, "%" LL_FMT_FLAGS "d", exchange_type_cast<x_long_long>(data_));
+                snprintf(buf_, bufSize, "%d", exchange_type_cast<x_int16>(data_));
             }
             break;
-        case x_unsigned_long_long:
+        case x_uint16:
             {
                 std::size_t const bufSize
-                    = std::numeric_limits<unsigned long long>::digits10 + 3;
+                    = std::numeric_limits<uint16_t>::digits10 + 3;
+                buf_ = new char[bufSize];
+                snprintf(buf_, bufSize, "%u", exchange_type_cast<x_uint16>(data_));
+            }
+            break;
+        case x_int32:
+            {
+                std::size_t const bufSize
+                    = std::numeric_limits<int32_t>::digits10 + 3;
+                buf_ = new char[bufSize];
+                snprintf(buf_, bufSize, "%d", exchange_type_cast<x_int32>(data_));
+            }
+            break;
+        case x_uint32:
+            {
+                std::size_t const bufSize
+                    = std::numeric_limits<uint32_t>::digits10 + 3;
+                buf_ = new char[bufSize];
+                snprintf(buf_, bufSize, "%u", exchange_type_cast<x_uint32>(data_));
+            }
+            break;
+        case x_int64:
+            {
+                std::size_t const bufSize
+                    = std::numeric_limits<int64_t>::digits10 + 3;
+                buf_ = new char[bufSize];
+                snprintf(buf_, bufSize, "%" LL_FMT_FLAGS "d",
+                    static_cast<long long>(exchange_type_cast<x_int64>(data_)));
+            }
+            break;
+        case x_uint64:
+            {
+                std::size_t const bufSize
+                    = std::numeric_limits<uint64_t>::digits10 + 3;
                 buf_ = new char[bufSize];
                 snprintf(buf_, bufSize, "%" LL_FMT_FLAGS "u",
-                         exchange_type_cast<x_unsigned_long_long>(data_));
+                    static_cast<unsigned long long>(exchange_type_cast<x_uint64>(data_)));
             }
             break;
 
@@ -119,11 +154,40 @@ void mysql_standard_use_type_backend::pre_use(indicator const *ind)
                 std::size_t const bufSize = 80;
                 buf_ = new char[bufSize];
 
-                std::tm const& t = exchange_type_cast<x_stdtm>(data_);
-                snprintf(buf_, bufSize,
-                    "\'%d-%02d-%02d %02d:%02d:%02d\'",
-                    t.tm_year + 1900, t.tm_mon + 1, t.tm_mday,
-                    t.tm_hour, t.tm_min, t.tm_sec);
+                int n = 0;
+                buf_[n++] = '\'';
+                n += format_std_tm(exchange_type_cast<x_stdtm>(data_),
+                                   buf_ + n, bufSize - n - 1);
+                buf_[n++] = '\'';
+                buf_[n] = '\0';
+            }
+            break;
+        case x_blob:
+            {
+                blob * b = static_cast<blob *>(data_);
+                mysql_blob_backend * bbe =
+                    static_cast<mysql_blob_backend *>(b->get_backend());
+
+                std::size_t hex_size = bbe->hex_str_size();
+                if (hex_size == 0)
+                {
+                    // We can't represent an empty BLOB as hex (thus hex_str_size returns 0). Instead, we'll
+                    // use '' to initialize and empty BLOB.
+                    buf_ = new char[3];
+                    buf_[0] = '\'';
+                    buf_[1] = '\'';
+                    buf_[2] = '\0';
+                }
+                else
+                {
+                    // Note: since the entire MySQL works by assembling the query in text form,
+                    // we can't make use of proper blob objects (that'd require a more sophisticated
+                    // way of using the MySQL API).
+                    buf_ = new char[hex_size + 1];
+                    bbe->write_hex_str(buf_, hex_size);
+                    // Add NULL terminator
+                    buf_[hex_size] = '\0';
+                }
             }
             break;
         default:
