@@ -3,7 +3,7 @@
 // MySQL backend copyright (C) 2006 Pawel Aleksander Fedorynski
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_MYSQL_SOURCE
@@ -19,9 +19,7 @@ using std::string;
 mysql_statement_backend::mysql_statement_backend(
     mysql_session_backend &session)
     : session_(session), result_(NULL),
-       rowsAffectedBulk_(-1LL), justDescribed_(false),
-       hasIntoElements_(false), hasVectorIntoElements_(false),
-       hasUseElements_(false), hasVectorUseElements_(false)
+       rowsAffectedBulk_(-1LL), justDescribed_(false)
 {
 }
 
@@ -402,24 +400,44 @@ int mysql_statement_backend::prepare_for_describe()
 }
 
 void mysql_statement_backend::describe_column(int colNum,
-    data_type & type, std::string & columnName)
+    db_type & dbtype, std::string & columnName)
 {
+    lastDescribedUnsignedMediumInt_ = false;
+
     int pos = colNum - 1;
     MYSQL_FIELD *field = mysql_fetch_field_direct(result_, pos);
     switch (field->type)
     {
     case FIELD_TYPE_CHAR:       //MYSQL_TYPE_TINY:
+        dbtype = field->flags & UNSIGNED_FLAG ? db_uint8 : db_int8;
+        break;
     case FIELD_TYPE_SHORT:      //MYSQL_TYPE_SHORT:
+        dbtype = field->flags & UNSIGNED_FLAG ? db_uint16 : db_int16;
+        break;
     case FIELD_TYPE_INT24:      //MYSQL_TYPE_INT24:
-        type = dt_integer;
+        dbtype = field->flags & UNSIGNED_FLAG ? db_uint32 : db_int32;
+        if (dbtype == db_uint32)
+            lastDescribedUnsignedMediumInt_ = true;
         break;
     case FIELD_TYPE_LONG:       //MYSQL_TYPE_LONG:
-        type = field->flags & UNSIGNED_FLAG ? dt_long_long
-                                            : dt_integer;
+        if (field->flags & UNSIGNED_FLAG)
+        {
+            dbtype = db_uint32;
+        }
+        else
+        {
+            dbtype = db_int32;
+        }
         break;
     case FIELD_TYPE_LONGLONG:   //MYSQL_TYPE_LONGLONG:
-        type = field->flags & UNSIGNED_FLAG ? dt_unsigned_long_long :
-                                              dt_long_long;
+        if (field->flags & UNSIGNED_FLAG)
+        {
+            dbtype = db_uint64;
+        }
+        else
+        {
+            dbtype = db_int64;
+        }
         break;
     case FIELD_TYPE_FLOAT:      //MYSQL_TYPE_FLOAT:
     case FIELD_TYPE_DOUBLE:     //MYSQL_TYPE_DOUBLE:
@@ -429,7 +447,7 @@ void mysql_statement_backend::describe_column(int colNum,
     // sends field type number 246, no matter which version of libraries
     // the client is using.
     case 246:                   //MYSQL_TYPE_NEWDECIMAL:
-        type = dt_double;
+        dbtype = db_double;
         break;
     case FIELD_TYPE_TIMESTAMP:  //MYSQL_TYPE_TIMESTAMP:
     case FIELD_TYPE_DATE:       //MYSQL_TYPE_DATE:
@@ -437,17 +455,26 @@ void mysql_statement_backend::describe_column(int colNum,
     case FIELD_TYPE_DATETIME:   //MYSQL_TYPE_DATETIME:
     case FIELD_TYPE_YEAR:       //MYSQL_TYPE_YEAR:
     case FIELD_TYPE_NEWDATE:    //MYSQL_TYPE_NEWDATE:
-        type = dt_date;
+        dbtype = db_date;
         break;
 //  case MYSQL_TYPE_VARCHAR:
+    case 245:                   //MYSQL_TYPE_JSON:
     case FIELD_TYPE_VAR_STRING: //MYSQL_TYPE_VAR_STRING:
     case FIELD_TYPE_STRING:     //MYSQL_TYPE_STRING:
-    case FIELD_TYPE_BLOB:       // TEXT OR BLOB
+        dbtype = db_string;
+        break;
+    case FIELD_TYPE_BLOB:       // BLOB
     case FIELD_TYPE_TINY_BLOB:
     case FIELD_TYPE_MEDIUM_BLOB:
     case FIELD_TYPE_LONG_BLOB:
-        type = dt_string;
+    {
+        // Quoted from the docs:
+        // To distinguish between binary and nonbinary data for string data types, check whether the
+        // charsetnr value is 63. If so, the character set is binary, which indicates binary rather
+        // than nonbinary data.
+        dbtype = field->charsetnr == 63 ? db_blob : db_string;
         break;
+    }
     default:
         //std::cerr << "field->type: " << field->type << std::endl;
         throw soci_error("Unknown data type.");
@@ -455,30 +482,42 @@ void mysql_statement_backend::describe_column(int colNum,
     columnName = field->name;
 }
 
+data_type mysql_statement_backend::to_data_type(db_type dbt) const
+{
+    // Before adding db_type, this backend returned dt_integer for 24 bit
+    // unsigned values but dt_long_long for 32 bit unsigned ones and now we
+    // return the same db_uint32 for both and translate it to different legacy
+    // values depending on the flag set by describe_column(). This is pretty
+    // ugly but needed to preserve compatibility for the people who use MySQL
+    // MEDIUMINT UNSIGNED with SOCI.
+    if (lastDescribedUnsignedMediumInt_ && dbt == db_uint32)
+    {
+        return dt_integer;
+    }
+
+    return statement_backend::to_data_type(dbt);
+}
+
 mysql_standard_into_type_backend *
 mysql_statement_backend::make_into_type_backend()
 {
-    hasIntoElements_ = true;
     return new mysql_standard_into_type_backend(*this);
 }
 
 mysql_standard_use_type_backend *
 mysql_statement_backend::make_use_type_backend()
 {
-    hasUseElements_ = true;
     return new mysql_standard_use_type_backend(*this);
 }
 
 mysql_vector_into_type_backend *
 mysql_statement_backend::make_vector_into_type_backend()
 {
-    hasVectorIntoElements_ = true;
     return new mysql_vector_into_type_backend(*this);
 }
 
 mysql_vector_use_type_backend *
 mysql_statement_backend::make_vector_use_type_backend()
 {
-    hasVectorUseElements_ = true;
     return new mysql_vector_use_type_backend(*this);
 }
