@@ -2,10 +2,10 @@
 // Copyright (C) 2004-2007 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
-#define soci_ORACLE_SOURCE
+#define SOCI_ORACLE_SOURCE
 #include "soci/oracle/soci-oracle.h"
 #include "soci/blob.h"
 #include "clob.h"
@@ -20,6 +20,7 @@
 #include "soci-mktime.h"
 
 #include <cctype>
+#include <cstdint>
 #include <cstdio>
 #include <cstring>
 #include <cstdlib>
@@ -28,7 +29,6 @@
 
 #ifdef _MSC_VER
 #pragma warning(disable:4355)
-#define snprintf _snprintf
 #endif
 
 using namespace soci;
@@ -52,18 +52,54 @@ void oracle_standard_use_type_backend::prepare_for_bind(
             data = buf_;
         }
         break;
-    case x_short:
+    case x_int8:
         oracleType = SQLT_INT;
-        size = sizeof(short);
+        size = sizeof(int8_t);
         if (readOnly)
         {
             buf_ = new char[size];
             data = buf_;
         }
         break;
-    case x_integer:
+    case x_uint8:
+        oracleType = SQLT_UIN;
+        size = sizeof(uint8_t);
+        if (readOnly)
+        {
+            buf_ = new char[size];
+            data = buf_;
+        }
+        break;
+    case x_int16:
         oracleType = SQLT_INT;
-        size = sizeof(int);
+        size = sizeof(int16_t);
+        if (readOnly)
+        {
+            buf_ = new char[size];
+            data = buf_;
+        }
+        break;
+    case x_uint16:
+        oracleType = SQLT_UIN;
+        size = sizeof(uint16_t);
+        if (readOnly)
+        {
+            buf_ = new char[size];
+            data = buf_;
+        }
+        break;
+    case x_int32:
+        oracleType = SQLT_INT;
+        size = sizeof(int32_t);
+        if (readOnly)
+        {
+            buf_ = new char[size];
+            data = buf_;
+        }
+        break;
+    case x_uint32:
+        oracleType = SQLT_UIN;
+        size = sizeof(uint32_t);
         if (readOnly)
         {
             buf_ = new char[size];
@@ -81,8 +117,8 @@ void oracle_standard_use_type_backend::prepare_for_bind(
         break;
 
     // cases that require adjustments and buffer management
-    case x_long_long:
-    case x_unsigned_long_long:
+    case x_int64:
+    case x_uint64:
         oracleType = SQLT_STR;
         size = 100; // arbitrary buffer length
         buf_ = new char[size];
@@ -112,7 +148,7 @@ void oracle_standard_use_type_backend::prepare_for_bind(
 
             oracle_statement_backend *stbe
                 = static_cast<oracle_statement_backend *>(st->get_backend());
-            size = 0;
+            size = sizeof(stbe->stmtp_);
             data = &stbe->stmtp_;
         }
         break;
@@ -125,7 +161,7 @@ void oracle_standard_use_type_backend::prepare_for_bind(
             oracle_rowid_backend *rbe
                 = static_cast<oracle_rowid_backend *>(rid->get_backend());
 
-            size = 0;
+            size = sizeof(rbe->rowidp_);
             data = &rbe->rowidp_;
         }
         break;
@@ -138,8 +174,13 @@ void oracle_standard_use_type_backend::prepare_for_bind(
             oracle_blob_backend *bbe
                 = static_cast<oracle_blob_backend *>(b->get_backend());
 
-            size = 0;
-            data = &bbe->lobp_;
+            // We have to make sure that our blob backend is actually properly initialized
+            bbe->ensure_initialized();
+
+            ociData_ = bbe->get_lob_locator();
+
+            size = sizeof(ociData_);
+            data = &ociData_;
         }
         break;
 
@@ -159,6 +200,9 @@ void oracle_standard_use_type_backend::prepare_for_bind(
             ociData_ = lobp;
         }
         break;
+
+    case x_stdwstring:
+        throw soci_error("Wide string use elements are not supported by Oracle backend.");
     }
 }
 
@@ -333,28 +377,54 @@ void oracle_standard_use_type_backend::pre_use(indicator const *ind)
             buf_[0] = exchange_type_cast<x_char>(data_);
         }
         break;
-    case x_short:
+    case x_int8:
         if (readOnly_)
         {
-            exchange_type_cast<x_short>(buf_) = exchange_type_cast<x_short>(data_);
+            exchange_type_cast<x_int8>(buf_) = exchange_type_cast<x_int8>(data_);
         }
         break;
-    case x_integer:
+    case x_uint8:
         if (readOnly_)
         {
-            exchange_type_cast<x_integer>(buf_) = exchange_type_cast<x_integer>(data_);
+            exchange_type_cast<x_uint8>(buf_) = exchange_type_cast<x_uint8>(data_);
         }
         break;
-    case x_long_long:
+    case x_int16:
+        if (readOnly_)
         {
-            size_t const size = 100; // arbitrary, but consistent with prepare_for_bind
-            snprintf(buf_, size, "%" LL_FMT_FLAGS "d", exchange_type_cast<x_long_long>(data_));
+            exchange_type_cast<x_int16>(buf_) = exchange_type_cast<x_int16>(data_);
         }
         break;
-    case x_unsigned_long_long:
+    case x_uint16:
+        if (readOnly_)
+        {
+            exchange_type_cast<x_uint16>(buf_) = exchange_type_cast<x_uint16>(data_);
+        }
+        break;
+    case x_int32:
+        if (readOnly_)
+        {
+            exchange_type_cast<x_int32>(buf_) = exchange_type_cast<x_int32>(data_);
+        }
+        break;
+    case x_uint32:
+        if (readOnly_)
+        {
+            exchange_type_cast<x_uint32>(buf_) = exchange_type_cast<x_uint32>(data_);
+        }
+        break;
+    case x_int64:
         {
             size_t const size = 100; // arbitrary, but consistent with prepare_for_bind
-            snprintf(buf_, size, "%" LL_FMT_FLAGS "u", exchange_type_cast<x_unsigned_long_long>(data_));
+            snprintf(buf_, size, "%" LL_FMT_FLAGS "d",
+                static_cast<long long>(exchange_type_cast<x_int64>(data_)));
+        }
+        break;
+    case x_uint64:
+        {
+            size_t const size = 100; // arbitrary, but consistent with prepare_for_bind
+            snprintf(buf_, size, "%" LL_FMT_FLAGS "u",
+                static_cast<unsigned long long>(exchange_type_cast<x_uint64>(data_)));
         }
         break;
     case x_double:
@@ -382,7 +452,7 @@ void oracle_standard_use_type_backend::pre_use(indicator const *ind)
             ub1* pos = reinterpret_cast<ub1*>(buf_);
 
             *pos++ = static_cast<ub1>(100 + (1900 + t.tm_year) / 100);
-            *pos++ = static_cast<ub1>(100 + t.tm_year % 100);
+            *pos++ = static_cast<ub1>(100 + (t.tm_year % 100 + 100) % 100);
             *pos++ = static_cast<ub1>(t.tm_mon + 1);
             *pos++ = static_cast<ub1>(t.tm_mday);
             *pos++ = static_cast<ub1>(t.tm_hour + 1);
@@ -404,6 +474,8 @@ void oracle_standard_use_type_backend::pre_use(indicator const *ind)
     case x_blob:
         // nothing to do
         break;
+    case x_stdwstring:
+        throw soci_error("Wide string use elements are not supported by Oracle backend.");
     }
 
     // then handle indicators
@@ -442,11 +514,11 @@ void oracle_standard_use_type_backend::post_use(bool gotData, indicator *ind)
                 }
             }
             break;
-        case x_short:
+        case x_int8:
             if (readOnly_)
             {
-                const short original = exchange_type_cast<x_short>(data_);
-                const short bound = exchange_type_cast<x_short>(buf_);
+                const int8_t original = exchange_type_cast<x_int8>(data_);
+                const int8_t bound = exchange_type_cast<x_int8>(buf_);
 
                 if (original != bound)
                 {
@@ -454,11 +526,11 @@ void oracle_standard_use_type_backend::post_use(bool gotData, indicator *ind)
                 }
             }
             break;
-        case x_integer:
+        case x_uint8:
             if (readOnly_)
             {
-                const int original = exchange_type_cast<x_integer>(data_);
-                const int bound = exchange_type_cast<x_integer>(buf_);
+                const uint8_t original = exchange_type_cast<x_uint8>(data_);
+                const uint8_t bound = exchange_type_cast<x_uint8>(buf_);
 
                 if (original != bound)
                 {
@@ -466,11 +538,11 @@ void oracle_standard_use_type_backend::post_use(bool gotData, indicator *ind)
                 }
             }
             break;
-        case x_long_long:
+        case x_int16:
             if (readOnly_)
             {
-                long long const original = exchange_type_cast<x_long_long>(data_);
-                long long const bound = std::strtoll(buf_, NULL, 10);
+                const int16_t original = exchange_type_cast<x_int16>(data_);
+                const int16_t bound = exchange_type_cast<x_int16>(buf_);
 
                 if (original != bound)
                 {
@@ -478,11 +550,59 @@ void oracle_standard_use_type_backend::post_use(bool gotData, indicator *ind)
                 }
             }
             break;
-        case x_unsigned_long_long:
+        case x_uint16:
             if (readOnly_)
             {
-                unsigned long long const original = exchange_type_cast<x_unsigned_long_long>(data_);
-                unsigned long long const bound = std::strtoull(buf_, NULL, 10);
+                const uint16_t original = exchange_type_cast<x_uint16>(data_);
+                const uint16_t bound = exchange_type_cast<x_uint16>(buf_);
+
+                if (original != bound)
+                {
+                    throw soci_error("Attempted modification of const use element");
+                }
+            }
+            break;
+        case x_int32:
+            if (readOnly_)
+            {
+                const int32_t original = exchange_type_cast<x_int32>(data_);
+                const int32_t bound = exchange_type_cast<x_int32>(buf_);
+
+                if (original != bound)
+                {
+                    throw soci_error("Attempted modification of const use element");
+                }
+            }
+            break;
+        case x_uint32:
+            if (readOnly_)
+            {
+                const uint32_t original = exchange_type_cast<x_uint32>(data_);
+                const uint32_t bound = exchange_type_cast<x_uint32>(buf_);
+
+                if (original != bound)
+                {
+                    throw soci_error("Attempted modification of const use element");
+                }
+            }
+            break;
+        case x_int64:
+            if (readOnly_)
+            {
+                int64_t const original = exchange_type_cast<x_int64>(data_);
+                int64_t const bound = std::strtoll(buf_, NULL, 10);
+
+                if (original != bound)
+                {
+                    throw soci_error("Attempted modification of const use element");
+                }
+            }
+            break;
+        case x_uint64:
+            if (readOnly_)
+            {
+                uint64_t const original = exchange_type_cast<x_uint64>(data_);
+                uint64_t const bound = std::strtoull(buf_, NULL, 10);
 
                 if (original != bound)
                 {
@@ -498,14 +618,14 @@ void oracle_standard_use_type_backend::post_use(bool gotData, indicator *ind)
 
                 // Exact comparison is fine here, they are really supposed to
                 // be exactly the same.
-                GCC_WARNING_SUPPRESS(float-equal)
+                SOCI_GCC_WARNING_SUPPRESS(float-equal)
 
                 if (original != bound)
                 {
                     throw soci_error("Attempted modification of const use element");
                 }
 
-                GCC_WARNING_RESTORE(float-equal)
+                SOCI_GCC_WARNING_RESTORE(float-equal)
             }
             break;
         case x_stdstring:
@@ -564,12 +684,14 @@ void oracle_standard_use_type_backend::post_use(bool gotData, indicator *ind)
                 s->define_and_bind();
             }
             break;
-        case x_rowid:
         case x_blob:
+        case x_rowid:
         case x_xmltype:
         case x_longstring:
             // nothing to do here
             break;
+        case x_stdwstring:
+            throw soci_error("Wide string use elements are not supported by Oracle backend.");
         }
     }
 
@@ -595,9 +717,23 @@ void oracle_standard_use_type_backend::post_use(bool gotData, indicator *ind)
 
 void oracle_standard_use_type_backend::clean_up()
 {
-    if (type_ == x_xmltype || type_ == x_longstring)
+    if (ociData_)
     {
-        free_temp_lob(statement_.session_, static_cast<OCILobLocator *>(ociData_));
+        switch (type_)
+        {
+            case x_xmltype:
+            case x_longstring:
+                free_temp_lob(statement_.session_, static_cast<OCILobLocator *>(ociData_));
+                break;
+
+            case x_blob:
+                // We don't own the LOB locator, oracle_blob_backend does.
+                break;
+
+            default:
+                throw soci_error("Internal error: OCI data used for unexpected type");
+        }
+
         ociData_ = NULL;
     }
 
