@@ -2,7 +2,7 @@
 // Copyright (C) 2004-2008 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_POSTGRESQL_SOURCE
@@ -15,8 +15,12 @@
 #include "soci/blob.h"
 #include "soci/type-wrappers.h"
 #include "soci-exchange-cast.h"
+
+#include <libpq-fe.h>
 #include <libpq/libpq-fs.h> // libpq
+
 #include <cctype>
+#include <cstdint>
 #include <cstdio>
 #include <cstring>
 #include <cstdlib>
@@ -90,17 +94,29 @@ void postgresql_standard_into_type_backend::post_fetch(
         case x_stdstring:
             exchange_type_cast<x_stdstring>(data_) = buf;
             break;
-        case x_short:
-            exchange_type_cast<x_short>(data_) = string_to_integer<short>(buf);
+        case x_int8:
+            exchange_type_cast<x_int8>(data_) = string_to_integer<int8_t>(buf);
             break;
-        case x_integer:
-            exchange_type_cast<x_integer>(data_) = string_to_integer<int>(buf);
+        case x_uint8:
+            exchange_type_cast<x_uint8>(data_) = string_to_integer<uint8_t>(buf);
             break;
-        case x_long_long:
-            exchange_type_cast<x_long_long>(data_) = string_to_integer<long long>(buf);
+        case x_int16:
+            exchange_type_cast<x_int16>(data_) = string_to_integer<int16_t>(buf);
             break;
-        case x_unsigned_long_long:
-            exchange_type_cast<x_unsigned_long_long>(data_) = string_to_unsigned_integer<unsigned long long>(buf);
+        case x_uint16:
+            exchange_type_cast<x_uint16>(data_) = string_to_integer<uint16_t>(buf);
+            break;
+        case x_int32:
+            exchange_type_cast<x_int32>(data_) = string_to_integer<int32_t>(buf);
+            break;
+        case x_uint32:
+            exchange_type_cast<x_uint32>(data_) = string_to_integer<uint32_t>(buf);
+            break;
+        case x_int64:
+            exchange_type_cast<x_int64>(data_) = string_to_integer<int64_t>(buf);
+            break;
+        case x_uint64:
+            exchange_type_cast<x_uint64>(data_) = string_to_unsigned_integer<uint64_t>(buf);
             break;
         case x_double:
             exchange_type_cast<x_double>(data_) = cstring_to_double(buf);
@@ -130,20 +146,15 @@ void postgresql_standard_into_type_backend::post_fetch(
                     INV_READ | INV_WRITE);
                 if (fd == -1)
                 {
-                    throw soci_error("Cannot open the blob object.");
+                    const char *errorMsg = PQerrorMessage(statement_.session_.conn_);
+                    throw soci_error(std::string("Cannot open the blob object: ") + errorMsg);
                 }
 
                 blob * b = static_cast<blob *>(data_);
                 postgresql_blob_backend * bbe
                      = static_cast<postgresql_blob_backend *>(b->get_backend());
 
-                if (bbe->fd_ != -1)
-                {
-                    lo_close(statement_.session_.conn_, bbe->fd_);
-                }
-
-                bbe->fd_ = fd;
-                bbe->oid_ = oid;
+                bbe->set_blob_details(postgresql_blob_backend::blob_details(oid, fd));
             }
             break;
         case x_xmltype:
