@@ -2,7 +2,7 @@
 // Copyright (C) 2004-2008 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_POSTGRESQL_SOURCE
@@ -13,8 +13,12 @@
 #include "soci/soci-platform.h"
 #include "soci-dtocstr.h"
 #include "soci-exchange-cast.h"
-#include <libpq/libpq-fs.h> // libpq
+#include "soci-mktime.h"
+
+#include <libpq-fe.h>
+
 #include <cctype>
+#include <cstdint>
 #include <cstdio>
 #include <cstring>
 #include <ctime>
@@ -67,40 +71,70 @@ void postgresql_standard_use_type_backend::pre_use(indicator const * ind)
         case x_stdstring:
             copy_from_string(exchange_type_cast<x_stdstring>(data_));
             break;
-        case x_short:
+        case x_int8:
             {
                 std::size_t const bufSize
-                    = std::numeric_limits<short>::digits10 + 3;
+                    = std::numeric_limits<int8_t>::digits10 + 3;
                 buf_ = new char[bufSize];
-                snprintf(buf_, bufSize, "%d",
-                    static_cast<int>(exchange_type_cast<x_short>(data_)));
+                snprintf(buf_, bufSize, "%d", exchange_type_cast<x_int8>(data_));
             }
             break;
-        case x_integer:
+        case x_uint8:
             {
                 std::size_t const bufSize
-                    = std::numeric_limits<int>::digits10 + 3;
+                    = std::numeric_limits<uint8_t>::digits10 + 3;
                 buf_ = new char[bufSize];
-                snprintf(buf_, bufSize, "%d",
-                    exchange_type_cast<x_integer>(data_));
+                snprintf(buf_, bufSize, "%u", exchange_type_cast<x_uint8>(data_));
             }
             break;
-        case x_long_long:
+        case x_int16:
             {
                 std::size_t const bufSize
-                    = std::numeric_limits<long long>::digits10 + 3;
+                    = std::numeric_limits<int16_t>::digits10 + 3;
+                buf_ = new char[bufSize];
+                snprintf(buf_, bufSize, "%d", exchange_type_cast<x_int16>(data_));
+            }
+            break;
+        case x_uint16:
+            {
+                std::size_t const bufSize
+                    = std::numeric_limits<uint16_t>::digits10 + 3;
+                buf_ = new char[bufSize];
+                snprintf(buf_, bufSize, "%u", exchange_type_cast<x_uint16>(data_));
+            }
+            break;
+        case x_int32:
+            {
+                std::size_t const bufSize
+                    = std::numeric_limits<int32_t>::digits10 + 3;
+                buf_ = new char[bufSize];
+                snprintf(buf_, bufSize, "%d", exchange_type_cast<x_int32>(data_));
+            }
+            break;
+        case x_uint32:
+            {
+                std::size_t const bufSize
+                    = std::numeric_limits<uint32_t>::digits10 + 3;
+                buf_ = new char[bufSize];
+                snprintf(buf_, bufSize, "%u", exchange_type_cast<x_uint32>(data_));
+            }
+            break;
+        case x_int64:
+            {
+                std::size_t const bufSize
+                    = std::numeric_limits<int64_t>::digits10 + 3;
                 buf_ = new char[bufSize];
                 snprintf(buf_, bufSize, "%" LL_FMT_FLAGS "d",
-                    exchange_type_cast<x_long_long>(data_));
+                    static_cast<long long>(exchange_type_cast<x_int64>(data_)));
             }
             break;
-        case x_unsigned_long_long:
+        case x_uint64:
             {
                 std::size_t const bufSize
-                    = std::numeric_limits<unsigned long long>::digits10 + 2;
+                    = std::numeric_limits<uint64_t>::digits10 + 2;
                 buf_ = new char[bufSize];
                 snprintf(buf_, bufSize, "%" LL_FMT_FLAGS "u",
-                    exchange_type_cast<x_unsigned_long_long>(data_));
+                    static_cast<unsigned long long>(exchange_type_cast<x_uint64>(data_)));
             }
             break;
         case x_double:
@@ -111,10 +145,7 @@ void postgresql_standard_use_type_backend::pre_use(indicator const * ind)
                 std::size_t const bufSize = 80;
                 buf_ = new char[bufSize];
 
-                std::tm const& t = exchange_type_cast<x_stdtm>(data_);
-                snprintf(buf_, bufSize, "%d-%02d-%02d %02d:%02d:%02d",
-                    t.tm_year + 1900, t.tm_mon + 1, t.tm_mday,
-                    t.tm_hour, t.tm_min, t.tm_sec);
+                format_std_tm(exchange_type_cast<x_stdtm>(data_), buf_, bufSize);
             }
             break;
         case x_rowid:
@@ -139,10 +170,29 @@ void postgresql_standard_use_type_backend::pre_use(indicator const * ind)
                 postgresql_blob_backend * bbe =
                     static_cast<postgresql_blob_backend *>(b->get_backend());
 
+                // In case the backend does not reference a proper BLOB yet, this will create a
+                // new, empty one
+                bbe->init();
+
+                unsigned long oid = bbe->get_blob_details().oid;
+
+                if (oid == InvalidOid)
+                {
+                    throw soci_error("Cannot insert invalid BLOB.");
+                }
+
+                // In case the blob backend has created a new BLOB in the DB, we have to tell it
+                // to not destroy it again, because we will now actually insert it into the DB.
+                bbe->set_destroy_on_close(false);
+
+                // We don't want any subsequent changes made to the blob object to leak to the
+                // object that we are currently writing into the DB
+                bbe->set_clone_before_modify(true);
+
                 std::size_t const bufSize
                     = std::numeric_limits<unsigned long>::digits10 + 2;
                 buf_ = new char[bufSize];
-                snprintf(buf_, bufSize, "%lu", bbe->oid_);
+                snprintf(buf_, bufSize, "%lu", oid);
             }
             break;
         case x_xmltype:
