@@ -2,13 +2,17 @@
 // Copyright (C) 2004-2008 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_POSTGRESQL_SOURCE
 #include "soci/postgresql/soci-postgresql.h"
 #include "soci/soci-platform.h"
-#include <libpq/libpq-fs.h> // libpq
+#include "soci-cstrtoi.h"
+#include "soci-ssize.h"
+
+#include <libpq-fe.h>
+
 #include <cctype>
 #include <cstdio>
 #include <cstdlib>
@@ -23,7 +27,6 @@ namespace // unnamed
 {
 
 // used only with asynchronous operations in single-row mode
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
 void wait_until_operation_complete(postgresql_session_backend & session)
 {
     for (;;)
@@ -49,7 +52,6 @@ void throw_soci_error(PGconn * conn, const char * msg)
 
     throw soci_error(description);
 }
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
 
 } // unnamed namespace
 
@@ -57,16 +59,8 @@ postgresql_statement_backend::postgresql_statement_backend(
     postgresql_session_backend &session, bool single_row_mode)
     : session_(session), single_row_mode_(single_row_mode),
       result_(session, NULL),
-      rowsAffectedBulk_(-1LL), justDescribed_(false),
-      hasIntoElements_(false), hasVectorIntoElements_(false),
-      hasUseElements_(false), hasVectorUseElements_(false)
+      rowsAffectedBulk_(-1LL), justDescribed_(false)
 {
-#ifdef SOCI_POSTGRESQL_NOSINGLEROWMODE
-  if (single_row_mode)
-  {
-    throw soci_error("Single row mode not supported in this version of the library");
-  }
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
 }
 
 postgresql_statement_backend::~postgresql_statement_backend()
@@ -98,6 +92,8 @@ void postgresql_statement_backend::clean_up()
     // potential new execution.
     rowsAffectedBulk_ = -1;
 
+    current_row_ = -1;
+
     // nothing to do here
 }
 
@@ -229,13 +225,12 @@ void postgresql_statement_backend::prepare(std::string const & query,
         // if it fails to prepare it we can't DEALLOCATE it.
         std::string statementName = session_.get_next_statement_name();
 
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
         if (single_row_mode_)
         {
             // prepare for single-row retrieval
 
             int result = PQsendPrepare(session_.conn_, statementName.c_str(),
-                query_.c_str(), static_cast<int>(names_.size()), NULL);
+                query_.c_str(), ssize(names_), NULL);
             if (result != 1)
             {
                 throw_soci_error(session_.conn_,
@@ -245,13 +240,12 @@ void postgresql_statement_backend::prepare(std::string const & query,
             wait_until_operation_complete(session_);
         }
         else
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
         {
             // default multi-row query execution
 
             postgresql_result result(session_,
                 PQprepare(session_.conn_, statementName.c_str(),
-                    query_.c_str(), static_cast<int>(names_.size()), NULL));
+                    query_.c_str(), ssize(names_), NULL));
             result.check_for_errors("Cannot prepare statement.");
         }
 
@@ -265,12 +259,10 @@ void postgresql_statement_backend::prepare(std::string const & query,
 statement_backend::exec_fetch_result
 postgresql_statement_backend::execute(int number)
 {
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
     if (single_row_mode_ && (number > 1))
     {
         throw soci_error("Bulk operations are not supported with single-row mode.");
     }
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
 
     // If the statement was "just described", then we know that
     // it was actually executed with all the use elements
@@ -318,8 +310,9 @@ postgresql_statement_backend::execute(int number)
                     "Binding for use elements must be either by position "
                     "or by name.");
             }
-            long long rowsAffectedBulkTemp = 0;
-            for (int i = 0; i != numberOfExecutions; ++i)
+
+            rowsAffectedBulk_ = 0;
+            for (current_row_ = 0; current_row_ != numberOfExecutions; ++current_row_)
             {
                 std::vector<char *> paramValues;
 
@@ -335,7 +328,7 @@ postgresql_statement_backend::execute(int number)
                          it != end; ++it)
                     {
                         char ** buffers = it->second;
-                        paramValues.push_back(buffers[i]);
+                        paramValues.push_back(buffers[current_row_]);
                     }
                 }
                 else
@@ -357,7 +350,7 @@ postgresql_statement_backend::execute(int number)
                             throw soci_error(msg);
                         }
                         char ** buffers = b->second;
-                        paramValues.push_back(buffers[i]);
+                        paramValues.push_back(buffers[current_row_]);
                     }
                 }
 
@@ -365,12 +358,11 @@ postgresql_statement_backend::execute(int number)
                 {
                     // this query was separately prepared
 
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
                     if (single_row_mode_)
                     {
                         int result = PQsendQueryPrepared(session_.conn_,
                             statementName_.c_str(),
-                            static_cast<int>(paramValues.size()),
+                            ssize(paramValues),
                             &paramValues[0], NULL, NULL, 0);
                         if (result != 1)
                         {
@@ -386,13 +378,12 @@ postgresql_statement_backend::execute(int number)
                         }
                     }
                     else
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
                     {
                         // default multi-row execution
 
                         result_.reset(PQexecPrepared(session_.conn_,
                                 statementName_.c_str(),
-                                static_cast<int>(paramValues.size()),
+                                ssize(paramValues),
                                 &paramValues[0], NULL, NULL, 0));
                     }
                 }
@@ -401,11 +392,10 @@ postgresql_statement_backend::execute(int number)
                     // this query was not separately prepared and should
                     // be executed as a one-time query
 
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
                     if (single_row_mode_)
                     {
                         int result = PQsendQueryParams(session_.conn_, query_.c_str(),
-                            static_cast<int>(paramValues.size()),
+                            ssize(paramValues),
                             NULL, &paramValues[0], NULL, NULL, 0);
                         if (result != 1)
                         {
@@ -421,12 +411,11 @@ postgresql_statement_backend::execute(int number)
                         }
                     }
                     else
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
                     {
                         // default multi-row execution
 
                         result_.reset(PQexecParams(session_.conn_, query_.c_str(),
-                                static_cast<int>(paramValues.size()),
+                                ssize(paramValues),
                                 NULL, &paramValues[0], NULL, NULL, 0));
                     }
                 }
@@ -435,15 +424,13 @@ postgresql_statement_backend::execute(int number)
                 {
                     // there are only bulk use elements (no intos)
 
-                    // preserve the number of rows affected so far.
-                    rowsAffectedBulk_ = rowsAffectedBulkTemp;
-
                     result_.check_for_errors("Cannot execute query.");
 
-                    rowsAffectedBulkTemp += get_affected_rows();
+                    rowsAffectedBulk_ += get_affected_rows();
                 }
             }
-            rowsAffectedBulk_ = rowsAffectedBulkTemp;
+
+            current_row_ = -1;
 
             if (numberOfExecutions > 1)
             {
@@ -462,7 +449,6 @@ postgresql_statement_backend::execute(int number)
             {
                 // this query was separately prepared
 
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
                 if (single_row_mode_)
                 {
                     int result = PQsendQueryPrepared(session_.conn_,
@@ -481,7 +467,6 @@ postgresql_statement_backend::execute(int number)
                     }
                 }
                 else
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
                 {
                     // default multi-row execution
 
@@ -491,7 +476,6 @@ postgresql_statement_backend::execute(int number)
             }
             else // stType_ == st_one_time_query
             {
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
                 if (single_row_mode_)
                 {
                     int result = PQsendQuery(session_.conn_, query_.c_str());
@@ -509,7 +493,6 @@ postgresql_statement_backend::execute(int number)
                     }
                 }
                 else
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
                 {
                     // default multi-row execution
 
@@ -520,7 +503,6 @@ postgresql_statement_backend::execute(int number)
     }
 
     bool process_result;
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
     if (single_row_mode_)
     {
         if (justDescribed_)
@@ -537,7 +519,6 @@ postgresql_statement_backend::execute(int number)
         process_result = result_.check_for_data("Cannot execute query.");
     }
     else
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
     {
         // default multi-row execution
 
@@ -579,12 +560,10 @@ postgresql_statement_backend::execute(int number)
 statement_backend::exec_fetch_result
 postgresql_statement_backend::fetch(int number)
 {
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
     if (single_row_mode_ && (number > 1))
     {
         throw soci_error("Bulk operations are not supported with single-row mode.");
     }
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
 
     // Note:
     // In the multi-row mode this function does not actually fetch anything from anywhere
@@ -599,7 +578,6 @@ postgresql_statement_backend::fetch(int number)
 
     if (currentRow_ >= numberOfRows_)
     {
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
         if (single_row_mode_)
         {
             PGresult* res = PQgetResult(session_.conn_);
@@ -626,7 +604,6 @@ postgresql_statement_backend::fetch(int number)
             }
         }
         else
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
         {
             // default multi-row execution
 
@@ -639,7 +616,6 @@ postgresql_statement_backend::fetch(int number)
     {
         if (currentRow_ + number > numberOfRows_)
         {
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
             if (single_row_mode_)
             {
                 rowsToConsume_ = 1;
@@ -647,7 +623,6 @@ postgresql_statement_backend::fetch(int number)
                 return ef_success;
             }
             else
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
             {
                 // default multi-row execution
 
@@ -661,13 +636,11 @@ postgresql_statement_backend::fetch(int number)
         }
         else
         {
-#ifndef SOCI_POSTGRESQL_NOSINGLEROWMODE
             if (single_row_mode_)
             {
                 rowsToConsume_ = 1;
             }
             else
-#endif // !SOCI_POSTGRESQL_NOSINGLEROWMODE
             {
                 rowsToConsume_ = number;
             }
@@ -681,21 +654,11 @@ long long postgresql_statement_backend::get_affected_rows()
 {
     // PQcmdTuples() doesn't really modify the result but it takes a non-const
     // pointer to it, so we can't rely on implicit conversion here.
-    const char * const resultStr = PQcmdTuples(result_.get_result());
-    char * end;
-    long long result = std::strtoll(resultStr, &end, 0);
-    if (end != resultStr)
-    {
+    long long result;
+    if (cstring_to_integer(result, PQcmdTuples(result_.get_result())))
         return result;
-    }
-    else if (rowsAffectedBulk_ >= 0)
-    {
-        return rowsAffectedBulk_;
-    }
-    else
-    {
-        return -1;
-    }
+
+    return rowsAffectedBulk_;
 }
 
 int postgresql_statement_backend::get_number_of_rows()
@@ -725,8 +688,23 @@ int postgresql_statement_backend::prepare_for_describe()
     return columns;
 }
 
-void postgresql_statement_backend::describe_column(int colNum, data_type & type,
-    std::string & columnName)
+void throw_soci_type_error(Oid typeOid, int colNum, char category, const char* typeName )
+{
+    std::stringstream message;
+    message << "unknown data type"
+        << " for column number: " << colNum
+        << " with type oid: " << (int)typeOid;
+    if( category != '\0' )
+    {
+        message << " with category: " << category;
+    }
+    message << " with name: " << typeName;
+
+    throw soci_error(message.str());
+}
+
+void postgresql_statement_backend::describe_column(int colNum,
+    db_type & dbtype, std::string & columnName)
 {
     // In postgresql_ column numbers start from 0
     int const pos = colNum - 1;
@@ -735,7 +713,7 @@ void postgresql_statement_backend::describe_column(int colNum, data_type & type,
     switch (typeOid)
     {
     // Note: the following list of OIDs was taken from the pg_type table
-    // we do not claim that this list is exchaustive or even correct.
+    // we do not claim that this list is exhaustive or even correct.
 
                // from pg_type:
 
@@ -749,7 +727,11 @@ void postgresql_statement_backend::describe_column(int colNum, data_type & type,
     case 114:  // json
     case 17:   // bytea
     case 2950: // uuid
-        type = dt_string;
+    case 829:  // macaddr
+    case 869:  // inet
+    case 650:  // cidr
+    case 774:  // macaddr8
+        dbtype = db_string;
         break;
 
     case 702:  // abstime
@@ -759,41 +741,76 @@ void postgresql_statement_backend::describe_column(int colNum, data_type & type,
     case 1114: // timestamp
     case 1184: // timestamptz
     case 1266: // timetz
-        type = dt_date;
+        dbtype = db_date;
         break;
 
     case 700:  // float4
     case 701:  // float8
     case 1700: // numeric
-        type = dt_double;
+        dbtype = db_double;
         break;
 
     case 16:   // bool
+        dbtype = db_int8;
+        break;
+
     case 21:   // int2
+        dbtype = db_int16;
+        break;
+
     case 23:   // int4
-    case 26:   // oid
-        type = dt_integer;
+        dbtype = db_int32;
         break;
 
     case 20:   // int8
-        type = dt_long_long;
+        dbtype = db_int64;
         break;
 
+    case 26:   // oid
+        // Note that in theory OIDs can refer to all sorts of things, but their use
+        // for anything but BLOBs seems to be deprecated since PostreSQL 8, so we simply
+        // assume any OID refers to a BLOB.
+       dbtype = db_blob;
+       break;
+
     default:
     {
-        int form = PQfformat(result_, pos);
-        int size = PQfsize(result_, pos);
-        if (form == 0 && size == -1)
+        auto typeCategoryIt = categoryByColumnOID_.find(typeOid);
+        if ( typeCategoryIt == categoryByColumnOID_.end() )
         {
-            type = dt_string;
+            std::stringstream query;
+            query << "SELECT typcategory FROM pg_type WHERE oid=" << (int)typeOid;
+
+            soci::details::postgresql_result res(session_, PQexec(session_.conn_, query.str().c_str()));
+            if ( PQresultStatus(res.get_result()) != PGRES_TUPLES_OK )
+            {
+                throw_soci_type_error(typeOid, colNum, '\0', PQfname(result_, pos));
+            }
+
+            char* typeVal = PQgetvalue(res.get_result(), 0, 0);
+            auto iter_inserted = categoryByColumnOID_.insert( std::pair<unsigned long, char>( typeOid, typeVal[0] ) );
+            if ( !iter_inserted.second )
+            {
+                throw_soci_type_error(typeOid, colNum, typeVal[0], PQfname(result_, pos));
+            }
+
+            typeCategoryIt = iter_inserted.first;
         }
-        else
+
+        char typeCategory = (*typeCategoryIt).second;
+        switch ( typeCategory )
         {
-            std::stringstream message;
-            message << "unknown data type with typelem: " << typeOid
-                << " for colNum: " << colNum
-                << " with name: " << PQfname(result_, pos);
-            throw soci_error(message.str());
+            case 'D': // date type
+            case 'E': // enum type
+            case 'T': // time type
+            case 'S': // string type
+            case 'U': // user type
+            case 'I': // network address type
+                dbtype = db_string;
+                break;
+
+            default:
+                throw_soci_type_error(typeOid, colNum, typeCategory, PQfname(result_, pos));
         }
     }
     }
@@ -804,27 +821,23 @@ void postgresql_statement_backend::describe_column(int colNum, data_type & type,
 postgresql_standard_into_type_backend *
 postgresql_statement_backend::make_into_type_backend()
 {
-    hasIntoElements_ = true;
     return new postgresql_standard_into_type_backend(*this);
 }
 
 postgresql_standard_use_type_backend *
 postgresql_statement_backend::make_use_type_backend()
 {
-    hasUseElements_ = true;
     return new postgresql_standard_use_type_backend(*this);
 }
 
 postgresql_vector_into_type_backend *
 postgresql_statement_backend::make_vector_into_type_backend()
 {
-    hasVectorIntoElements_ = true;
     return new postgresql_vector_into_type_backend(*this);
 }
 
 postgresql_vector_use_type_backend *
 postgresql_statement_backend::make_vector_use_type_backend()
 {
-    hasVectorUseElements_ = true;
     return new postgresql_vector_use_type_backend(*this);
 }
