@@ -2,7 +2,7 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, David Courtney
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_SQLITE3_SOURCE
@@ -12,6 +12,7 @@
 #include "soci/blob.h"
 #include "soci-dtocstr.h"
 #include "soci-exchange-cast.h"
+#include "soci-mktime.h"
 // std
 #include <cstdio>
 #include <cstdlib>
@@ -26,7 +27,7 @@ using namespace soci::details;
 
 sqlite3_standard_use_type_backend::sqlite3_standard_use_type_backend(
         sqlite3_statement_backend &st)
-    : statement_(st), data_(NULL), type_(x_integer), position_(-1)
+    : statement_(st), data_(NULL), type_(x_int32), position_(-1)
 {
 }
 
@@ -79,7 +80,9 @@ void sqlite3_standard_use_type_backend::pre_use(indicator const * ind)
 
     if (statement_.useData_[0].size() < static_cast<std::size_t>(position_))
     {
-        statement_.useData_[0].resize(position_);
+        sqlite3_column empty;
+        empty.isNull_ = true;
+        statement_.useData_[0].resize(position_, empty);
     }
 
     sqlite3_column &col = statement_.useData_[0][pos];
@@ -97,6 +100,7 @@ void sqlite3_standard_use_type_backend::pre_use(indicator const * ind)
     {
         case x_char:
             col.type_ = dt_string;
+            col.dataType_ = db_string;
             col.buffer_.constData_ = &exchange_type_cast<x_char>(data_);
             col.buffer_.size_ = 1;
             break;
@@ -105,55 +109,82 @@ void sqlite3_standard_use_type_backend::pre_use(indicator const * ind)
         {
             const std::string &s = exchange_type_cast<x_stdstring>(data_);
             col.type_ = dt_string;
+            col.dataType_ = db_string;
             col.buffer_.constData_ = s.c_str();
             col.buffer_.size_ = s.size();
             break;
         }
 
-        case x_short:
+        case x_int8:
             col.type_ = dt_integer;
-            col.int32_ = exchange_type_cast<x_short>(data_);
+            col.dataType_ = db_int8;
+            col.int8_ = exchange_type_cast<x_int8>(data_);
             break;
 
-        case x_integer:
+        case x_uint8:
             col.type_ = dt_integer;
-            col.int32_ = exchange_type_cast<x_integer>(data_);
+            col.dataType_ = db_uint8;
+            col.uint8_ = exchange_type_cast<x_uint8>(data_);
             break;
 
-        case x_long_long:
-            col.type_ = dt_long_long;
-            col.int64_ = exchange_type_cast<x_long_long>(data_);
+        case x_int16:
+            col.type_ = dt_integer;
+            col.dataType_ = db_int16;
+            col.int16_ = exchange_type_cast<x_int16>(data_);
             break;
 
-        case x_unsigned_long_long:
+        case x_uint16:
+            col.type_ = dt_integer;
+            col.dataType_ = db_uint16;
+            col.uint16_ = exchange_type_cast<x_uint16>(data_);
+            break;
+
+        case x_int32:
+            col.type_ = dt_integer;
+            col.dataType_ = db_int32;
+            col.int32_ = exchange_type_cast<x_int32>(data_);
+            break;
+
+        case x_uint32:
             col.type_ = dt_long_long;
-            col.int64_ = exchange_type_cast<x_unsigned_long_long>(data_);
+            col.dataType_ = db_uint32;
+            col.uint32_ = exchange_type_cast<x_uint32>(data_);
+            break;
+
+        case x_int64:
+            col.type_ = dt_long_long;
+            col.dataType_ = db_int64;
+            col.int64_ = exchange_type_cast<x_int64>(data_);
+            break;
+
+        case x_uint64:
+            col.type_ = dt_unsigned_long_long;
+            col.dataType_ = db_uint64;
+            col.uint64_ = exchange_type_cast<x_uint64>(data_);
             break;
 
         case x_double:
             col.type_ = dt_double;
+            col.dataType_ = db_double;
             col.double_ = exchange_type_cast<x_double>(data_);
             break;
 
         case x_stdtm:
         {
             col.type_ = dt_date;
+            col.dataType_ = db_date;
             static const size_t bufSize = 20;
             std::tm &t = exchange_type_cast<x_stdtm>(data_);
 
             col.buffer_.data_ = new char[bufSize];
-            col.buffer_.size_
-                = snprintf(
-                    col.buffer_.data_, bufSize, "%d-%02d-%02d %02d:%02d:%02d",
-                    t.tm_year + 1900, t.tm_mon + 1, t.tm_mday,
-                    t.tm_hour, t.tm_min, t.tm_sec
-                );
+            col.buffer_.size_ = format_std_tm(t, col.buffer_.data_, bufSize);
             break;
         }
 
         case x_rowid:
         {
             col.type_ = dt_long_long;
+            col.dataType_ = db_int64;
             // RowID is internally identical to unsigned long
             rowid *rid = static_cast<rowid *>(data_);
             sqlite3_rowid_backend *rbe = static_cast<sqlite3_rowid_backend *>(rid->get_backend());
@@ -165,14 +196,33 @@ void sqlite3_standard_use_type_backend::pre_use(indicator const * ind)
         case x_blob:
         {
             col.type_ = dt_blob;
+            col.dataType_ = db_blob;
             blob *b = static_cast<blob *>(data_);
             sqlite3_blob_backend *bbe = static_cast<sqlite3_blob_backend *>(b->get_backend());
 
-            col.buffer_.constData_ = bbe->get_buffer();
+            // In case the internal buffer has not been initialized yet, get_buffer() will return nullptr. In this case,
+            // we want to make sure to insert an empty BLOB into the DB. However, when passing a nullptr to the
+            // sqlite3_bind_blob routine (in statement.cpp), it has the same effect as sqlite3_bind_null and thus
+            // is inserting NULL instead.
+            // Therefore, we want to make sure that the buffer is definitely initialized (though it can still be empty).
+            bbe->ensure_buffer_initialized();
+
+            static_assert(sizeof(decltype(*bbe->get_buffer())) == 1, "Expected byte-like type");
+            col.buffer_.constData_ = reinterpret_cast<const char *>(bbe->get_buffer());
             col.buffer_.size_ = bbe->get_len();
             break;
         }
 
+        case x_xmltype:
+        {
+            const soci::xml_type &xml = exchange_type_cast<x_xmltype>(data_);
+            col.type_ = dt_string;
+            col.dataType_ = db_string;
+            col.buffer_.constData_ = xml.value.c_str();
+            col.buffer_.size_ = xml.value.size();
+            break;
+        }
+
         default:
             throw soci_error("Use element used with non-supported type.");
     }
