@@ -2,29 +2,26 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, David Courtney
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_SQLITE3_SOURCE
 #include "soci/sqlite3/soci-sqlite3.h"
+#include "soci-ssize.h"
 // std
 #include <algorithm>
 #include <cctype>
+#include <cstdint>
 #include <cstring>
 #include <functional>
-#include <sstream>
 #include <string>
 
+#include "soci-case.h"
+
 #ifdef _MSC_VER
 #pragma warning(disable:4355)
 #endif
 
-// This is used instead of tolower() just to avoid warnings about int to char
-// casts inside MSVS std::transform() implementation.
-char toLowerCh(char c) {
-    return static_cast<char>( std::tolower(c) );
-}
-
 using namespace soci;
 using namespace soci::details;
 using namespace sqlite_api;
@@ -38,7 +35,6 @@ sqlite3_statement_backend::sqlite3_statement_backend(
     , databaseReady_(false)
     , boundByName_(false)
     , boundByPos_(false)
-    , hasVectorIntoElements_(false)
     , rowsAffectedBulk_(-1LL)
 {
 }
@@ -68,18 +64,12 @@ void sqlite3_statement_backend::prepare(std::string const & query,
     char const* tail = 0; // unused;
     int const res = sqlite3_prepare_v2(session_.conn_,
                               query.c_str(),
-                              static_cast<int>(query.size()),
+                              ssize(query),
                               &stmt_,
                               &tail);
     if (res != SQLITE_OK)
-    {
-        char const* zErrMsg = sqlite3_errmsg(session_.conn_);
+        throw sqlite3_soci_error(session_.conn_, "error preparing statement");
 
-        std::ostringstream ss;
-        ss << "sqlite3_statement_backend::prepare: "
-           << zErrMsg;
-        throw sqlite3_soci_error(ss.str(), res);
-    }
     databaseReady_ = true;
 }
 
@@ -95,6 +85,8 @@ void sqlite3_statement_backend::reset_if_needed()
 
 void sqlite3_statement_backend::reset()
 {
+    current_row_ = -1;
+
     int const res = sqlite3_reset(stmt_);
     if (SQLITE_OK == res)
     {
@@ -115,13 +107,13 @@ sqlite3_statement_backend::load_rowset(int totalRows)
     if (columns_.empty())
     {
         numCols = sqlite3_column_count(stmt_);
-        data_type type;
+        db_type dbtype;
         std::string name;
         for (int c = 1; c <= numCols; ++c)
-            describe_column(c, type, name);
+            describe_column(c, dbtype, name);
     }
     else
-        numCols = static_cast<int>(columns_.size());
+        numCols = ssize(columns_);
 
 
     if (!databaseReady_)
@@ -163,47 +155,62 @@ sqlite3_statement_backend::load_rowset(int totalRows)
 
                     col.isNull_ = false;
                     col.type_ = coldef.type_;
+                    col.dataType_ = coldef.dataType_;
 
-                    switch (coldef.type_)
+                    switch (coldef.dataType_)
                     {
-                        case dt_string:
-                        case dt_date:
+                        case db_string:
+                        case db_date:
                             col.buffer_.size_ = sqlite3_column_bytes(stmt_, c);
                             col.buffer_.data_ = new char[col.buffer_.size_+1];
                             memcpy(col.buffer_.data_, sqlite3_column_text(stmt_, c), col.buffer_.size_+1);
                             break;
 
-                        case dt_double:
+                        case db_double:
                             col.double_ = sqlite3_column_double(stmt_, c);
                             break;
 
-                        case dt_integer:
-                            col.int32_ = sqlite3_column_int(stmt_, c);
+                        case db_int8:
+                            col.int8_ = static_cast<int8_t>(sqlite3_column_int(stmt_, c));
                             break;
-
-                        case dt_long_long:
-                        case dt_unsigned_long_long:
+                        case db_uint8:
+                            col.uint8_ = static_cast<uint8_t>(sqlite3_column_int(stmt_, c));
+                            break;
+                        case db_int16:
+                            col.int16_ = static_cast<int16_t>(sqlite3_column_int(stmt_, c));
+                            break;
+                        case db_uint16:
+                            col.uint16_ = static_cast<uint16_t>(sqlite3_column_int(stmt_, c));
+                            break;
+                        case db_int32:
+                            col.int32_ = static_cast<int32_t>(sqlite3_column_int(stmt_, c));
+                            break;
+                        case db_uint32:
+                            col.uint32_ = static_cast<uint32_t>(sqlite3_column_int(stmt_, c));
+                            break;
+                        case db_int64:
                             col.int64_ = sqlite3_column_int64(stmt_, c);
                             break;
+                        case db_uint64:
+                            col.uint64_ = static_cast<sqlite_api::sqlite3_uint64>(sqlite3_column_int64(stmt_, c));
+                            break;
 
-                        case dt_blob:
+                        case db_blob:
                             col.buffer_.size_ = sqlite3_column_bytes(stmt_, c);
                             col.buffer_.data_ = (col.buffer_.size_ > 0 ? new char[col.buffer_.size_] : NULL);
                             memcpy(col.buffer_.data_, sqlite3_column_blob(stmt_, c), col.buffer_.size_);
                             break;
 
-                        case dt_xml:
+                        case db_xml:
                             throw soci_error("XML data type is not supported");
+                        case db_wstring:
+                            throw soci_error("Wide string data type is not supported");
                     }
                 }
             }
             else
             {
-                char const* zErrMsg = sqlite3_errmsg(session_.conn_);
-                std::ostringstream ss;
-                ss << "sqlite3_statement_backend::loadRS: "
-                   << zErrMsg;
-                throw sqlite3_soci_error(ss.str(), res);
+                throw sqlite3_soci_error(session_.conn_, "error loading row set");
             }
         }
     }
@@ -233,12 +240,8 @@ sqlite3_statement_backend::load_one()
     }
     else
     {
-        char const* zErrMsg = sqlite3_errmsg(session_.conn_);
-
-        std::ostringstream ss;
-        ss << "sqlite3_statement_backend::loadOne: "
-            << zErrMsg;
-        throw sqlite3_soci_error(ss.str(), res);
+        // There is no useful context we can add to the error message here.
+        throw sqlite3_soci_error(session_.conn_, {});
     }
     return retVal;
 }
@@ -249,64 +252,80 @@ sqlite3_statement_backend::bind_and_execute(int number)
 {
     statement_backend::exec_fetch_result retVal = ef_no_data;
 
-    long long rowsAffectedBulkTemp = 0;
+    rowsAffectedBulk_ = 0;
 
-    rowsAffectedBulk_ = -1;
-
-    int const rows = static_cast<int>(useData_.size());
-    for (int row = 0; row < rows; ++row)
+    int const rows = ssize(useData_);
+    for (current_row_ = 0; current_row_ < rows; ++current_row_)
     {
         sqlite3_reset(stmt_);
 
-        int const totalPositions = static_cast<int>(useData_[0].size());
+        int const totalPositions = ssize(useData_[0]);
         for (int pos = 1; pos <= totalPositions; ++pos)
         {
             int bindRes = SQLITE_OK;
-            const sqlite3_column &col = useData_[row][pos-1];
+            const sqlite3_column &col = useData_[current_row_][pos-1];
             if (col.isNull_)
             {
                 bindRes = sqlite3_bind_null(stmt_, pos);
             }
             else
             {
-                switch (col.type_)
+                switch (col.dataType_)
                 {
-                    case dt_string:
+                    case db_string:
                         bindRes = sqlite3_bind_text(stmt_, pos, col.buffer_.constData_, static_cast<int>(col.buffer_.size_), NULL);
                         break;
 
-                    case dt_date:
+                    case db_date:
                         bindRes = sqlite3_bind_text(stmt_, pos, col.buffer_.constData_, static_cast<int>(col.buffer_.size_), SQLITE_TRANSIENT);
                         break;
 
-                    case dt_double:
+                    case db_double:
                         bindRes = sqlite3_bind_double(stmt_, pos, col.double_);
                         break;
 
-                    case dt_integer:
-                        bindRes = sqlite3_bind_int(stmt_, pos, col.int32_);
+                    case db_int8:
+                        bindRes = sqlite3_bind_int(stmt_, pos, static_cast<int>(col.int8_));
                         break;
-
-                    case dt_long_long:
-                    case dt_unsigned_long_long:
+                    case db_uint8:
+                        bindRes = sqlite3_bind_int(stmt_, pos, static_cast<int>(col.uint8_));
+                        break;
+                    case db_int16:
+                        bindRes = sqlite3_bind_int(stmt_, pos, static_cast<int>(col.int16_));
+                        break;
+                    case db_uint16:
+                        bindRes = sqlite3_bind_int(stmt_, pos, static_cast<int>(col.uint16_));
+                        break;
+                    case db_int32:
+                        bindRes = sqlite3_bind_int(stmt_, pos, static_cast<int>(col.int32_));
+                        break;
+                    case db_uint32:
+                        bindRes = sqlite3_bind_int64(stmt_, pos, static_cast<sqlite_api::sqlite3_int64>(col.uint32_));
+                        break;
+                    case db_int64:
                         bindRes = sqlite3_bind_int64(stmt_, pos, col.int64_);
                         break;
-
-                    case dt_blob:
-                        bindRes = sqlite3_bind_blob(stmt_, pos, col.buffer_.constData_, static_cast<int>(col.buffer_.size_), NULL);
+                    case db_uint64:
+                        bindRes = sqlite3_bind_int64(stmt_, pos, static_cast<sqlite_api::sqlite3_int64>(col.int64_));
                         break;
 
-                    case dt_xml:
+                    case db_blob:
+                        // Since we don't own the buffer_ pointer we are passing here, we can't make any lifetime
+                        // guarantees other than it is currently valid. Thus, we ask SQLite to make a copy of the
+                        // underlying buffer to ensure the database can always access a valid buffer.
+                        bindRes = sqlite3_bind_blob(stmt_, pos, col.buffer_.constData_,
+                                static_cast<int>(col.buffer_.size_), SQLITE_TRANSIENT);
+                        break;
+
+                    case db_xml:
                         throw soci_error("XML data type is not supported");
+                    case db_wstring:
+                        throw soci_error("Wide string data type is not supported");
                 }
             }
 
             if (SQLITE_OK != bindRes)
-            {
-                // preserve the number of rows affected so far.
-                rowsAffectedBulk_ = rowsAffectedBulkTemp;
-                throw sqlite3_soci_error("Failure to bind on bulk operations", bindRes);
-            }
+                throw sqlite3_soci_error(session_.conn_, "failed to bind a parameter");
         }
 
         // Handle the case where there are both into and use elements
@@ -318,10 +337,9 @@ sqlite3_statement_backend::bind_and_execute(int number)
 
         databaseReady_=true; // Mark sqlite engine is ready to perform sqlite3_step
         retVal = load_one(); // execute each bound line
-        rowsAffectedBulkTemp += get_affected_rows();
+        rowsAffectedBulk_ += sqlite3_changes(session_.conn_);
     }
 
-    rowsAffectedBulk_ = rowsAffectedBulkTemp;
     return retVal;
 }
 
@@ -330,7 +348,7 @@ sqlite3_statement_backend::execute(int number)
 {
     if (stmt_ == NULL)
     {
-        throw soci_error("No sqlite statement created");
+        throw soci_error("SQLite statement wasn't created");
     }
 
     sqlite3_reset(stmt_);
@@ -371,7 +389,7 @@ long long sqlite3_statement_backend::get_affected_rows()
 
 int sqlite3_statement_backend::get_number_of_rows()
 {
-    return static_cast<int>(dataCache_.size());
+    return ssize(dataCache_);
 }
 
 std::string sqlite3_statement_backend::get_parameter_name(int index) const
@@ -408,74 +426,82 @@ int sqlite3_statement_backend::prepare_for_describe()
     return sqlite3_column_count(stmt_);
 }
 
-typedef std::map<std::string, data_type> sqlite3_data_type_map;
+typedef std::map<std::string, db_type> sqlite3_data_type_map;
 static sqlite3_data_type_map get_data_type_map()
 {
     sqlite3_data_type_map m;
 
     // Spaces are removed from decltype before looking up in this map, so we don't use them here as well
 
-    // dt_blob
-    m["blob"]               = dt_blob;
+    // db_blob
+    m["blob"]               = db_blob;
 
-    // dt_date
-    m["date"]               = dt_date;
-    m["time"]               = dt_date;
-    m["datetime"]           = dt_date;
+    // db_date
+    m["date"]               = db_date;
+    m["time"]               = db_date;
+    m["datetime"]           = db_date;
+    m["timestamp"]          = db_date;
 
-    // dt_double
-    m["decimal"]            = dt_double;
-    m["double"]             = dt_double;
-    m["doubleprecision"]    = dt_double;
-    m["float"]              = dt_double;
-    m["number"]             = dt_double;
-    m["numeric"]            = dt_double;
-    m["real"]               = dt_double;
+    // db_double
+    m["decimal"]            = db_double;
+    m["double"]             = db_double;
+    m["doubleprecision"]    = db_double;
+    m["float"]              = db_double;
+    m["number"]             = db_double;
+    m["numeric"]            = db_double;
+    m["real"]               = db_double;
 
-    // dt_integer
-    m["boolean"]            = dt_integer;
-    m["int"]                = dt_integer;
-    m["integer"]            = dt_integer;
-    m["int2"]               = dt_integer;
-    m["mediumint"]          = dt_integer;
-    m["smallint"]           = dt_integer;
-    m["tinyint"]            = dt_integer;
+    // integer types
+    m["tinyint"]            = db_int8;
 
-    // dt_long_long
-    m["bigint"]             = dt_long_long;
-    m["int8"]               = dt_long_long;
+    m["unsignedtinyint"]    = db_uint8;
 
-    // dt_string
-    m["char"]               = dt_string;
-    m["character"]          = dt_string;
-    m["clob"]               = dt_string;
-    m["nativecharacter"]    = dt_string;
-    m["nchar"]              = dt_string;
-    m["nvarchar"]           = dt_string;
-    m["text"]               = dt_string;
-    m["varchar"]            = dt_string;
-    m["varyingcharacter"]   = dt_string;
+    m["smallint"]           = db_int16;
+    m["int2"]               = db_int16;
 
-    // dt_unsigned_long_long
-    m["unsignedbigint"]   = dt_unsigned_long_long;
+    m["unsignedsmallint"]   = db_uint16;
 
+    m["boolean"]            = db_int32;
+    m["int"]                = db_int32;
+    m["integer"]            = db_int32;
+    m["mediumint"]          = db_int32;
+    m["int4"]               = db_int32;
+
+    m["unsignedint"]        = db_uint32;
+
+    m["bigint"]             = db_int64;
+    m["int8"]               = db_int64;
+
+    m["unsignedbigint"]     = db_uint64;
+
+    // db_string
+    m["char"]               = db_string;
+    m["character"]          = db_string;
+    m["clob"]               = db_string;
+    m["nativecharacter"]    = db_string;
+    m["nchar"]              = db_string;
+    m["nvarchar"]           = db_string;
+    m["text"]               = db_string;
+    m["varchar"]            = db_string;
+    m["varyingcharacter"]   = db_string;
 
     return m;
 }
 
-void sqlite3_statement_backend::describe_column(int colNum, data_type & type,
+void sqlite3_statement_backend::describe_column(int colNum,
+                                                db_type & dbtype,
                                                 std::string & columnName)
 {
     static const sqlite3_data_type_map dataTypeMap = get_data_type_map();
 
-    if (columns_.size() < (size_t)colNum)
+    if (ssize(columns_) < colNum)
         columns_.resize(colNum);
     sqlite3_column_info &coldef = columns_[colNum - 1];
 
     if (!coldef.name_.empty())
     {
         columnName = coldef.name_;
-        type = coldef.type_;
+        dbtype = coldef.dataType_;
         return;
     }
 
@@ -497,25 +523,20 @@ void sqlite3_statement_backend::describe_column(int colNum, data_type & type,
     std::string dt = declType;
 
     // remove extra characters for example "(20)" in "varchar(20)" and all spaces
-#if defined(SOCI_HAVE_CXX11) || (defined(_MSC_VER) && _MSC_VER >= 1800)
     dt.erase(std::remove_if(dt.begin(), dt.end(), [](char const c) { return std::isspace(c); }), dt.end());
 
     std::string::iterator siter = std::find_if(dt.begin(), dt.end(), [](char const c) { return !std::isalnum(c); });
-#else
-    dt.erase(std::remove_if(dt.begin(), dt.end(), std::ptr_fun(isspace)), dt.end());
-
-    std::string::iterator siter = std::find_if(dt.begin(), dt.end(), std::not1(std::ptr_fun(isalnum)));
-#endif
     if (siter != dt.end())
         dt.resize(siter - dt.begin());
 
     // do all comparisons in lower case
-    std::transform(dt.begin(), dt.end(), dt.begin(), toLowerCh);
+    dt = string_tolower(dt);
 
     sqlite3_data_type_map::const_iterator iter = dataTypeMap.find(dt);
     if (iter != dataTypeMap.end())
     {
-        coldef.type_ = type = iter->second;
+        coldef.dataType_ = dbtype = iter->second;
+        coldef.type_ = to_data_type(dbtype);
         return;
     }
 
@@ -529,20 +550,23 @@ void sqlite3_statement_backend::describe_column(int colNum, data_type & type,
     switch (sqlite3_type)
     {
     case SQLITE_INTEGER:
-        type = dt_integer;
+        dbtype = db_int32;
         break;
     case SQLITE_FLOAT:
-        type = dt_double;
+        dbtype = db_double;
         break;
     case SQLITE_BLOB:
+        dbtype = db_blob;
+        break;
     case SQLITE_TEXT:
-        type = dt_string;
+        dbtype = db_string;
         break;
     default:
-        type = dt_string;
+        dbtype = db_string;
         break;
     }
-    coldef.type_ = type;
+    coldef.dataType_ = dbtype;
+    coldef.type_ = to_data_type(dbtype);
 
     sqlite3_reset(stmt_);
 }
@@ -561,7 +585,6 @@ sqlite3_standard_use_type_backend * sqlite3_statement_backend::make_use_type_bac
 sqlite3_vector_into_type_backend *
 sqlite3_statement_backend::make_vector_into_type_backend()
 {
-    hasVectorIntoElements_ = true;
     return new sqlite3_vector_into_type_backend(*this);
 }
 
@@ -570,3 +593,28 @@ sqlite3_statement_backend::make_vector_use_type_backend()
 {
     return new sqlite3_vector_use_type_backend(*this);
 }
+
+db_type sqlite3_statement_backend::exchange_dbtype_for(db_type type) const
+{
+    // Due to SQLite not really having a type system, any integer type may hold
+    // values that could be way outside of its range.
+    // Hence, we have to be prepared to get a huge number, even if the determined
+    // db_type is e.g. db_int8.
+    // In order to do that, we ensure that we'll always select into an (u)int64,
+    // in cases where we have to select the exchange type ourselves (e.g. in rows).
+    switch (type)
+    {
+        case db_int8:
+        case db_int16:
+        case db_int32:
+        case db_int64:
+            return db_int64;
+        case db_uint8:
+        case db_uint16:
+        case db_uint32:
+        case db_uint64:
+            return db_uint64;
+        default:
+            return type;
+    }
+}
