@@ -2,7 +2,7 @@
 // Copyright (C) 2014 Vadim Zeitlin
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_SOURCE
@@ -15,7 +15,7 @@ namespace // anonymous
 {
 
 // Helper to throw from not implemented logger_impl methods.
-void throw_not_supported()
+[[noreturn]] void throw_not_supported()
 {
     throw soci_error("Legacy method not supported by this logger.");
 }
@@ -23,6 +23,21 @@ void throw_not_supported()
 } // namespace anonymous
 
 
+void logger_impl::start_query(const std::string &)
+{
+    clear_query_parameters();
+}
+
+void logger_impl::add_query_parameter(std::string name, std::string value)
+{
+    queryParams_.emplace_back(std::move(name), std::move(value));
+}
+
+void logger_impl::clear_query_parameters()
+{
+    queryParams_.clear();
+}
+
 logger_impl * logger_impl::clone() const
 {
     logger_impl * const impl = do_clone();
@@ -46,18 +61,34 @@ void logger_impl::set_stream(std::ostream *)
 std::ostream * logger_impl::get_stream() const
 {
     throw_not_supported();
-
-    return NULL;
 }
 
 std::string logger_impl::get_last_query() const
 {
     throw_not_supported();
-
-    return std::string();
 }
 
+std::string logger_impl::get_last_query_context() const
+{
+    std::string context;
 
+    bool first = true;
+    for (const query_parameter &param : queryParams_)
+    {
+        if (first)
+        {
+            first = false;
+        }
+        else
+        {
+            context += ", ";
+        }
+
+        context += ":" + param.name + "=" + param.value;
+    }
+
+    return context;
+}
 
 logger::logger(logger_impl * impl)
     : m_impl(impl)
