@@ -2,7 +2,7 @@
 // Copyright (C) 2004-2016 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_SOURCE
@@ -35,7 +35,7 @@ once_temp_type & once_temp_type::operator=(once_temp_type const & o)
     return *this;
 }
 
-once_temp_type::~once_temp_type() SOCI_NOEXCEPT_FALSE
+once_temp_type::~once_temp_type() noexcept(false)
 {
     rcst_->dec_ref();
 }
@@ -75,7 +75,7 @@ ddl_type & ddl_type::operator=(const ddl_type & d)
     return *this;
 }
 
-ddl_type::~ddl_type() SOCI_NOEXCEPT_FALSE
+ddl_type::~ddl_type() noexcept(false)
 {
     rcst_->dec_ref();
 }
@@ -86,7 +86,7 @@ void ddl_type::create_table(const std::string & tableName)
 }
 
 void ddl_type::add_column(const std::string & tableName,
-    const std::string & columnName, data_type dt,
+    const std::string & columnName, db_type dt,
     int precision, int scale)
 {
     rcst_->accumulate(s_->get_backend()->add_column(
@@ -94,7 +94,7 @@ void ddl_type::add_column(const std::string & tableName,
 }
 
 void ddl_type::alter_column(const std::string & tableName,
-    const std::string & columnName, data_type dt,
+    const std::string & columnName, db_type dt,
     int precision, int scale)
 {
     rcst_->accumulate(s_->get_backend()->alter_column(
@@ -108,8 +108,8 @@ void ddl_type::drop_column(const std::string & tableName,
             tableName, columnName));
 }
 
-ddl_type & ddl_type::column(const std::string & columnName, data_type dt,
-    int precision, int scale)
+ddl_type & ddl_type::column(const std::string & columnName, db_type dt,
+                            int precision, int scale)
 {
     if (rcst_->get_need_comma())
     {
@@ -119,7 +119,7 @@ ddl_type & ddl_type::column(const std::string & columnName, data_type dt,
     rcst_->accumulate(columnName);
     rcst_->accumulate(" ");
     rcst_->accumulate(
-        s_->get_backend()->create_column_type(dt, precision, scale));
+            s_->get_backend()->create_column_type(dt, precision, scale));
 
     rcst_->set_need_comma(true);
 
