@@ -2,7 +2,7 @@
 // Copyright (C) 2004-2008 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_SOURCE
@@ -11,12 +11,20 @@
 #include "soci/connection-pool.h"
 #include "soci/soci-backend.h"
 #include "soci/query_transformation.h"
+#include "soci/log-context.h"
 
 using namespace soci;
 using namespace soci::details;
 
-namespace // anonymous
+namespace soci
 {
+    struct schema_table_name
+    {
+        std::string schema;
+        std::string table;
+        soci::indicator ind;
+    };
+} // soci
 
 void ensureConnected(session_backend * backEnd)
 {
@@ -35,8 +43,10 @@ public:
         logStream_ = NULL;
     }
 
-    virtual void start_query(std::string const & query)
+    virtual void start_query(std::string const & query) override
     {
+        logger_impl::start_query(query);
+
         if (logStream_ != NULL)
         {
             *logStream_ << query << '\n';
@@ -45,23 +55,23 @@ public:
         lastQuery_ = query;
     }
 
-    virtual void set_stream(std::ostream * s)
+    virtual void set_stream(std::ostream * s) override
     {
         logStream_ = s;
     }
 
-    virtual std::ostream * get_stream() const
+    virtual std::ostream * get_stream() const override
     {
         return logStream_;
     }
 
-    virtual std::string get_last_query() const
+    virtual std::string get_last_query() const override
     {
         return lastQuery_;
     }
 
 private:
-    virtual logger_impl* do_clone() const
+    virtual logger_impl* do_clone() const override
     {
         return new standard_logger_impl;
     }
@@ -70,61 +80,48 @@ private:
     std::string lastQuery_;
 };
 
-} // namespace anonymous
-
 session::session()
-    : once(this), prepare(this), query_transformation_(NULL),
-      logger_(new standard_logger_impl),
-      uppercaseColumnNames_(false), backEnd_(NULL),
-      isFromPool_(false), pool_(NULL)
+    : once(this), prepare(this),
+      logger_(new standard_logger_impl)
 {
 }
 
 session::session(connection_parameters const & parameters)
-    : once(this), prepare(this), query_transformation_(NULL),
+    : once(this), prepare(this),
       logger_(new standard_logger_impl),
-      lastConnectParameters_(parameters),
-      uppercaseColumnNames_(false), backEnd_(NULL),
-      isFromPool_(false), pool_(NULL)
+      lastConnectParameters_(parameters)
 {
     open(lastConnectParameters_);
 }
 
 session::session(backend_factory const & factory,
     std::string const & connectString)
-    : once(this), prepare(this), query_transformation_(NULL),
-    logger_(new standard_logger_impl),
-      lastConnectParameters_(factory, connectString),
-      uppercaseColumnNames_(false), backEnd_(NULL),
-      isFromPool_(false), pool_(NULL)
+    : once(this), prepare(this),
+      logger_(new standard_logger_impl),
+      lastConnectParameters_(factory, connectString)
 {
     open(lastConnectParameters_);
 }
 
 session::session(std::string const & backendName,
     std::string const & connectString)
-    : once(this), prepare(this), query_transformation_(NULL),
+    : once(this), prepare(this),
       logger_(new standard_logger_impl),
-      lastConnectParameters_(backendName, connectString),
-      uppercaseColumnNames_(false), backEnd_(NULL),
-      isFromPool_(false), pool_(NULL)
+      lastConnectParameters_(backendName, connectString)
 {
     open(lastConnectParameters_);
 }
 
 session::session(std::string const & connectString)
-    : once(this), prepare(this), query_transformation_(NULL),
+    : once(this), prepare(this),
       logger_(new standard_logger_impl),
-      lastConnectParameters_(connectString),
-      uppercaseColumnNames_(false), backEnd_(NULL),
-      isFromPool_(false), pool_(NULL)
+      lastConnectParameters_(connectString)
 {
     open(lastConnectParameters_);
 }
 
 session::session(connection_pool & pool)
-    : query_transformation_(NULL),
-      logger_(new standard_logger_impl),
+    : logger_(new standard_logger_impl),
       isFromPool_(true), pool_(&pool)
 {
     poolPosition_ = pool.lease();
@@ -135,6 +132,107 @@ session::session(connection_pool & pool)
     backEnd_ = pooledSession.get_backend();
 }
 
+session::session(session && other)
+    : once(std::move(other.once)),
+      prepare(std::move(other.prepare)),
+      query_stream_(std::move(other.query_stream_)),
+      query_transformation_(std::move(other.query_transformation_)),
+      logger_(std::move(other.logger_)),
+      lastConnectParameters_(std::move(other.lastConnectParameters_)),
+      uppercaseColumnNames_(std::move(other.uppercaseColumnNames_)),
+      backEnd_(std::move(other.backEnd_)),
+      gotData_(std::move(other.gotData_)),
+      isFromPool_(std::move(other.isFromPool_)),
+      poolPosition_(std::move(other.poolPosition_)),
+      pool_(std::move(other.pool_))
+{
+    if (!isFromPool_)
+    {
+        // If 'other' session was from a pool, 'once' and 'prepare'
+        // will have been already setup to point at the pooled session above.
+        // Otherwise reset them to reference 'this'.
+        once.set_session(this);
+        prepare.set_session(this);
+    }
+
+    other.reset_after_move ();
+}
+
+session& session::operator=(session && other)
+{
+    if (this != &other)
+    {
+        if (isFromPool_)
+        {
+            pool_->give_back(poolPosition_);
+        }
+        else if (backEnd_ != other.backEnd_)
+        {
+            delete backEnd_;
+        }
+
+        if (other.isFromPool_)
+        {
+            // If 'other' session was from a pool, 'once' and 'prepare'
+            // will have been already setup reference the pooled session above.
+            // Otherwise leave them alone and let them continue referencing 'this'.
+            once = std::move(other.once);
+            prepare = std::move(other.prepare);
+        }
+
+        query_stream_ = std::move(other.query_stream_);
+        query_transformation_ = std::move(other.query_transformation_);
+        logger_ = std::move(other.logger_);
+        lastConnectParameters_ = std::move(other.lastConnectParameters_);
+        uppercaseColumnNames_ = std::move(other.uppercaseColumnNames_);
+        backEnd_ = std::move(other.backEnd_);
+        gotData_ = std::move(other.gotData_);
+        isFromPool_ = std::move(other.isFromPool_);
+        poolPosition_ = std::move(other.poolPosition_);
+        pool_ = std::move(other.pool_);
+
+        other.reset_after_move();
+    }
+
+    return *this;
+}
+
+void session::reset_after_move()
+{
+    backEnd_ = nullptr;
+    gotData_ = false;
+    isFromPool_ = false;
+    poolPosition_ = 0;
+    pool_ = nullptr;
+}
+
+schema_table_name& session::alloc_schema_table_name(const std::string & tableName)
+{
+    std::size_t dot_pos = tableName.find( '.' );
+    schema_table_name stn;
+
+    if (dot_pos == std::string::npos)
+    {
+        stn.ind   = soci::i_null;
+        stn.table = tableName;
+    }
+    else
+    {
+        stn.ind    = soci::i_ok;
+        stn.schema = tableName.substr(0, dot_pos);
+        stn.table  = tableName.substr(dot_pos + 1);
+    }
+
+    // Store the memory at the front of the list of schema_table_names.
+    // This memory is stored to allow for the schema, table, and ind
+    // parameters to be stored for the lifetime of the prepare_temp_type
+    // created in prepare_column_descriptions.
+    // These will remain in memory until the session object is deleted.
+    schema_table_name_.push_front(stn);
+
+    return schema_table_name_.front();
+}
+
 session::~session()
 {
     if (isFromPool_)
@@ -143,7 +241,6 @@ session::~session()
     }
     else
     {
-        delete query_transformation_;
         delete backEnd_;
     }
 }
@@ -237,7 +334,7 @@ void session::reconnect()
     }
 }
 
-bool session::is_connected() const SOCI_NOEXCEPT
+bool session::is_connected() const noexcept
 {
     try
     {
@@ -306,16 +403,15 @@ std::string session::get_query() const
 }
 
 
-void session::set_query_transformation_(cxx_details::auto_ptr<details::query_transformation_function>& qtf)
+void session::set_query_transformation_(std::unique_ptr<details::query_transformation_function>&& qtf)
 {
     if (isFromPool_)
     {
-        pool_->at(poolPosition_).set_query_transformation_(qtf);
+        pool_->at(poolPosition_).set_query_transformation_(std::move(qtf));
     }
     else
     {
-        delete query_transformation_;
-        query_transformation_= qtf.release();
+        query_transformation_= std::move(qtf);
     }
 }
 
@@ -367,6 +463,16 @@ std::ostream * session::get_log_stream() const
     }
 }
 
+void session::set_query_context_logging_mode(log_context ctx)
+{
+    query_ctx_logging_mode_ = ctx;
+}
+
+log_context session::get_query_context_logging_mode() const
+{
+    return query_ctx_logging_mode_;
+}
+
 void session::log_query(std::string const & query)
 {
     if (isFromPool_)
@@ -379,6 +485,30 @@ void session::log_query(std::string const & query)
     }
 }
 
+void session::clear_query_parameters()
+{
+    if (isFromPool_)
+    {
+        pool_->at(poolPosition_).clear_query_parameters();
+    }
+    else
+    {
+        logger_.clear_query_parameters();
+    }
+}
+
+void session::add_query_parameter(std::string name, std::string value)
+{
+    if (isFromPool_)
+    {
+        pool_->at(poolPosition_).add_query_parameter(std::move(name), std::move(value));
+    }
+    else
+    {
+        logger_.add_query_parameter(std::move(name), std::move(value));
+    }
+}
+
 std::string session::get_last_query() const
 {
     if (isFromPool_)
@@ -391,6 +521,18 @@ std::string session::get_last_query() const
     }
 }
 
+std::string session::get_last_query_context() const
+{
+    if (isFromPool_)
+    {
+        return pool_->at(poolPosition_).get_last_query_context();
+    }
+    else
+    {
+        return logger_.get_last_query_context();
+    }
+}
+
 void session::set_got_data(bool gotData)
 {
     if (isFromPool_)
@@ -471,7 +613,17 @@ details::prepare_temp_type session::prepare_column_descriptions(std::string & ta
 {
     ensureConnected(backEnd_);
 
-    return prepare << backEnd_->get_column_descriptions_query(), use(table_name, "t");
+    std::string column_description_query = backEnd_->get_column_descriptions_query();
+    if (column_description_query.find(":s") == std::string::npos)
+    {
+        return prepare << column_description_query, use(table_name, "t");
+    }
+    else
+    {
+        schema_table_name& stn = alloc_schema_table_name(table_name);
+
+        return prepare << column_description_query, use(stn.schema, stn.ind, "s"), use(stn.table, "t");
+    }
 }
 
 ddl_type session::create_table(const std::string & tableName)
@@ -499,7 +651,7 @@ void session::truncate_table(const std::string & tableName)
 }
 
 ddl_type session::add_column(const std::string & tableName,
-    const std::string & columnName, data_type dt,
+    const std::string & columnName, db_type dt,
     int precision, int scale)
 {
     ddl_type ddl(*this);
@@ -510,7 +662,7 @@ ddl_type session::add_column(const std::string & tableName,
 }
 
 ddl_type session::alter_column(const std::string & tableName,
-    const std::string & columnName, data_type dt,
+    const std::string & columnName, db_type dt,
     int precision, int scale)
 {
     ddl_type ddl(*this);
