@@ -2,23 +2,44 @@
 // Copyright (C) 2004-2008 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #define SOCI_SOURCE
+#include "soci/blob.h"
+#include "soci/blob-exchange.h"
 #include "soci/statement.h"
 #include "soci/session.h"
 #include "soci/into-type.h"
 #include "soci/use-type.h"
 #include "soci/values.h"
 #include "soci-compiler.h"
+#include "soci-ssize.h"
+#include "soci/log-context.h"
 #include <ctime>
 #include <cctype>
+#include <cstdint>
+#include <string>
+#include <sstream>
 
 using namespace soci;
 using namespace soci::details;
 
 
+std::string get_name(const details::use_type_base &param, std::size_t position,
+        const statement_backend *backend)
+{
+    // Use the name specified in the "use()" call if any,
+    // otherwise get the name of the matching parameter from
+    // the query itself, as parsed by the backend.
+    std::string name = param.get_name();
+    if (backend && name.empty())
+        name = backend->get_parameter_name(static_cast<int>(position));
+
+    return name.empty() ? std::to_string(position + 1) : name;
+}
+
+
 statement_impl::statement_impl(session & s)
     : session_(s), refCount_(1), row_(0),
       fetchSize_(1), initialFetchSize_(1),
@@ -85,7 +106,7 @@ void statement_impl::bind(values & values)
             {
                 // positional use element
 
-                int position = static_cast<int>(uses_.size());
+                int position = ssize(uses_);
                 (*it)->bind(*this, position);
                 uses_.push_back(*it);
                 indicators_.push_back(values.indicators_[cnt]);
@@ -111,7 +132,7 @@ void statement_impl::bind(values & values)
                     }
                     else
                     {
-                        int position = static_cast<int>(uses_.size());
+                        int position = ssize(uses_);
                         (*it)->bind(*this, position);
                         uses_.push_back(*it);
                         indicators_.push_back(values.indicators_[cnt]);
@@ -173,6 +194,7 @@ void statement_impl::bind_clean_up()
         delete indicators_[i];
         indicators_[i] = NULL;
     }
+    indicators_.clear();
 
     row_ = NULL;
     alreadyDescribed_ = false;
@@ -228,15 +250,6 @@ void statement_impl::define_and_bind()
     }
 }
 
-void statement_impl::define_for_row()
-{
-    std::size_t const isize = intosForRow_.size();
-    for (std::size_t i = 0; i != isize; ++i)
-    {
-        intosForRow_[i]->define(*this, definePositionForRow_);
-    }
-}
-
 void statement_impl::undefine_and_bind()
 {
     std::size_t const isize = intos_.size();
@@ -294,7 +307,6 @@ bool statement_impl::execute(bool withDataExchange)
         if (row_ != NULL && alreadyDescribed_ == false)
         {
             describe();
-            define_for_row();
         }
 
         int num = 0;
@@ -318,6 +330,16 @@ bool statement_impl::execute(bool withDataExchange)
 
         statement_backend::exec_fetch_result res = backEnd_->execute(num);
 
+        // another hack related to description: the first call to describe()
+        // above may not have done anything if we didn't have the correct
+        // number of columns before calling execute() as happens with at least
+        // the ODBC backend for some complex queries (see #1151), so call it
+        // again in this case
+        if (row_ != NULL && alreadyDescribed_ == false)
+        {
+            describe();
+        }
+
         bool gotData = false;
 
         if (res == statement_backend::ef_success)
@@ -568,13 +590,32 @@ void statement_impl::pre_fetch()
     }
 }
 
-void statement_impl::pre_use()
+void statement_impl::do_add_query_parameters()
 {
+    std::size_t const usize = uses_.size();
+    for (std::size_t i = 0; i != usize; ++i)
+    {
+        std::string name = get_name(*uses_[i], i, backEnd_);
+        std::stringstream value;
+        uses_[i]->dump_value(value, backEnd_->get_row_to_dump());
+        session_.add_query_parameter(std::move(name), value.str());
+    }
+}
+
+void statement_impl::pre_use()
+{
+    session_.clear_query_parameters();
+
     std::size_t const usize = uses_.size();
     for (std::size_t i = 0; i != usize; ++i)
     {
         uses_[i]->pre_use();
     }
+
+    if (session_.get_query_context_logging_mode() == log_context::always)
+    {
+        do_add_query_parameters();
+    }
 }
 
 void statement_impl::post_fetch(bool gotData, bool calledFromFetch)
@@ -628,86 +669,152 @@ namespace details
 // Map data_types to stock types for dynamic result set support
 
 template<>
-void statement_impl::bind_into<dt_string>()
+void statement_impl::bind_into<db_string>()
 {
     into_row<std::string>();
 }
 
 template<>
-void statement_impl::bind_into<dt_double>()
+void statement_impl::bind_into<db_wstring>()
+{
+    into_row<std::wstring>();
+}
+
+template<>
+void statement_impl::bind_into<db_double>()
 {
     into_row<double>();
 }
 
 template<>
-void statement_impl::bind_into<dt_integer>()
+void statement_impl::bind_into<db_int8>()
 {
-    into_row<int>();
+    into_row<int8_t>();
 }
 
 template<>
-void statement_impl::bind_into<dt_long_long>()
+void statement_impl::bind_into<db_uint8>()
 {
-    into_row<long long>();
+    into_row<uint8_t>();
 }
 
 template<>
-void statement_impl::bind_into<dt_unsigned_long_long>()
+void statement_impl::bind_into<db_int16>()
 {
-    into_row<unsigned long long>();
+    into_row<int16_t>();
 }
 
 template<>
-void statement_impl::bind_into<dt_date>()
+void statement_impl::bind_into<db_uint16>()
+{
+    into_row<uint16_t>();
+}
+
+template<>
+void statement_impl::bind_into<db_int32>()
+{
+    into_row<int32_t>();
+}
+
+template<>
+void statement_impl::bind_into<db_uint32>()
+{
+    into_row<uint32_t>();
+}
+
+template<>
+void statement_impl::bind_into<db_int64>()
+{
+    into_row<int64_t>();
+}
+
+template<>
+void statement_impl::bind_into<db_uint64>()
+{
+    into_row<uint64_t>();
+}
+
+template<>
+void statement_impl::bind_into<db_date>()
 {
     into_row<std::tm>();
 }
 
+template<>
+void statement_impl::bind_into<db_blob>()
+{
+    into_row<blob>();
+}
+
 void statement_impl::describe()
 {
     row_->clean_up();
 
     int const numcols = backEnd_->prepare_for_describe();
+    if (!numcols)
+    {
+        // Return without setting alreadyDescribed_ to true, we'll be called
+        // again in this case.
+        return;
+    }
+
     for (int i = 1; i <= numcols; ++i)
     {
-        data_type dtype;
+        db_type dbtype;
         std::string columnName;
 
-        backEnd_->describe_column(i, dtype, columnName);
+        backEnd_->describe_column(i, dbtype, columnName);
 
         column_properties props;
         props.set_name(columnName);
-        props.set_data_type(dtype);
+        props.set_db_type(dbtype);
+        props.set_data_type(backEnd_->to_data_type(dbtype));
 
-        switch (dtype)
+        switch (backEnd_->exchange_dbtype_for(dbtype))
         {
-        case dt_string:
-            bind_into<dt_string>();
+        case db_string:
+        case db_xml:
+            bind_into<db_string>();
             break;
-        case dt_blob:
-            bind_into<dt_string>();
+        case db_wstring:
+            bind_into<db_wstring>();
             break;
-        case dt_xml:
-            bind_into<dt_string>();
+        case db_blob:
+            bind_into<db_blob>();
             break;
-        case dt_double:
-            bind_into<dt_double>();
+        case db_double:
+            bind_into<db_double>();
             break;
-        case dt_integer:
-            bind_into<dt_integer>();
+        case db_int8:
+            bind_into<db_int8>();
             break;
-        case dt_long_long:
-            bind_into<dt_long_long>();
+        case db_uint8:
+            bind_into<db_uint8>();
             break;
-        case dt_unsigned_long_long:
-            bind_into<dt_unsigned_long_long>();
+        case db_int16:
+            bind_into<db_int16>();
             break;
-        case dt_date:
-            bind_into<dt_date>();
+        case db_uint16:
+            bind_into<db_uint16>();
+            break;
+        case db_int32:
+            bind_into<db_int32>();
+            break;
+        case db_uint32:
+            bind_into<db_uint32>();
+            break;
+        case db_int64:
+            bind_into<db_int64>();
+            break;
+        case db_uint64:
+            bind_into<db_uint64>();
+            break;
+        case db_date:
+            bind_into<db_date>();
             break;
         default:
             std::ostringstream msg;
-            msg << "db column type " << dtype
+            msg << "db column type " << dbtype
                 <<" not supported for dynamic selects"<<std::endl;
             throw soci_error(msg.str());
         }
@@ -715,6 +822,14 @@ void statement_impl::describe()
     }
 
     alreadyDescribed_ = true;
+
+    // Calling bind_into() above could have added row into elements, so
+    // initialize them.
+    std::size_t const isize = intosForRow_.size();
+    for (std::size_t i = 0; i != isize; ++i)
+    {
+        intosForRow_[i]->define(*this, definePositionForRow_);
+    }
 }
 
 } // namespace details
@@ -753,24 +868,32 @@ void statement_impl::dec_ref()
 standard_into_type_backend *
 statement_impl::make_into_type_backend()
 {
+    backEnd_->hasIntoElements_ = true;
+
     return backEnd_->make_into_type_backend();
 }
 
 standard_use_type_backend *
 statement_impl::make_use_type_backend()
 {
+    backEnd_->hasUseElements_ = true;
+
     return backEnd_->make_use_type_backend();
 }
 
 vector_into_type_backend *
 statement_impl::make_vector_into_type_backend()
 {
+    backEnd_->hasVectorIntoElements_ = true;
+
     return backEnd_->make_vector_into_type_backend();
 }
 
 vector_use_type_backend *
 statement_impl::make_vector_use_type_backend()
 {
+    backEnd_->hasVectorUseElements_ = true;
+
     return backEnd_->make_vector_use_type_backend();
 }
 
@@ -788,34 +911,17 @@ statement_impl::rethrow_current_exception_with_context(char const* operation)
             std::ostringstream oss;
             oss << "while " << operation << " \"" << query_ << "\"";
 
-            if (!uses_.empty())
+            if (!uses_.empty() && session_.get_query_context_logging_mode() != log_context::never)
             {
-                oss << " with ";
+                // We have to clear previously cached query parameters as different backends behave differently
+                // in whether they error before or after the caching has happened. Therefore, we can't rely on
+                // the parameters to be or to not be cached at the point of error in a cross-backend way (that
+                // works for all kinds of errors).
+                session_.clear_query_parameters();
 
-                std::size_t const usize = uses_.size();
-                for (std::size_t i = 0; i != usize; ++i)
-                {
-                    if (i != 0)
-                        oss << ", ";
+                do_add_query_parameters();
 
-                    details::use_type_base const& u = *uses_[i];
-
-                    // Use the name specified in the "use()" call if any,
-                    // otherwise get the name of the matching parameter from
-                    // the query itself, as parsed by the backend.
-                    std::string name = u.get_name();
-                    if (name.empty())
-                        name = backEnd_->get_parameter_name(static_cast<int>(i));
-
-                    oss << ":";
-                    if (!name.empty())
-                        oss << name;
-                    else
-                        oss << (i + 1);
-                    oss << "=";
-
-                    u.dump_value(oss);
-                }
+                oss << " with " << session_.get_last_query_context();
             }
 
             e.add_context(oss.str());
@@ -824,3 +930,12 @@ statement_impl::rethrow_current_exception_with_context(char const* operation)
         throw;
     }
 }
+
+template<>
+void statement_impl::into_row<blob>()
+{
+    blob * b = new blob(session_);
+    indicator * ind = new indicator(i_ok);
+    row_->add_holder(b, ind);
+    exchange_for_row(into(*b, *ind));
+}
