@@ -2,7 +2,7 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, Rafal Bobrowski
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 //
 
@@ -11,13 +11,16 @@
 #include "soci-compiler.h"
 #include "firebird/error-firebird.h"            // soci::details::Firebird::throw_iscerror()
 #include "firebird/common.h"
-#include "common-tests.h"
+#include "test-assert.h"
+#include "test-context.h"
 #include <iostream>
 #include <string>
 #include <ctime>
 #include <cstring>
 #include <cmath>
 
+#include <catch.hpp>
+
 using namespace soci;
 
 std::string connectString;
@@ -500,142 +503,6 @@ TEST_CASE("Firebird bulk operations", "[firebird][bulk]")
     sql << "drop table test6";
 }
 
-// blob test
-TEST_CASE("Firebird blobs", "[firebird][blob]")
-{
-    soci::session sql(backEnd, connectString);
-
-    try
-    {
-        sql << "drop table test7";
-    }
-    catch (std::runtime_error &)
-    {} // ignore if error
-
-    sql << "create table test7(id integer, img blob)";
-    sql.commit();
-
-    sql.begin();
-    {
-        // verify empty blob
-        blob b(sql);
-        indicator ind;
-
-        sql << "insert into test7(id, img) values(1,?)", use(b);
-        sql << "select img from test7 where id = 1", into(b, ind);
-
-        CHECK(ind == i_ok);
-        CHECK(b.get_len() == 0);
-
-        sql << "delete from test7";
-    }
-
-    {
-        // create a new blob
-        blob b(sql);
-
-        char str1[] = "Hello";
-        b.write(0, str1, strlen(str1));
-
-        char str2[20];
-        std::size_t i = b.read(3, str2, 2);
-        str2[i] = '\0';
-        CHECK(str2[0] == 'l');
-        CHECK(str2[1] == 'o');
-        CHECK(str2[2] == '\0');
-
-        char str3[] = ", Firebird!";
-        b.append(str3, strlen(str3));
-
-        sql << "insert into test7(id, img) values(1,?)", use(b);
-    }
-
-    {
-        // read & update blob
-        blob b(sql);
-
-        sql << "select img from test7 where id = 1", into(b);
-
-        std::vector<char> text(b.get_len());
-        b.read(0, &text[0], b.get_len());
-        CHECK(strncmp(&text[0], "Hello, Firebird!", b.get_len()) == 0);
-
-        char str1[] = "FIREBIRD";
-        b.write(7, str1, strlen(str1));
-
-        // after modification blob must be written to database
-        sql << "update test7 set img=? where id=1", use(b);
-    }
-
-    {
-        // read blob from database, modify and write to another record
-        blob b(sql);
-
-        sql << "select img from test7 where id = 1", into(b);
-
-        std::vector<char> text(b.get_len());
-        b.read(0, &text[0], b.get_len());
-
-        char str1[] = "HELLO";
-        b.write(0, str1, strlen(str1));
-
-        b.read(0, &text[0], b.get_len());
-        CHECK(strncmp(&text[0], "HELLO, FIREBIRD!", b.get_len()) == 0);
-
-        b.trim(5);
-        sql << "insert into test7(id, img) values(2,?)", use(b);
-    }
-
-    {
-        blob b(sql);
-        statement st = (sql.prepare << "select img from test7", into(b));
-
-        st.execute();
-
-        st.fetch();
-        std::vector<char> text(b.get_len());
-        b.read(0, &text[0], b.get_len());
-        CHECK(strncmp(&text[0], "Hello, FIREBIRD!", b.get_len()) == 0);
-
-        st.fetch();
-        text.resize(b.get_len());
-        b.read(0, &text[0], b.get_len());
-        CHECK(strncmp(&text[0], "HELLO", b.get_len()) == 0);
-    }
-
-    {
-        // delete blob
-        blob b(sql);
-        indicator ind=i_null;
-        sql << "update test7 set img=? where id = 1", use(b, ind);
-
-        sql << "select img from test7 where id = 2", into(b, ind);
-        CHECK(ind==i_ok);
-
-        sql << "select img from test7 where id = 1", into(b, ind);
-        CHECK(ind==i_null);
-    }
-
-    {
-        //create large blob
-        const int blobSize = 65536; //max segment size is 65535(unsigned short)
-        std::vector<char> data(blobSize);
-        blob b(sql);
-        b.write(0, data.data(), blobSize);
-        sql << "insert into test7(id, img) values(3,?)", use(b);
-
-        //now read blob back from database and make sure it has correct content and size
-        blob br(sql);
-        sql << "select img from test7 where id = 3", into(br);
-        std::vector<char> data2(br.get_len());
-        if(br.get_len()>0)
-            br.read(0, data2.data(), br.get_len());
-        CHECK(data == data2);
-    }
-
-    sql << "drop table test7";
-}
-
 // named parameters
 TEST_CASE("Firebird named parameters", "[firebird][named-params]")
 {
@@ -773,8 +640,11 @@ TEST_CASE("Firebird dynamic binding", "[firebird][dynamic]")
     CHECK(r.get_properties(2).get_name() == "NTEST");
 
     CHECK(r.get_properties(0).get_data_type() == dt_integer);
+    CHECK(r.get_properties(0).get_db_type() == db_int32);
     CHECK(r.get_properties(1).get_data_type() == dt_string);
+    CHECK(r.get_properties(1).get_db_type() == db_string);
     CHECK(r.get_properties(2).get_data_type() == dt_double);
+    CHECK(r.get_properties(2).get_db_type() == db_double);
 
     // get properties by name
     CHECK(r.get_properties("ID").get_name() == "ID");
@@ -782,8 +652,11 @@ TEST_CASE("Firebird dynamic binding", "[firebird][dynamic]")
     CHECK(r.get_properties("NTEST").get_name() == "NTEST");
 
     CHECK(r.get_properties("ID").get_data_type() == dt_integer);
+    CHECK(r.get_properties("ID").get_db_type() == db_int32);
     CHECK(r.get_properties("MSG").get_data_type() == dt_string);
+    CHECK(r.get_properties("MSG").get_db_type() == db_string);
     CHECK(r.get_properties("NTEST").get_data_type() == dt_double);
+    CHECK(r.get_properties("NTEST").get_db_type() == db_double);
 
     // get values by position
     CHECK(r.get<int>(0) == 1);
@@ -803,10 +676,10 @@ TEST_CASE("Firebird dynamic binding", "[firebird][dynamic]")
     // verify default values
     CHECK(tests::are_doubles_exactly_equal(r.get<double>("NTEST", 2), 2));
 
-    CHECK_THROWS_AS(r.get<double>("NTEST"), soci_error&);
+    CHECK_THROWS_AS(r.get<double>("NTEST"), soci_error);
 
     // verify exception thrown on invalid get<>
-    CHECK_THROWS_AS(r.get<std::string>(0), std::bad_cast&);
+    CHECK_THROWS_AS(r.get<std::string>(0), std::bad_cast);
 
     sql << "drop table test9";
 }
@@ -1190,18 +1063,24 @@ TEST_CASE("Firebird decimals as strings", "[firebird][decimal][string]")
     // get properties by position
     CHECK(r.get_properties(0).get_name() == "NTEST1");
     CHECK(r.get_properties(0).get_data_type() == dt_string);
+    CHECK(r.get_properties(0).get_db_type() == db_string);
     CHECK(r.get_properties(1).get_name() == "NTEST2");
     CHECK(r.get_properties(1).get_data_type() == dt_string);
+    CHECK(r.get_properties(1).get_db_type() == db_string);
     CHECK(r.get_properties(2).get_name() == "NTEST3");
     CHECK(r.get_properties(2).get_data_type() == dt_string);
+    CHECK(r.get_properties(2).get_db_type() == db_string);
 
     // get properties by name
     CHECK(r.get_properties("NTEST1").get_name() == "NTEST1");
     CHECK(r.get_properties("NTEST1").get_data_type() == dt_string);
+    CHECK(r.get_properties("NTEST1").get_db_type() == db_string);
     CHECK(r.get_properties("NTEST2").get_name() == "NTEST2");
     CHECK(r.get_properties("NTEST2").get_data_type() == dt_string);
+    CHECK(r.get_properties("NTEST2").get_db_type() == db_string);
     CHECK(r.get_properties("NTEST3").get_name() == "NTEST3");
     CHECK(r.get_properties("NTEST3").get_data_type() == dt_string);
+    CHECK(r.get_properties("NTEST3").get_db_type() == db_string);
 
     // get values by position
     CHECK(r.get<std::string>(0) == d_str1);
@@ -1231,8 +1110,8 @@ struct TableCreator1 : public tests::table_creator_base
             : tests::table_creator_base(sql)
     {
         sql << "create table soci_test(id integer, val integer, c char, "
-        "str varchar(20), sh smallint, ul bigint, d double precision, "
-        "num76 numeric(7,6), "
+        "str varchar(20), sh smallint, ll bigint, ul bigint, "
+        "d double precision, num76 numeric(7,6), "
         "tm timestamp, i1 integer, i2 integer, i3 integer, name varchar(20))";
         sql.commit();
         sql.begin();
@@ -1285,6 +1164,17 @@ struct TableCreatorCLOB : public tests::table_creator_base
     }
 };
 
+struct TableCreatorBLOB : public tests::table_creator_base
+{
+    TableCreatorBLOB(soci::session & sql)
+            : tests::table_creator_base(sql)
+    {
+        sql << "create table soci_test(id integer, b blob)";
+        sql.commit();
+        sql.begin();
+    }
+};
+
 struct TableCreatorXML : public tests::table_creator_base
 {
     TableCreatorXML(soci::session & sql)
@@ -1296,94 +1186,82 @@ struct TableCreatorXML : public tests::table_creator_base
     }
 };
 
-class test_context : public tests::test_context_base
+class test_context : public tests::test_context_common
 {
     public:
-        test_context(backend_factory const &backEnd,
-                    std::string const &connectString)
-                : test_context_base(backEnd, connectString)
-        {}
+        test_context() = default;
 
-        tests::table_creator_base* table_creator_1(soci::session& s) const SOCI_OVERRIDE
+        std::string get_example_connection_string() const override
+        {
+            return "service=/usr/local/firebird/db/test.fdb user=SYSDBA password=masterkey";
+        }
+
+        std::string get_backend_name() const override
+        {
+            return "firebird";
+        }
+
+        tests::table_creator_base* table_creator_1(soci::session& s) const override
         {
             return new TableCreator1(s);
         }
 
-        tests::table_creator_base* table_creator_2(soci::session& s) const SOCI_OVERRIDE
+        tests::table_creator_base* table_creator_2(soci::session& s) const override
         {
             return new TableCreator2(s);
         }
 
-        tests::table_creator_base* table_creator_3(soci::session& s) const SOCI_OVERRIDE
+        tests::table_creator_base* table_creator_3(soci::session& s) const override
         {
             return new TableCreator3(s);
         }
 
-        tests::table_creator_base* table_creator_4(soci::session& s) const SOCI_OVERRIDE
+        tests::table_creator_base* table_creator_4(soci::session& s) const override
         {
             return new TableCreator4(s);
         }
 
-        tests::table_creator_base* table_creator_clob(soci::session& s) const SOCI_OVERRIDE
+        tests::table_creator_base* table_creator_clob(soci::session& s) const override
         {
             return new TableCreatorCLOB(s);
         }
 
-        tests::table_creator_base* table_creator_xml(soci::session& s) const SOCI_OVERRIDE
+
+        tests::table_creator_base* table_creator_blob(soci::session& s) const override
+        {
+            return new TableCreatorBLOB(s);
+        }
+
+        tests::table_creator_base* table_creator_xml(soci::session& s) const override
         {
             return new TableCreatorXML(s);
         }
 
-        std::string to_date_time(std::string const &datdt_string) const SOCI_OVERRIDE
+        std::string to_date_time(std::string const &datdt_string) const override
         {
             return "'" + datdt_string + "'";
         }
 
-        void on_after_ddl(soci::session& sql) const SOCI_OVERRIDE
+        bool has_uint64_storage_bug() const override
+        {
+            // Firebird does not support unsigned integer types.
+            // We're using Firebird 3, which does not yet support a data
+            // type that can correctly store a UINT64_MAX. The biggest
+            // numeric data type available is numeric(18,0).
+            // Firebird 4 introduces the data type int128 and numeric(36,0),
+            // which will be sufficient for that in the future.
+            return true;
+        }
+
+        void on_after_ddl(soci::session& sql) const override
         {
             sql.commit();
         }
 
-        std::string sql_length(std::string const& s) const SOCI_OVERRIDE
+        std::string sql_length(std::string const& s) const override
         {
             return "char_length(" + s + ")";
         }
 };
 
-
-int main(int argc, char** argv)
-{
-
-#ifdef _MSC_VER
-    // Redirect errors, unrecoverable problems, and assert() failures to STDERR,
-    // instead of debug message window.
-    // This hack is required to run assert()-driven tests by Buildbot.
-    // NOTE: Comment this 2 lines for debugging with Visual C++ debugger to catch assertions inside.
-    _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE);
-    _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDERR);
-#endif //_MSC_VER
-
-    if (argc >= 2)
-    {
-        connectString = argv[1];
-
-        // Replace the connect string with the process name to ensure that
-        // CATCH uses the correct name in its messages.
-        argv[1] = argv[0];
-
-        argc--;
-        argv++;
-    }
-    else
-    {
-        std::cout << "usage: " << argv[0]
-            << " connectstring [test-arguments...]\n"
-            << "example: " << argv[0]
-            << " \"service=/usr/local/firebird/db/test.fdb user=SYSDBA password=masterkey\"\n";
-        return EXIT_FAILURE;
-    }
-
-    test_context tc(backEnd, connectString);
-
-    return Catch::Session().run(argc, argv);
-}
+test_context tc_firebird;
