@@ -3,10 +3,13 @@
 // MySQL backend copyright (C) 2006 Pawel Aleksander Fedorynski
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #include "soci/soci.h"
+
+#include "soci-compiler.h"
+#include "soci-ssize.h"
 #include "soci/mysql/soci-mysql.h"
 #include "mysql/test-mysql.h"
 #include <string.h>
@@ -20,6 +23,9 @@
 #include <cstdlib>
 #include <mysqld_error.h>
 #include <errmsg.h>
+#include <cstdint>
+
+#include <catch.hpp>
 
 std::string connectString;
 backend_factory const &backEnd = *soci::factory_mysql();
@@ -368,112 +374,59 @@ TEST_CASE("MySQL datetime", "[mysql][datetime]")
     CHECK(t.tm_sec == 52);
 }
 
-// TEXT and BLOB types support test.
-TEST_CASE("MySQL text and blob", "[mysql][text][blob]")
+// TEXT type support test.
+TEST_CASE("MySQL text", "[mysql][text]")
 {
     soci::session sql(backEnd, connectString);
     std::string a("asdfg\0hjkl", 10);
-    std::string b("lkjhg\0fd\0\0sa\0", 13);
-    std::string c("\\0aa\\0bb\\0cc\\0", 10);
     // The maximum length for TEXT and BLOB is 65536.
     std::string x(60000, 'X');
-    std::string y(60000, 'Y');
-    // The default max_allowed_packet value for a MySQL server is 1M,
-    // so let's limit ourselves to 800k, even though the maximum length
-    // for LONGBLOB is 4G.
-    std::string z(800000, 'Z');
 
-    sql << "create table soci_test (id int, text_value text, "
-        "blob_value blob, longblob_value longblob)";
-    sql << "insert into soci_test values (1, \'foo\', \'bar\', \'baz\')";
+    sql << "create table soci_test (id int, text_value text)";
+    sql << "insert into soci_test values (1, \'foo\')";
     sql << "insert into soci_test "
-        << "values (2, \'qwerty\\0uiop\', \'zxcv\\0bnm\', "
-        << "\'qwerty\\0uiop\\0zxcvbnm\\0\')";
-    sql << "insert into soci_test values (3, :a, :b, :c)",
-           use(a), use(b), use(c);
-    sql << "insert into soci_test values (4, :x, :y, :z)",
-           use(x), use(y), use(z);
+        << "values (2, \'qwerty\\0uiop\')";
+    sql << "insert into soci_test values (3, :a)",
+           use(a);
+    sql << "insert into soci_test values (4, :x)",
+           use(x);
 
     std::vector<std::string> text_vec(100);
-    std::vector<std::string> blob_vec(100);
-    std::vector<std::string> longblob_vec(100);
-    sql << "select text_value, blob_value, longblob_value "
-        << "from soci_test order by id",
-           into(text_vec), into(blob_vec), into(longblob_vec);
+    sql << "select text_value from soci_test order by id", into(text_vec);
     REQUIRE(text_vec.size() == 4);
-    REQUIRE(blob_vec.size() == 4);
-    REQUIRE(longblob_vec.size() == 4);
     CHECK(text_vec[0] == "foo");
-    CHECK(blob_vec[0] == "bar");
-    CHECK(longblob_vec[0] == "baz");
     CHECK(text_vec[1] == std::string("qwerty\0uiop", 11));
-    CHECK(blob_vec[1] == std::string("zxcv\0bnm", 8));
-    CHECK(longblob_vec[1] == std::string("qwerty\0uiop\0zxcvbnm\0", 20));
     CHECK(text_vec[2] == a);
-    CHECK(blob_vec[2] == b);
-    CHECK(longblob_vec[2] == c);
     CHECK(text_vec[3] == x);
-    CHECK(blob_vec[3] == y);
-    CHECK(longblob_vec[3] == z);
 
-    std::string text, blob, longblob;
-    sql << "select text_value, blob_value, longblob_value "
-        << "from soci_test where id = 1",
-           into(text), into(blob), into(longblob);
+    std::string text;
+    sql << "select text_value from soci_test where id = 1", into(text);
     CHECK(text == "foo");
-    CHECK(blob == "bar");
-    CHECK(longblob == "baz");
-    sql << "select text_value, blob_value, longblob_value "
-        << "from soci_test where id = 2",
-           into(text), into(blob), into(longblob);
+    sql << "select text_value from soci_test where id = 2", into(text);
     CHECK(text == std::string("qwerty\0uiop", 11));
-    CHECK(blob == std::string("zxcv\0bnm", 8));
-    CHECK(longblob == std::string("qwerty\0uiop\0zxcvbnm\0", 20));
-    sql << "select text_value, blob_value, longblob_value "
-        << "from soci_test where id = 3",
-           into(text), into(blob), into(longblob);
+    sql << "select text_value from soci_test where id = 3", into(text);
     CHECK(text == a);
-    CHECK(blob == b);
-    CHECK(longblob == c);
-    sql << "select text_value, blob_value, longblob_value "
-        << "from soci_test where id = 4",
-           into(text), into(blob), into(longblob);
+    sql << "select text_value from soci_test where id = 4", into(text);
     CHECK(text == x);
-    CHECK(blob == y);
-    CHECK(longblob == z);
 
     rowset<row> rs =
-        (sql.prepare << "select text_value, blob_value, longblob_value "
-                        "from soci_test order by id");
+        (sql.prepare << "select text_value from soci_test order by id");
     rowset<row>::const_iterator r = rs.begin();
     CHECK(r->get_properties(0).get_data_type() == dt_string);
+    CHECK(r->get_properties(0).get_db_type() == db_string);
     CHECK(r->get<std::string>(0) == "foo");
-    CHECK(r->get_properties(1).get_data_type() == dt_string);
-    CHECK(r->get<std::string>(1) == "bar");
-    CHECK(r->get_properties(2).get_data_type() == dt_string);
-    CHECK(r->get<std::string>(2) == "baz");
     ++r;
     CHECK(r->get_properties(0).get_data_type() == dt_string);
+    CHECK(r->get_properties(0).get_db_type() == db_string);
     CHECK(r->get<std::string>(0) == std::string("qwerty\0uiop", 11));
-    CHECK(r->get_properties(1).get_data_type() == dt_string);
-    CHECK(r->get<std::string>(1) == std::string("zxcv\0bnm", 8));
-    CHECK(r->get_properties(2).get_data_type() == dt_string);
-    CHECK(r->get<std::string>(2) ==
-           std::string("qwerty\0uiop\0zxcvbnm\0", 20));
     ++r;
     CHECK(r->get_properties(0).get_data_type() == dt_string);
+    CHECK(r->get_properties(0).get_db_type() == db_string);
     CHECK(r->get<std::string>(0) == a);
-    CHECK(r->get_properties(1).get_data_type() == dt_string);
-    CHECK(r->get<std::string>(1) == b);
-    CHECK(r->get_properties(2).get_data_type() == dt_string);
-    CHECK(r->get<std::string>(2) == c);
     ++r;
     CHECK(r->get_properties(0).get_data_type() == dt_string);
+    CHECK(r->get_properties(0).get_db_type() == db_string);
     CHECK(r->get<std::string>(0) == x);
-    CHECK(r->get_properties(1).get_data_type() == dt_string);
-    CHECK(r->get<std::string>(1) == y);
-    CHECK(r->get_properties(2).get_data_type() == dt_string);
-    CHECK(r->get<std::string>(2) == z);
     ++r;
     CHECK(r == rs.end());
 
@@ -686,8 +639,10 @@ TEST_CASE("MySQL tinyint", "[mysql][int][tinyint]")
     sql << "select val from soci_test", into(r);
     REQUIRE(r.size() == 1);
     CHECK(r.get_properties("val").get_data_type() == dt_long_long);
+    CHECK(r.get_properties("val").get_db_type() == db_uint32);
     CHECK(r.get<long long>("val") == 0xffffff00);
     CHECK(r.get<unsigned>("val") == 0xffffff00);
+    CHECK(r.get<uint32_t>("val") == 0xffffff00);
   }
   {
     soci::session sql(backEnd, connectString);
@@ -697,7 +652,9 @@ TEST_CASE("MySQL tinyint", "[mysql][int][tinyint]")
     sql << "select val from soci_test", into(r);
     REQUIRE(r.size() == 1);
     CHECK(r.get_properties("val").get_data_type() == dt_integer);
+    CHECK(r.get_properties("val").get_db_type() == db_int8);
     CHECK(r.get<int>("val") == -123);
+    CHECK(r.get<int8_t>("val") == -123);
   }
   {
     soci::session sql(backEnd, connectString);
@@ -707,7 +664,9 @@ TEST_CASE("MySQL tinyint", "[mysql][int][tinyint]")
     sql << "select val from soci_test", into(r);
     REQUIRE(r.size() == 1);
     CHECK(r.get_properties("val").get_data_type() == dt_integer);
+    CHECK(r.get_properties("val").get_db_type() == db_uint8);
     CHECK(r.get<int>("val") == 123);
+    CHECK(r.get<uint8_t>("val") == 123);
   }
   {
     soci::session sql(backEnd, connectString);
@@ -717,7 +676,9 @@ TEST_CASE("MySQL tinyint", "[mysql][int][tinyint]")
     sql << "select val from soci_test", into(r);
     REQUIRE(r.size() == 1);
     CHECK(r.get_properties("val").get_data_type() == dt_unsigned_long_long);
+    CHECK(r.get_properties("val").get_db_type() == db_uint64);
     CHECK(r.get<unsigned long long>("val") == 123456789012345ULL);
+    CHECK(r.get<uint64_t>("val") == 123456789012345ULL);
   }
   {
     soci::session sql(backEnd, connectString);
@@ -727,7 +688,9 @@ TEST_CASE("MySQL tinyint", "[mysql][int][tinyint]")
     sql << "select val from soci_test", into(r);
     REQUIRE(r.size() == 1);
     CHECK(r.get_properties("val").get_data_type() == dt_long_long);
+    CHECK(r.get_properties("val").get_db_type() == db_int64);
     CHECK(r.get<long long>("val") == -123456789012345LL);
+    CHECK(r.get<int64_t>("val") == -123456789012345LL);
   }
 }
 
@@ -738,8 +701,7 @@ struct strings_table_creator : table_creator_base
     {
         sql << "create table soci_test(s1 char(20), s2 varchar(20), "
             "s3 tinytext, s4 mediumtext, s5 text, s6 longtext, "
-            "b1 binary(20), b2 varbinary(20), b3 tinyblob, b4 mediumblob, "
-            "b5 blob, b6 longblob, e1 enum ('foo', 'bar', 'baz'))";
+            "b1 binary(20), b2 varbinary(20), e1 enum ('foo', 'bar', 'baz'))";
     }
 };
 
@@ -750,21 +712,20 @@ TEST_CASE("MySQL strings", "[mysql][string]")
     std::string text = "Ala ma kota.";
     std::string binary("Ala\0ma\0kota.........", 20);
     sql << "insert into soci_test "
-        "(s1, s2, s3, s4, s5, s6, b1, b2, b3, b4, b5, b6, e1) values "
-        "(:s1, :s2, :s3, :s4, :d5, :s6, :b1, :b2, :b3, :b4, :b5, :b6, "
-        "\'foo\')",
+        "(s1, s2, s3, s4, s5, s6, b1, b2, e1) values "
+        "(:s1, :s2, :s3, :s4, :d5, :s6, :b1, :b2, 'foo')",
         use(text), use(text), use(text), use(text), use(text), use(text),
-        use(binary), use(binary), use(binary), use(binary), use(binary),
-        use(binary);
+        use(binary), use(binary);
     row r;
-    sql << "select s1, s2, s3, s4, s5, s6, b1, b2, b3, b4, b5, b6, e1 "
+    sql << "select s1, s2, s3, s4, s5, s6, b1, b2, e1 "
         "from soci_test", into(r);
-    REQUIRE(r.size() == 13);
-    for (int i = 0; i < 13; i++) {
+    REQUIRE(r.size() == 9);
+    for (int i = 0; i < ssize(r); i++) {
         CHECK(r.get_properties(i).get_data_type() == dt_string);
+        CHECK(r.get_properties(i).get_db_type() == db_string);
         if (i < 6) {
             CHECK(r.get<std::string>(i) == text);
-        } else if (i < 12) {
+        } else if (i < 8) {
             CHECK(r.get<std::string>(i) == binary);
         } else {
             CHECK(r.get<std::string>(i) == "foo");
@@ -794,6 +755,338 @@ TEST_CASE("MySQL last insert id", "[mysql][last-insert-id]")
     CHECK(id == 42);
 }
 
+TEST_CASE("MySQL DDL with metadata", "[mysql][ddl]")
+{
+    soci::session sql(backEnd, connectString);
+
+    // note: prepare_column_descriptions expects l-value
+    std::string ddl_t1 = "ddl_t1";
+    std::string ddl_t2 = "ddl_t2";
+    std::string ddl_t3 = "ddl_t3";
+
+    // single-expression variant:
+    sql.create_table(ddl_t1).column("i", soci::dt_integer).column("j", soci::dt_integer);
+
+    // check whether this table was created:
+
+    bool ddl_t1_found = false;
+    bool ddl_t2_found = false;
+    bool ddl_t3_found = false;
+    std::string table_name;
+    soci::statement st = (sql.prepare_table_names(), into(table_name));
+    st.execute();
+    while (st.fetch())
+    {
+        if (table_name == ddl_t1) { ddl_t1_found = true; }
+        if (table_name == ddl_t2) { ddl_t2_found = true; }
+        if (table_name == ddl_t3) { ddl_t3_found = true; }
+    }
+
+    CHECK(ddl_t1_found);
+    CHECK(ddl_t2_found == false);
+    CHECK(ddl_t3_found == false);
+
+    // check whether ddl_t1 has the right structure:
+
+    bool i_found = false;
+    bool j_found = false;
+    bool other_found = false;
+    soci::column_info ci;
+    soci::statement st1 = (sql.prepare_column_descriptions(ddl_t1), into(ci));
+    st1.execute();
+    while (st1.fetch())
+    {
+        if (ci.name == "i")
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable);
+            i_found = true;
+        }
+        else if (ci.name == "j")
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable);
+            j_found = true;
+        }
+        else
+        {
+            other_found = true;
+        }
+    }
+
+    CHECK(i_found);
+    CHECK(j_found);
+    CHECK(other_found == false);
+
+    // two more tables:
+
+    // separately defined columns:
+    // (note: statement is executed when ddl object goes out of scope)
+    {
+        soci::ddl_type ddl = sql.create_table(ddl_t2);
+        ddl.column("i", soci::dt_integer);
+        ddl.column("j", soci::dt_integer);
+        ddl.column("k", soci::dt_integer)("not null");
+        ddl.primary_key("t2_pk", "j");
+    }
+
+    sql.add_column(ddl_t1, "k", soci::dt_integer);
+    sql.add_column(ddl_t1, "big", soci::dt_string, 0); // "unlimited" length -> text
+    sql.drop_column(ddl_t1, "i");
+
+    // or with constraint as in t2:
+    sql.add_column(ddl_t2, "m", soci::dt_integer)("not null");
+
+    // third table with a foreign key to the second one
+    {
+        soci::ddl_type ddl = sql.create_table(ddl_t3);
+        ddl.column("x", soci::dt_integer);
+        ddl.column("y", soci::dt_integer);
+        ddl.foreign_key("t3_fk", "x", ddl_t2, "j");
+    }
+
+    // check if all tables were created:
+
+    ddl_t1_found = false;
+    ddl_t2_found = false;
+    ddl_t3_found = false;
+    soci::statement st2 = (sql.prepare_table_names(), into(table_name));
+    st2.execute();
+    while (st2.fetch())
+    {
+        if (table_name == ddl_t1) { ddl_t1_found = true; }
+        if (table_name == ddl_t2) { ddl_t2_found = true; }
+        if (table_name == ddl_t3) { ddl_t3_found = true; }
+    }
+
+    CHECK(ddl_t1_found);
+    CHECK(ddl_t2_found);
+    CHECK(ddl_t3_found);
+
+    // check if ddl_t1 has the right structure (it was altered):
+
+    i_found = false;
+    j_found = false;
+    bool k_found = false;
+    bool big_found = false;
+    other_found = false;
+    soci::statement st3 = (sql.prepare_column_descriptions(ddl_t1), into(ci));
+    st3.execute();
+    while (st3.fetch())
+    {
+        if (ci.name == "j")
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable);
+            j_found = true;
+        }
+        else if (ci.name == "k")
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable);
+            k_found = true;
+        }
+        else if (ci.name == "big")
+        {
+            CHECK(ci.type == soci::dt_string);
+            CHECK(ci.dataType == soci::db_string);
+            CHECK(ci.precision == 0); // "unlimited" for strings
+            big_found = true;
+        }
+        else
+        {
+            other_found = true;
+        }
+    }
+
+    CHECK(i_found == false);
+    CHECK(j_found);
+    CHECK(k_found);
+    CHECK(big_found);
+    CHECK(other_found == false);
+
+    // check if ddl_t2 has the right structure:
+
+    i_found = false;
+    j_found = false;
+    k_found = false;
+    bool m_found = false;
+    other_found = false;
+    soci::statement st4 = (sql.prepare_column_descriptions(ddl_t2), into(ci));
+    st4.execute();
+    while (st4.fetch())
+    {
+        if (ci.name == "i")
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable);
+            i_found = true;
+        }
+        else if (ci.name == "j")
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable == false); // primary key
+            j_found = true;
+        }
+        else if (ci.name == "k")
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable == false);
+            k_found = true;
+        }
+        else if (ci.name == "m")
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable == false);
+            m_found = true;
+        }
+        else
+        {
+            other_found = true;
+        }
+    }
+
+    CHECK(i_found);
+    CHECK(j_found);
+    CHECK(k_found);
+    CHECK(m_found);
+    CHECK(other_found == false);
+
+    sql.drop_table(ddl_t1);
+    sql.drop_table(ddl_t3); // note: this must be dropped before ddl_t2
+    sql.drop_table(ddl_t2);
+
+    // check if all tables were dropped:
+
+    ddl_t1_found = false;
+    ddl_t2_found = false;
+    ddl_t3_found = false;
+    st2 = (sql.prepare_table_names(), into(table_name));
+    st2.execute();
+    while (st2.fetch())
+    {
+        if (table_name == ddl_t1) { ddl_t1_found = true; }
+        if (table_name == ddl_t2) { ddl_t2_found = true; }
+        if (table_name == ddl_t3) { ddl_t3_found = true; }
+    }
+
+    CHECK(ddl_t1_found == false);
+    CHECK(ddl_t2_found == false);
+    CHECK(ddl_t3_found == false);
+}
+
+// Test cross-schema metadata
+TEST_CASE("Cross-schema metadata", "[mysql][cross-schema]")
+{
+    soci::session sql(backEnd, connectString);
+
+    // note: prepare_column_descriptions expects l-value
+    std::string tables = "tables";
+    std::string column_name = "TABLE_NAME";
+
+    // Get the database name - which happens to be the schema
+    // name in generic DB lingo.
+    std::string schema;
+    sql << "select DATABASE()", into(schema);
+    CHECK(!schema.empty());
+
+    // create a table in local schema (database) with the same name
+    // as the tables table in the information_schema
+    sql.create_table(tables).column(column_name, soci::dt_integer);
+
+    // check whether this table was created:
+
+    bool tables_found = false;
+    std::string table_name;
+    soci::statement st1 = (sql.prepare_table_names(), into(table_name));
+    st1.execute();
+    while (st1.fetch())
+    {
+        if (table_name == tables)
+        {
+            tables_found = true;
+        }
+    }
+
+    CHECK(tables_found);
+
+    // Get information for the tables table we just created and not
+    // the tables table in information_schema which isn't in our path.
+    int  records = 0;
+    soci::column_info ci;
+    soci::statement st2 = (sql.prepare_column_descriptions(tables), into(ci));
+    st2.execute();
+    while (st2.fetch())
+    {
+        if (ci.name == column_name)
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            records++;
+        }
+    }
+
+    CHECK(records == 1);
+
+    // Run the same query but this time specific with the schema.
+    std::string schemaTables = schema + "." + tables;
+    records = 0;
+    soci::statement st3 = (sql.prepare_column_descriptions(schemaTables), into(ci));
+    st3.execute();
+    while (st3.fetch())
+    {
+        if (ci.name == column_name)
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            records++;
+        }
+    }
+
+    CHECK(records == 1);
+
+    // Finally run the query with the information_schema.
+    std::string information_schemaTables = "information_schema." + tables;
+    records = 0;
+    soci::statement st4 = (sql.prepare_column_descriptions(information_schemaTables), into(ci));
+    st4.execute();
+    while (st4.fetch())
+    {
+        if (ci.name == column_name)
+        {
+            CHECK(ci.type == soci::dt_string);
+            CHECK(ci.dataType == soci::db_string);
+            records++;
+        }
+    }
+
+    CHECK(records == 1);
+
+    // Delete table and check that it is gone
+    sql.drop_table(tables);
+    tables_found = false;
+    st4 = (sql.prepare_table_names(), into(table_name));
+    st4.execute();
+    while (st4.fetch())
+    {
+        if (ci.name == column_name)
+        {
+            tables_found = true;
+        }
+    }
+
+    CHECK(tables_found == false);
+}
+
+
 std::string escape_string(soci::session& sql, const std::string& s)
 {
     mysql_session_backend* backend = static_cast<mysql_session_backend*>(
@@ -836,29 +1129,4 @@ void test15()
     std::cout << "test 15 passed" << std::endl;
 }
 
-int main(int argc, char** argv)
-{
-    if (argc >= 2)
-    {
-        connectString = argv[1];
-
-        // Replace the connect string with the process name to ensure that
-        // CATCH uses the correct name in its messages.
-        argv[1] = argv[0];
-
-        argc--;
-        argv++;
-    }
-    else
-    {
-        std::cout << "usage: " << argv[0]
-            << " connectstring [test-arguments...]\n"
-            << "example: " << argv[0]
-            << " \"dbname=test user=root password=\'Ala ma kota\'\"\n";
-        std::exit(1);
-    }
-
-    test_context tc(backEnd, connectString);
-
-    return Catch::Session().run(argc, argv);
-}
+test_context tc_mysql;
