@@ -2,17 +2,19 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, David Courtney
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #include "soci/soci.h"
 #include "soci/odbc/soci-odbc.h"
-#include "common-tests.h"
+#include "test-context.h"
 #include <iostream>
 #include <string>
 #include <ctime>
 #include <cmath>
 
+#include <catch.hpp>
+
 using namespace soci;
 using namespace soci::tests;
 
@@ -26,8 +28,8 @@ struct table_creator_one : public table_creator_base
         : table_creator_base(sql)
     {
         sql << "create table soci_test(id integer, val integer, c char, "
-                 "str varchar(20), sh integer, ul number, d float, "
-                 "num76 numeric(7,6), "
+                 "str varchar(20), sh integer, ll number, ul number, "
+                 "d float, num76 numeric(7,6), "
                  "tm timestamp, i1 integer, i2 integer, i3 integer, "
                  "name varchar(20))";
     }
@@ -66,12 +68,21 @@ struct table_creator_for_get_affected_rows : table_creator_base
 // Support for SOCI Common Tests
 //
 
-class test_context : public test_context_base
+class test_context : public test_context_common
 {
 public:
 
-    test_context(backend_factory const &backend, std::string const &connstr)
-        : test_context_base(backend, connstr) {}
+    test_context() = default;
+
+    std::string get_example_connection_string() const override
+    {
+        return "FILEDSN=./test-access.dsn";
+    }
+
+    std::string get_backend_name() const override
+    {
+        return "odbc";
+    }
 
     table_creator_base * table_creator_1(soci::session& s) const
     {
@@ -114,35 +125,4 @@ public:
     }
 };
 
-int main(int argc, char** argv)
-{
-
-#ifdef _MSC_VER
-    // Redirect errors, unrecoverable problems, and assert() failures to STDERR,
-    // instead of debug message window.
-    // This hack is required to run assert()-driven tests by Buildbot.
-    // NOTE: Comment this 2 lines for debugging with Visual C++ debugger to catch assertions inside.
-    _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE);
-    _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDERR);
-#endif //_MSC_VER
-
-    if (argc >= 2 && argv[1][0] != '-')
-    {
-        connectString = argv[1];
-
-        // Replace the connect string with the process name to ensure that
-        // CATCH uses the correct name in its messages.
-        argv[1] = argv[0];
-
-        argc--;
-        argv++;
-    }
-    else
-    {
-        connectString = "FILEDSN=./test-access.dsn";
-    }
-
-    test_context tc(backEnd, connectString);
-
-    return Catch::Session().run(argc, argv);
-}
+test_context tc_odbc_access;
