@@ -2,17 +2,19 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, David Courtney
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #include "soci/soci.h"
 #include "soci/odbc/soci-odbc.h"
-#include "common-tests.h"
+#include "test-context.h"
 #include <iostream>
 #include <string>
 #include <ctime>
 #include <cmath>
 
+#include <catch.hpp>
+
 using namespace soci;
 using namespace soci::tests;
 
@@ -71,10 +73,124 @@ TEST_CASE("MS SQL long string", "[odbc][mssql][long]")
     // column.
     CHECK_THROWS_AS(
         (sql << "insert into soci_test(fixed_text) values(:str)", use(str_in)),
-        soci_error&
+        soci_error
     );
 }
 
+struct wide_text_table_creator : public table_creator_base
+{
+  explicit wide_text_table_creator(soci::session &sql)
+      : table_creator_base(sql)
+  {
+      sql << "create table soci_test ("
+                  "wide_text nvarchar(40) null"
+              ")";
+  }
+};
+
+TEST_CASE("MS SQL wide string", "[odbc][mssql][wstring]")
+{
+  soci::session sql(backEnd, connectString);
+
+  wide_text_table_creator create_wide_text_table(sql);
+
+  std::wstring const str_in = L"Привет, SOCI!";
+
+  sql << "insert into soci_test(wide_text) values(:str)", use(str_in);
+
+  std::wstring str_out;
+  sql << "select wide_text from soci_test", into(str_out);
+
+  CHECK(str_out == str_in);
+}
+
+TEST_CASE("MS SQL wide string vector", "[odbc][mssql][vector][wstring]")
+{
+  soci::session sql(backEnd, connectString);
+
+  wide_text_table_creator create_wide_text_table(sql);
+
+  std::vector<std::wstring> const str_in = {
+      L"Привет, SOCI!",
+      L"Привет, World!",
+      L"Привет, Universe!",
+      L"Привет, Galaxy!"};
+
+  sql << "insert into soci_test(wide_text) values(:str)", use(str_in);
+
+  std::vector<std::wstring> str_out(4);
+
+  sql << "select wide_text from soci_test", into(str_out);
+
+  CHECK(str_out.size() == str_in.size());
+  for (std::size_t i = 0; i != str_in.size(); ++i)
+  {
+    CHECK(str_out[i] == str_in[i]);
+  }
+}
+
+TEST_CASE("MS SQL table records count", "[odbc][mssql][count]")
+{
+    soci::session sql(backEnd, connectString);
+
+    // Execute the provided SQL query to count records in tables
+    std::string sql_query = R"(
+        SET NOCOUNT ON;
+        DECLARE db_cursor CURSOR FOR
+        SELECT name FROM sys.databases
+        WHERE state_desc = 'ONLINE'
+        AND name IN ('master');
+        DECLARE @DatabaseName NVARCHAR(128);
+        DECLARE @outset TABLE(
+            INSTANCENAME varchar(50),
+            DATABASENAME varchar(100),
+            TABLENAME varchar(100),
+            NUMBEROFRECORDS_I bigint
+        );
+        OPEN db_cursor;
+        FETCH NEXT FROM db_cursor INTO @DatabaseName;
+        WHILE @@FETCH_STATUS = 0
+        BEGIN
+            DECLARE @command nvarchar(1000) = 'USE ' + QUOTENAME(@DatabaseName) +
+            '; SELECT @@SERVERNAME, DB_NAME(), T.NAME, P.[ROWS] FROM sys.tables T ' +
+            'INNER JOIN sys.indexes I ON T.OBJECT_ID = I.OBJECT_ID ' +
+            'INNER JOIN sys.partitions P ON I.OBJECT_ID = P.OBJECT_ID AND I.INDEX_ID = P.INDEX_ID ' +
+            'INNER JOIN sys.allocation_units A ON P.PARTITION_ID = A.CONTAINER_ID ' +
+            'WHERE T.NAME NOT LIKE ''DT%'' AND I.OBJECT_ID > 255 AND I.INDEX_ID <= 1 ' +
+            'GROUP BY T.NAME, I.OBJECT_ID, I.INDEX_ID, I.NAME, P.[ROWS] ' +
+            'ORDER BY OBJECT_NAME(I.OBJECT_ID)';
+            INSERT INTO @outset EXEC (@command)
+            FETCH NEXT FROM db_cursor INTO @DatabaseName
+        END
+        CLOSE db_cursor
+        DEALLOCATE db_cursor
+        SELECT INSTANCENAME, DATABASENAME, TABLENAME, NUMBEROFRECORDS_I
+        FROM @outset;
+    )";
+
+    soci::rowset<soci::row> rs = (sql.prepare << sql_query);
+
+    // Check that we can access the results.
+    for (auto it = rs.begin(); it != rs.end(); ++it)
+    {
+        soci::row const& row = *it;
+        std::string instance_name = row.get<std::string>(0);
+        std::string database_name = row.get<std::string>(1);
+        std::string table_name = row.get<std::string>(2);
+        long long number_of_records = row.get<long long>(3);
+
+        // Use the variables above to avoid warnings about unused variables and
+        // check the only one of them that we can be sure about because we have
+        // "name IN ('master')" in the SQL query above.
+        INFO("Table " << instance_name << "." << table_name <<
+             " has " << number_of_records << " records");
+        CHECK( database_name == "master" );
+        return;
+    }
+
+    FAIL("No tables found in the master database");
+}
+
 // DDL Creation objects for common tests
 struct table_creator_one : public table_creator_base
 {
@@ -82,8 +198,8 @@ struct table_creator_one : public table_creator_base
         : table_creator_base(sql)
     {
         sql << "create table soci_test(id integer, val integer, c char, "
-                 "str varchar(20), sh smallint, ul numeric(20), d float, "
-                 "num76 numeric(7,6), "
+                 "str varchar(20), sh smallint, ll bigint, ul numeric(20), "
+                 "d float, num76 numeric(7,6), "
                  "tm datetime, i1 integer, i2 integer, i3 integer, "
                  "name varchar(20))";
     }
@@ -149,59 +265,67 @@ struct table_creator_for_get_last_insert_id : table_creator_base
 // Support for SOCI Common Tests
 //
 
-class test_context : public test_context_base
+class test_context : public test_context_common
 {
 public:
-    test_context(backend_factory const &backend,
-                std::string const &connstr)
-        : test_context_base(backend, connstr) {}
+    test_context() = default;
 
-    table_creator_base* table_creator_1(soci::session& s) const SOCI_OVERRIDE
+    std::string get_example_connection_string() const override
+    {
+        return "FILEDSN=./test-mssql.dsn";
+    }
+
+    std::string get_backend_name() const override
+    {
+        return "odbc";
+    }
+
+    table_creator_base* table_creator_1(soci::session& s) const override
     {
         return new table_creator_one(s);
     }
 
-    table_creator_base* table_creator_2(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_2(soci::session& s) const override
     {
         return new table_creator_two(s);
     }
 
-    table_creator_base* table_creator_3(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_3(soci::session& s) const override
     {
         return new table_creator_three(s);
     }
 
-    table_creator_base * table_creator_4(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base * table_creator_4(soci::session& s) const override
     {
         return new table_creator_for_get_affected_rows(s);
     }
 
-    tests::table_creator_base* table_creator_clob(soci::session& s) const SOCI_OVERRIDE
+    tests::table_creator_base* table_creator_clob(soci::session& s) const override
     {
         return new table_creator_for_clob(s);
     }
 
-    tests::table_creator_base* table_creator_xml(soci::session& s) const SOCI_OVERRIDE
+    tests::table_creator_base* table_creator_xml(soci::session& s) const override
     {
         return new table_creator_for_xml(s);
     }
 
-    tests::table_creator_base* table_creator_get_last_insert_id(soci::session& s) const SOCI_OVERRIDE
+    tests::table_creator_base* table_creator_get_last_insert_id(soci::session& s) const override
     {
         return new table_creator_for_get_last_insert_id(s);
     }
 
-    bool has_real_xml_support() const SOCI_OVERRIDE
+    bool has_real_xml_support() const override
     {
         return true;
     }
 
-    std::string to_date_time(std::string const &datdt_string) const SOCI_OVERRIDE
+    std::string to_date_time(std::string const &datdt_string) const override
     {
         return "convert(datetime, \'" + datdt_string + "\', 120)";
     }
 
-    bool has_multiple_select_bug() const SOCI_OVERRIDE
+    bool has_multiple_select_bug() const override
     {
         // MS SQL does support MARS (multiple active result sets) since 2005
         // version, but this support needs to be explicitly enabled and is not
@@ -210,40 +334,10 @@ public:
         return true;
     }
 
-    std::string sql_length(std::string const& s) const SOCI_OVERRIDE
+    std::string sql_length(std::string const& s) const override
     {
         return "len(" + s + ")";
     }
 };
 
-int main(int argc, char** argv)
-{
-#ifdef _MSC_VER
-    // Redirect errors, unrecoverable problems, and assert() failures to STDERR,
-    // instead of debug message window.
-    // This hack is required to run assert()-driven tests by Buildbot.
-    // NOTE: Comment this 2 lines for debugging with Visual C++ debugger to catch assertions inside.
-    _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE);
-    _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDERR);
-#endif //_MSC_VER
-
-    if (argc >= 2 && argv[1][0] != '-')
-    {
-        connectString = argv[1];
-
-        // Replace the connect string with the process name to ensure that
-        // CATCH uses the correct name in its messages.
-        argv[1] = argv[0];
-
-        argc--;
-        argv++;
-    }
-    else
-    {
-        connectString = "FILEDSN=./test-mssql.dsn";
-    }
-
-    test_context tc(backEnd, connectString);
-
-    return Catch::Session().run(argc, argv);
-}
+test_context tc_odbc_mssql;
