@@ -2,18 +2,20 @@
 // Copyright (C) 2004-2006 Maciej Sobczak, Stephen Hutton, David Courtney
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #include "soci/soci.h"
 #include "soci/odbc/soci-odbc.h"
-#include "common-tests.h"
+#include "test-context.h"
 #include <iostream>
 #include <string>
 #include <cstdio>
 #include <ctime>
 #include <cmath>
 
+#include <catch.hpp>
+
 using namespace soci;
 using namespace soci::tests;
 
@@ -50,7 +52,7 @@ public:
             char buf[128];
             // This uses the ODBC convention of padding the minor and release
             // versions with 0 and might be not appropriate in general.
-            std::sprintf(buf, "%u.%02u.%04u", major_, minor_, release_);
+            snprintf(buf, sizeof(buf), "%u.%02u.%04u", major_, minor_, release_);
             return buf;
         }
         else
@@ -92,8 +94,8 @@ struct table_creator_one : public table_creator_base
         : table_creator_base(sql)
     {
         sql << "create table soci_test(id integer, val integer, c char, "
-                 "str varchar(20), sh int2, ul numeric(20), d float8, "
-                 "num76 numeric(7,6), "
+                 "str varchar(20), sh int2, ll bigint, ul numeric(20), "
+                 "d float8, num76 numeric(7,6), "
                  "tm timestamp, i1 integer, i2 integer, i3 integer, "
                  "name varchar(20))";
     }
@@ -150,58 +152,74 @@ struct table_creator_for_clob : table_creator_base
 // Support for SOCI Common Tests
 //
 
-class test_context : public test_context_base
+class test_context : public test_context_common
 {
 public:
-    test_context(backend_factory const &backend,
-                std::string const &connstr)
-        : test_context_base(backend, connstr),
-          m_verDriver(get_driver_version())
+    test_context() = default;
+
+    bool initialize_connect_string(std::string argFromCommandLine) override
     {
+        if (!test_context_base::initialize_connect_string(argFromCommandLine))
+            return false;
+
+        m_verDriver = get_driver_version();
+
         std::cout << "Using ODBC driver version " << m_verDriver << "\n";
+
+        return true;
     }
 
-    table_creator_base * table_creator_1(soci::session& s) const SOCI_OVERRIDE
+    std::string get_example_connection_string() const override
+    {
+        return "FILEDSN=./test-postgresql.dsn";
+    }
+
+    std::string get_backend_name() const override
+    {
+        return "odbc";
+    }
+
+    table_creator_base * table_creator_1(soci::session& s) const override
     {
         return new table_creator_one(s);
     }
 
-    table_creator_base * table_creator_2(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base * table_creator_2(soci::session& s) const override
     {
         return new table_creator_two(s);
     }
 
-    table_creator_base * table_creator_3(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base * table_creator_3(soci::session& s) const override
     {
         return new table_creator_three(s);
     }
 
-    table_creator_base * table_creator_4(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base * table_creator_4(soci::session& s) const override
     {
         return new table_creator_for_get_affected_rows(s);
     }
 
-    table_creator_base* table_creator_xml(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_xml(soci::session& s) const override
     {
         return new table_creator_for_xml(s);
     }
 
-    table_creator_base* table_creator_clob(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_clob(soci::session& s) const override
     {
         return new table_creator_for_clob(s);
     }
 
-    bool has_real_xml_support() const SOCI_OVERRIDE
+    bool has_real_xml_support() const override
     {
         return true;
     }
 
-    std::string to_date_time(std::string const &datdt_string) const SOCI_OVERRIDE
+    std::string to_date_time(std::string const &datdt_string) const override
     {
         return "timestamptz(\'" + datdt_string + "\')";
     }
 
-    bool has_fp_bug() const SOCI_OVERRIDE
+    bool has_fp_bug() const override
     {
         // The bug with using insufficiently many digits for double values was
         // only fixed in 9.03.0400 version of the ODBC driver (see commit
@@ -212,7 +230,22 @@ public:
         return !m_verDriver.is_initialized() || m_verDriver < odbc_version(9, 3, 400);
     }
 
-    std::string fix_crlf_if_necessary(std::string const& s) const SOCI_OVERRIDE
+    bool has_partial_update_bug() const override
+    {
+        // ODBC driver has version-dependent bugs related to handling array
+        // parameters: after v13.02, it fails to insert anything, see
+        // https://github.com/postgresql-interfaces/psqlodbc/issues/89, and
+        // with the previous versions (e.g. v10.03) it's even worse, as it does
+        // insert the row with valid values but still returns fatal error at
+        // ODBC level.
+        //
+        // So far there is no known version where it works correctly, but if
+        // the issue above is fixed, we should check for the version including
+        // this fix here.
+        return true;
+    }
+
+    std::string fix_crlf_if_necessary(std::string const& s) const override
     {
         // Version 9.03.0300 (ancient, but still used on AppVeyor CI) is known
         // to have a bug which replaces new lines, i.e. LF characters, with CR
@@ -233,7 +266,7 @@ public:
         return s2;
     }
 
-    std::string sql_length(std::string const& s) const SOCI_OVERRIDE
+    std::string sql_length(std::string const& s) const override
     {
         return "char_length(" + s + ")";
     }
@@ -279,38 +312,7 @@ private:
         }
     }
 
-    odbc_version const m_verDriver;
+    odbc_version m_verDriver;
 };
 
-int main(int argc, char** argv)
-{
-
-#ifdef _MSC_VER
-    // Redirect errors, unrecoverable problems, and assert() failures to STDERR,
-    // instead of debug message window.
-    // This hack is required to run assert()-driven tests by Buildbot.
-    // NOTE: Comment this 2 lines for debugging with Visual C++ debugger to catch assertions inside.
-    _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE);
-    _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDERR);
-#endif //_MSC_VER
-
-    if (argc >= 2 && argv[1][0] != '-')
-    {
-        connectString = argv[1];
-
-        // Replace the connect string with the process name to ensure that
-        // CATCH uses the correct name in its messages.
-        argv[1] = argv[0];
-
-        argc--;
-        argv++;
-    }
-    else
-    {
-        connectString = "FILEDSN=./test-postgresql.dsn";
-    }
-
-    test_context tc(backEnd, connectString);
-
-    return Catch::Session().run(argc, argv);
-}
+test_context tc_odbc_postgresql;
