@@ -2,24 +2,105 @@
 // // Copyright (C) 2004-2007 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
-#include "common-tests.h"
 #include "soci/soci.h"
 #include "soci/oracle/soci-oracle.h"
+#include "test-assert.h"
+#include "test-context.h"
+#include "test-myint.h"
+#include <iomanip>
 #include <iostream>
 #include <string>
 #include <cstdlib>
 #include <cstring>
 #include <ctime>
 
+#include <catch.hpp>
+
 using namespace soci;
 using namespace soci::tests;
 
 std::string connectString;
 backend_factory const &backEnd = *soci::factory_oracle();
 
+// Helpers for creating tables for different tests.
+struct table_creator_one : public table_creator_base
+{
+    table_creator_one(soci::session & sql)
+        : table_creator_base(sql)
+    {
+        sql << "create table soci_test(id number(10,0), val number(8,0), c char, "
+                 "str varchar2(20), sh number, ll number, ul number, d number, "
+                 "num76 numeric(7,6), "
+                 "tm date, i1 number, i2 number, i3 number, name varchar2(20))";
+    }
+};
+
+struct table_creator_two : public table_creator_base
+{
+    table_creator_two(soci::session & sql)
+        : table_creator_base(sql)
+    {
+        sql  << "create table soci_test(num_float number, num_int numeric(4,0),"
+                    " name varchar2(20), sometime date, chr char)";
+    }
+};
+
+struct table_creator_three : public table_creator_base
+{
+    table_creator_three(soci::session & sql)
+        : table_creator_base(sql)
+    {
+        sql << "create table soci_test(name varchar2(100) not null, "
+            "phone varchar2(15))";
+    }
+};
+
+struct table_creator_four : public table_creator_base
+{
+    table_creator_four(soci::session & sql)
+        : table_creator_base(sql)
+    {
+        sql << "create table soci_test(val number)";
+    }
+};
+
+struct table_creator_for_xml : table_creator_base
+{
+    table_creator_for_xml(soci::session& sql)
+        : table_creator_base(sql)
+    {
+        sql << "create table soci_test(id integer, x xmltype)";
+    }
+};
+
+struct table_creator_for_clob : table_creator_base
+{
+    table_creator_for_clob(soci::session& sql)
+        : table_creator_base(sql)
+    {
+        sql << "create table soci_test(id integer, s clob)";
+    }
+};
+
+struct table_creator_for_blob : public tests::table_creator_base
+{
+    table_creator_for_blob(soci::session &sql) : tests::table_creator_base(sql)
+    {
+        sql << "create table soci_test(id integer, b blob)";
+    }
+};
+
+struct table_creator_for_timestamp : public tests::table_creator_base
+{
+    table_creator_for_timestamp(soci::session &sql) : tests::table_creator_base(sql)
+    {
+        sql << "create table soci_test(id integer, t timestamp (6))";
+    }
+};
+
 // Extra tests for date/time
 TEST_CASE("Oracle datetime", "[oracle][datetime]")
 {
@@ -85,6 +166,54 @@ TEST_CASE("Oracle datetime", "[oracle][datetime]")
 
         CHECK(t_out == std::string(buf));
     }
+
+    {
+        // date and time - between years 1- 2201
+        soci::session sql(backEnd, connectString);
+
+        table_creator_for_timestamp tableCreator(sql);
+
+        for(int i = 100; i <= 2201; i = i + 50)
+        {
+            char t[10];
+            snprintf(t, sizeof(t), "%04d", i);
+
+            std::string date = std::string(t) + "-03-28 14:06:13";
+            std::tm t1 {}, t2 {}, t4 {};
+
+            std::istringstream is(date.c_str());
+            is.imbue(std::locale(setlocale(LC_ALL, nullptr)));
+            is >> std::get_time(&t2, "%Y-%m-%d %H:%M:%S");
+            REQUIRE(!is.fail());
+
+            std::tm t3 = t2;
+            sql << "select t from (select :t as t from dual)",
+                into(t1), use(t3);
+
+            char buf1[25];
+            char buf2[25];
+            strftime(buf1, sizeof(buf1), "%Y-%m-%d %H:%M:%S", &t1);
+            strftime(buf2, sizeof(buf2), "%Y-%m-%d %H:%M:%S", &t2);
+            CHECK(std::string(buf1) == std::string(buf2));
+            CHECK(t1.tm_sec == t2.tm_sec);
+            CHECK(t1.tm_min == t2.tm_min);
+            CHECK(t1.tm_hour == t2.tm_hour);
+            CHECK(t1.tm_mday == t2.tm_mday);
+            CHECK(t1.tm_mon == t2.tm_mon);
+            CHECK(t1.tm_year == t2.tm_year);
+            CHECK((1900 + t1.tm_year) == i);
+
+            sql << "insert into soci_test(id, t) values(:i, :t)", use(i), use(t3);
+            sql << "select t from soci_test where id = :i", use(i), into(t4);
+            CHECK(t4.tm_sec == t2.tm_sec);
+            CHECK(t4.tm_min == t2.tm_min);
+            CHECK(t4.tm_hour == t2.tm_hour);
+            CHECK(t4.tm_mday == t2.tm_mday);
+            CHECK(t4.tm_mon == t2.tm_mon);
+            CHECK(t4.tm_year == t2.tm_year);
+            CHECK((1900 + t4.tm_year) == i);
+        }
+    }   
 }
 
 // explicit calls test
@@ -102,116 +231,7 @@ TEST_CASE("Oracle explicit calls", "[oracle]")
     CHECK(i == 7);
 }
 
-// DDL + blob test
-
-struct blob_table_creator : public table_creator_base
-{
-    blob_table_creator(soci::session & sql)
-    : table_creator_base(sql)
-    {
-        sql <<
-            "create table soci_test ("
-            "    id number(10) not null,"
-            "    img blob"
-            ")";
-    }
-};
-
-TEST_CASE("Oracle blob", "[oracle][blob]")
-{
-    {
-        soci::session sql(backEnd, connectString);
-
-        blob_table_creator tableCreator(sql);
-
-        char buf[] = "abcdefghijklmnopqrstuvwxyz";
-        sql << "insert into soci_test (id, img) values (7, empty_blob())";
-
-        {
-            blob b(sql);
-
-            oracle_session_backend *sessionBackEnd
-                = static_cast<oracle_session_backend *>(sql.get_backend());
-
-            oracle_blob_backend *blobBackEnd
-                = static_cast<oracle_blob_backend *>(b.get_backend());
-
-            OCILobDisableBuffering(sessionBackEnd->svchp_,
-                sessionBackEnd->errhp_, blobBackEnd->lobp_);
-
-            sql << "select img from soci_test where id = 7", into(b);
-            CHECK(b.get_len() == 0);
-
-            // note: blob offsets start from 1
-            b.write(1, buf, sizeof(buf));
-            CHECK(b.get_len() == sizeof(buf));
-            b.trim(10);
-            CHECK(b.get_len() == 10);
-
-            // append does not work (Oracle bug #886191 ?)
-            //b.append(buf, sizeof(buf));
-            //assert(b.get_len() == sizeof(buf) + 10);
-            sql.commit();
-        }
-
-        {
-            blob b(sql);
-            sql << "select img from soci_test where id = 7", into(b);
-            //assert(b.get_len() == sizeof(buf) + 10);
-            CHECK(b.get_len() == 10);
-            char buf2[100];
-            b.read(1, buf2, 10);
-            CHECK(strncmp(buf2, "abcdefghij", 10) == 0);
-        }
-    }
-
-    // additional sibling test for read_from_start and write_from_start
-    {
-        soci::session sql(backEnd, connectString);
-
-        blob_table_creator tableCreator(sql);
-
-        char buf[] = "abcdefghijklmnopqrstuvwxyz";
-        sql << "insert into soci_test (id, img) values (7, empty_blob())";
-
-        {
-            blob b(sql);
-
-            oracle_session_backend *sessionBackEnd
-                = static_cast<oracle_session_backend *>(sql.get_backend());
-
-            oracle_blob_backend *blobBackEnd
-                = static_cast<oracle_blob_backend *>(b.get_backend());
-
-            OCILobDisableBuffering(sessionBackEnd->svchp_,
-                sessionBackEnd->errhp_, blobBackEnd->lobp_);
-
-            sql << "select img from soci_test where id = 7", into(b);
-            CHECK(b.get_len() == 0);
-
-            // note: blob offsets start from 1
-            b.write_from_start(buf, sizeof(buf));
-            CHECK(b.get_len() == sizeof(buf));
-            b.trim(10);
-            CHECK(b.get_len() == 10);
-
-            // append does not work (Oracle bug #886191 ?)
-            //b.append(buf, sizeof(buf));
-            //assert(b.get_len() == sizeof(buf) + 10);
-            sql.commit();
-        }
-
-        {
-            blob b(sql);
-            sql << "select img from soci_test where id = 7", into(b);
-            //assert(b.get_len() == sizeof(buf) + 10);
-            CHECK(b.get_len() == 10);
-            char buf2[100];
-            b.read_from_start(buf2, 10);
-            CHECK(strncmp(buf2, "abcdefghij", 10) == 0);
-        }
-    }
-}
+// DDL test
 
 // nested statement test
 // (the same syntax is used for output cursors in PL/SQL)
@@ -287,10 +307,30 @@ TEST_CASE("Oracle rowid", "[oracle][rowid]")
 }
 
 // Stored procedures
-struct procedure_creator : procedure_creator_base
+class creator_base
+{
+public:
+    explicit creator_base(session& sql, std::string what = "procedure soci_test")
+        : msession(sql), what_{std::move(what)} { drop(); }
+
+    ~creator_base() { drop();}
+
+private:
+    void drop()
+    {
+        try { msession << "drop " + what_; } catch (soci_error&) {}
+    }
+    session& msession;
+
+    std::string const what_;
+
+    SOCI_NOT_COPYABLE(creator_base)
+};
+
+struct procedure_creator : creator_base
 {
     procedure_creator(soci::session & sql)
-        : procedure_creator_base(sql)
+        : creator_base(sql)
     {
         sql <<
              "create or replace procedure soci_test(output out varchar2,"
@@ -356,20 +396,20 @@ namespace soci
     };
 }
 
-struct in_out_procedure_creator : public procedure_creator_base
+struct in_out_procedure_creator : public creator_base
 {
     in_out_procedure_creator(soci::session & sql)
-        : procedure_creator_base(sql)
+        : creator_base(sql)
     {
         sql << "create or replace procedure soci_test(s in out varchar2)"
                 " as begin s := s || s; end;";
     }
 };
 
-struct returns_null_procedure_creator : public procedure_creator_base
+struct returns_null_procedure_creator : public creator_base
 {
     returns_null_procedure_creator(soci::session & sql)
-        : procedure_creator_base(sql)
+        : creator_base(sql)
     {
         sql << "create or replace procedure soci_test(s in out varchar2)"
             " as begin s := NULL; end;";
@@ -523,13 +563,13 @@ TEST_CASE("Oracle bulk insert", "[oracle][insert][bulk]")
     //verify an exception is thrown if into vector is zero length
     {
         std::vector<int> ids;
-        CHECK_THROWS_AS((sql << "select id from soci_test", into(ids)), soci_error&);
+        CHECK_THROWS_AS((sql << "select id from soci_test", into(ids)), soci_error);
     }
 
     // verify an exception is thrown if use vector is zero length
     {
         std::vector<int> ids;
-        CHECK_THROWS_AS((sql << "insert into soci_test(id) values(:id)", use(ids)), soci_error&);
+        CHECK_THROWS_AS((sql << "insert into soci_test(id) values(:id)", use(ids)), soci_error);
     }
 
     // test "no data" condition
@@ -805,10 +845,10 @@ struct person_table_creator : public table_creator_base
     }
 };
 
-struct times100_procedure_creator : public procedure_creator_base
+struct times100_procedure_creator : public creator_base
 {
     times100_procedure_creator(soci::session & sql)
-        : procedure_creator_base(sql)
+        : creator_base(sql)
     {
         sql << "create or replace procedure soci_test(id in out number)"
                " as begin id := id * 100; end;";
@@ -1078,8 +1118,8 @@ TEST_CASE("Oracle to_number with rowset", "[oracle][rowset][to_number]")
         rs = (sql.prepare << "select to_number('123456789012345') from dual");
     double d = rs.begin()->get<double>(0);
     ASSERT_EQUAL_EXACT(d, 123456789012345);
-
-    rs = (sql.prepare << "select to_number(:t) from dual", use(3.14));
+    const double pi = 3.14;
+    rs = (sql.prepare << "select to_number(:t) from dual", use(pi));
     d = rs.begin()->get<double>(0);
     ASSERT_EQUAL_EXACT(d, 3.14);
 }
@@ -1438,134 +1478,151 @@ TEST_CASE("Bulk iterators", "[oracle][bulkiters]")
     sql << "drop table t";
 }
 
+TEST_CASE ( "Oracle CLOB", "[oracle][clob]" )
+{
+    soci::session sql ( backEnd, connectString );
+
+    // Use a non-ASCII string to test that CLOBs work when byte count differs
+    // from character count.
+    //
+    // Note that this requires some Unicode encoding to be used, e.g. UTF-8 by
+    // setting NLS_LANG=.AL32UTF8 in the environment.
+    std::string const test_utf8{"Привет système"};
+
+    table_creator_for_clob clob_table(sql);
+    long_string ls;
+    ls.value = test_utf8;
+    sql << "insert into soci_test(id, s) values(1, :s)", use(ls);
+    ls.value.clear();
+    sql << "select s from soci_test where id=1", into(ls);
+    CHECK(ls.value == test_utf8);
+
+    // RAII helper to create the function we use below.
+    struct soci_repeat : creator_base
+    {
+        explicit soci_repeat(session& sql)
+            : creator_base(sql, "function soci_repeat")
+        {
+            sql << R"(
+create function soci_repeat(s in string, xCount in integer) return clob as
+  tmp clob;
+begin
+  for i in 1..xCount loop tmp := tmp || s; end loop;
+  return tmp;
+end;
+)"
+                ;
+        }
+    } soci_repeat_func(sql);
+
+    // Append a big number of Unicode chars to the CLOB to test that things
+    // work with CLOBs larger than a single chunk (which is ~8KiB by default).
+    //
+    // Note: Oracle 11 used in the CI tests doesn't seem to handle characters
+    // outside of the BMP correctly even if current Oracle versions have no
+    // problems with them, so don't use them here for now.
+    unsigned xCount = 10000;
+    sql << "select :s || soci_repeat('Я', :xCount) from dual",
+           use(ls), use(xCount), into(ls);
+
+    REQUIRE(ls.value.length() == test_utf8.length() + 2*xCount);
+}
+
 //
 // Support for soci Common Tests
 //
 
-struct table_creator_one : public table_creator_base
-{
-    table_creator_one(soci::session & sql)
-        : table_creator_base(sql)
-    {
-        sql << "create table soci_test(id number(10,0), val number(4,0), c char, "
-                 "str varchar2(20), sh number, ul number, d number, "
-                 "num76 numeric(7,6), "
-                 "tm date, i1 number, i2 number, i3 number, name varchar2(20))";
-    }
-};
-
-struct table_creator_two : public table_creator_base
-{
-    table_creator_two(soci::session & sql)
-        : table_creator_base(sql)
-    {
-        sql  << "create table soci_test(num_float number, num_int numeric(4,0),"
-                    " name varchar2(20), sometime date, chr char)";
-    }
-};
-
-struct table_creator_three : public table_creator_base
-{
-    table_creator_three(soci::session & sql)
-        : table_creator_base(sql)
-    {
-        sql << "create table soci_test(name varchar2(100) not null, "
-            "phone varchar2(15))";
-    }
-};
-
-struct table_creator_four : public table_creator_base
-{
-    table_creator_four(soci::session & sql)
-        : table_creator_base(sql)
-    {
-        sql << "create table soci_test(val number)";
-    }
-};
-
-struct table_creator_for_xml : table_creator_base
-{
-    table_creator_for_xml(soci::session& sql)
-        : table_creator_base(sql)
-    {
-        sql << "create table soci_test(id integer, x xmltype)";
-    }
-};
-
-struct table_creator_for_clob : table_creator_base
-{
-    table_creator_for_clob(soci::session& sql)
-        : table_creator_base(sql)
-    {
-        sql << "create table soci_test(id integer, s clob)";
-    }
-};
-
-class test_context :public test_context_base
+class test_context :public test_context_common
 {
 public:
-    test_context(backend_factory const &backEnd,
-                std::string const &connectString)
-        : test_context_base(backEnd, connectString) {}
+    test_context() = default;
 
-    table_creator_base* table_creator_1(soci::session& s) const SOCI_OVERRIDE
+    std::string get_example_connection_string() const override
+    {
+        return "service=orcl user=scott password=tiger";
+    }
+
+    bool start_testing() override
+    {
+        if (!std::getenv("ORACLE_HOME"))
+        {
+            std::cerr << "ORACLE_HOME environment variable must be defined for Oracle tests.\n";
+            return false;
+        }
+
+        return test_context_base::start_testing();
+    }
+
+    std::string get_backend_name() const override
+    {
+        return "oracle";
+    }
+
+    table_creator_base* table_creator_1(soci::session& s) const override
     {
         return new table_creator_one(s);
     }
 
-    table_creator_base* table_creator_2(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_2(soci::session& s) const override
     {
         return new table_creator_two(s);
     }
 
-    table_creator_base* table_creator_3(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_3(soci::session& s) const override
     {
         return new table_creator_three(s);
     }
 
-    table_creator_base* table_creator_4(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_4(soci::session& s) const override
     {
         return new table_creator_four(s);
     }
 
-    table_creator_base* table_creator_clob(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_clob(soci::session& s) const override
     {
         return new table_creator_for_clob(s);
     }
 
-    table_creator_base* table_creator_xml(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_blob(soci::session& s) const override
+    {
+      return new table_creator_for_blob(s);
+    }
+
+    table_creator_base* table_creator_xml(soci::session& s) const override
     {
         return new table_creator_for_xml(s);
     }
 
-    std::string to_xml(std::string const& x) const SOCI_OVERRIDE
+    std::string to_xml(std::string const& x) const override
     {
-        return "xmltype(" + x + ")";
+        // We can't apply xmltype() to null values as this appears to parse
+        // empty string as XML -- and failing.
+        return "case when " + x  + " is not null then xmltype(" + x + ") else null end";
     }
 
-    std::string from_xml(std::string const& x) const SOCI_OVERRIDE
+    std::string from_xml(std::string const& x) const override
     {
         // Notice that using just x.getCLOBVal() doesn't work, only
         // table.x.getCLOBVal() or (x).getCLOBVal(), as used here, does.
         return "(" + x + ").getCLOBVal()";
     }
 
-    bool has_real_xml_support() const SOCI_OVERRIDE
+    bool has_real_xml_support() const override
     {
         return true;
     }
 
-    bool treats_empty_strings_as_null() const SOCI_OVERRIDE
+    bool treats_empty_strings_as_null() const override
     {
         return true;
     }
 
-    std::string to_date_time(std::string const &datdt_string) const SOCI_OVERRIDE
+    std::string to_date_time(std::string const &datdt_string) const override
     {
         return "to_date('" + datdt_string + "', 'YYYY-MM-DD HH24:MI:SS')";
     }
 
-    std::string sql_length(std::string const& s) const SOCI_OVERRIDE
+    std::string sql_length(std::string const& s) const override
     {
         // Oracle treats empty strings as NULLs, but we want to return the
         // length of 0 for them for consistency with the other backends, so use
@@ -1574,44 +1631,4 @@ public:
     }
 };
 
-int main(int argc, char** argv)
-{
-#ifdef _MSC_VER
-    // Redirect errors, unrecoverable problems, and assert() failures to STDERR,
-    // instead of debug message window.
-    // This hack is required to run assert()-driven tests by Buildbot.
-    // NOTE: Comment this 2 lines for debugging with Visual C++ debugger to catch assertions inside.
-    _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE);
-    _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDERR);
-#endif //_MSC_VER
-
-    if (argc >= 2)
-    {
-        connectString = argv[1];
-
-        // Replace the connect string with the process name to ensure that
-        // CATCH uses the correct name in its messages.
-        argv[1] = argv[0];
-
-        argc--;
-        argv++;
-    }
-    else
-    {
-        std::cout << "usage: " << argv[0]
-            << " connectstring [test-arguments...]\n"
-            << "example: " << argv[0]
-            << " \'service=orcl user=scott password=tiger\'\n";
-        std::exit(1);
-    }
-
-    if (!std::getenv("ORACLE_HOME"))
-    {
-        std::cerr << "ORACLE_HOME environment variable must be defined for Oracle tests.\n";
-        std::exit(1);
-    }
-
-    test_context tc(backEnd, connectString);
-
-    return Catch::Session().run(argc, argv);
-}
+test_context tc_oracle;
