@@ -2,12 +2,13 @@
 // Copyright (C) 2004-2008 Maciej Sobczak, Stephen Hutton
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt or copy at
-// http://www.boost.org/LICENSE_1_0.txt)
+// https://www.boost.org/LICENSE_1_0.txt)
 //
 
 #include "soci/soci.h"
 #include "soci/postgresql/soci-postgresql.h"
-#include "common-tests.h"
+#include "test-context.h"
+#include "test-myint.h"
 #include <iostream>
 #include <sstream>
 #include <string>
@@ -16,6 +17,8 @@
 #include <ctime>
 #include <cstdlib>
 
+#include <catch.hpp>
+
 using namespace soci;
 using namespace soci::tests;
 
@@ -24,6 +27,97 @@ backend_factory const &backEnd = *soci::factory_postgresql();
 
 // Postgres-specific tests
 
+enum TestStringEnum
+{
+    VALUE_STR_1=0,
+    VALUE_STR_2,
+    VALUE_STR_3
+};
+
+enum TestIntEnum
+{
+    VALUE_INT_1=0,
+    VALUE_INT_2,
+    VALUE_INT_3
+};
+
+namespace soci {
+template <> struct type_conversion<TestStringEnum>
+{
+    typedef std::string base_type;
+    static void from_base(const std::string & v, indicator & ind, TestStringEnum & p)
+    {
+        if ( ind == i_null )
+            throw soci_error("Null value not allowed for this type");
+
+        if ( v.compare("A") == 0 )
+            p = TestStringEnum::VALUE_STR_1;
+        else if ( v.compare("B") == 0 )
+            p = TestStringEnum::VALUE_STR_2;
+        else if ( v.compare("C") == 0 )
+            p = TestStringEnum::VALUE_STR_3;
+        else
+            throw soci_error("Value not allowed for this type");
+    }
+    static void to_base(TestStringEnum & p, std::string & v, indicator & ind)
+    {
+        switch ( p )
+        {
+        case TestStringEnum::VALUE_STR_1:
+            v = "A";
+            ind = i_ok;
+            return;
+        case TestStringEnum::VALUE_STR_2:
+            v = "B";
+            ind = i_ok;
+            return;
+        case TestStringEnum::VALUE_STR_3:
+            v = "C";
+            ind = i_ok;
+            return;
+        default:
+            throw soci_error("Value not allowed for this type");
+        }
+    }
+};
+
+template <> struct type_conversion<TestIntEnum>
+{
+    typedef int base_type;
+    static void from_base(const int & v, indicator & ind, TestIntEnum & p)
+    {
+        if ( ind == i_null )
+            throw soci_error("Null value not allowed for this type");
+
+        switch( v )
+        {
+        case 0:
+        case 1:
+        case 2:
+            p = (TestIntEnum)v;
+            ind = i_ok;
+            return;
+        default:
+            throw soci_error("Null value not allowed for this type");
+        }
+    }
+    static void to_base(TestIntEnum & p, int & v, indicator & ind)
+    {
+        switch( p )
+        {
+        case TestIntEnum::VALUE_INT_1:
+        case TestIntEnum::VALUE_INT_2:
+        case TestIntEnum::VALUE_INT_3:
+            v = (int)p;
+            ind = i_ok;
+            return;
+        default:
+            throw soci_error("Value not allowed for this type");
+        }
+    }
+};
+}
+
 struct oid_table_creator : public table_creator_base
 {
     oid_table_creator(soci::session& sql)
@@ -36,6 +130,23 @@ struct oid_table_creator : public table_creator_base
     }
 };
 
+TEST_CASE("PostgreSQL connection string", "[postgresql][connstring]")
+{
+    // There are no required parts in libpq connection string, so we can only
+    // test that invalid options are detected.
+    CHECK_THROWS_WITH(soci::session(backEnd, "bloordyblop=1"),
+                      Catch::Contains(R"(invalid connection option "bloordyblop")"));
+
+    CHECK_THROWS_WITH(soci::session(backEnd, "sslmode=bloordyblop"),
+                      Catch::Contains(R"(invalid sslmode value: "bloordyblop")"));
+
+    // This tests that quoted strings work as expected
+    CHECK_THROWS_WITH(soci::session(backEnd, "sslmode='dummy value'"),
+                      Catch::Contains(R"(invalid sslmode value: "dummy value")"));
+    CHECK_THROWS_WITH(soci::session(backEnd, "sslmode=\"dummy value\""),
+                      Catch::Contains(R"(invalid sslmode value: "dummy value")"));
+}
+
 // ROWID test
 // Note: in PostgreSQL, there is no ROWID, there is OID.
 // It is still provided as a separate type for "portability",
@@ -80,13 +191,14 @@ TEST_CASE("PostgreSQL prepare error", "[postgresql][exception]")
 }
 
 // function call test
-class function_creator : function_creator_base
+class function_creator
 {
 public:
-
-    function_creator(soci::session & sql)
-    : function_creator_base(sql)
+    explicit function_creator(soci::session & sql)
+        : sql_(sql)
     {
+        drop();
+
         // before a language can be used it must be defined
         // if it has already been defined then an error will occur
         try { sql << "create language plpgsql"; }
@@ -101,12 +213,16 @@ public:
             "end $$ language plpgsql";
     }
 
-protected:
+    ~function_creator() { drop(); }
 
-    std::string drop_statement()
+private:
+    void drop()
     {
-        return "drop function soci_test(varchar)";
+        try { sql_ << "drop function soci_test(varchar)"; } catch (soci_error&) {}
     }
+    session& sql_;
+
+    SOCI_NOT_COPYABLE(function_creator)
 };
 
 TEST_CASE("PostgreSQL function call", "[postgresql][function]")
@@ -137,100 +253,6 @@ TEST_CASE("PostgreSQL function call", "[postgresql][function]")
     }
 }
 
-// BLOB test
-struct blob_table_creator : public table_creator_base
-{
-    blob_table_creator(soci::session & sql)
-    : table_creator_base(sql)
-    {
-        sql <<
-             "create table soci_test ("
-             "    id integer,"
-             "    img oid"
-             ")";
-    }
-};
-
-TEST_CASE("PostgreSQL blob", "[postgresql][blob]")
-{
-    {
-        soci::session sql(backEnd, connectString);
-
-        blob_table_creator tableCreator(sql);
-
-        char buf[] = "abcdefghijklmnopqrstuvwxyz";
-
-        sql << "insert into soci_test(id, img) values(7, lo_creat(-1))";
-
-        // in PostgreSQL, BLOB operations must be within transaction block
-        transaction tr(sql);
-
-        {
-            blob b(sql);
-
-            sql << "select img from soci_test where id = 7", into(b);
-            CHECK(b.get_len() == 0);
-
-            b.write(0, buf, sizeof(buf));
-            CHECK(b.get_len() == sizeof(buf));
-
-            b.append(buf, sizeof(buf));
-            CHECK(b.get_len() == 2 * sizeof(buf));
-        }
-        {
-            blob b(sql);
-            sql << "select img from soci_test where id = 7", into(b);
-            CHECK(b.get_len() == 2 * sizeof(buf));
-            char buf2[100];
-            b.read(0, buf2, 10);
-            CHECK(std::strncmp(buf2, "abcdefghij", 10) == 0);
-        }
-
-        unsigned long oid;
-        sql << "select img from soci_test where id = 7", into(oid);
-        sql << "select lo_unlink(" << oid << ")";
-    }
-
-    // additional sibling test for read_from_start and write_from_start
-    {
-        soci::session sql(backEnd, connectString);
-
-        blob_table_creator tableCreator(sql);
-
-        char buf[] = "abcdefghijklmnopqrstuvwxyz";
-
-        sql << "insert into soci_test(id, img) values(7, lo_creat(-1))";
-
-        // in PostgreSQL, BLOB operations must be within transaction block
-        transaction tr(sql);
-
-        {
-            blob b(sql);
-
-            sql << "select img from soci_test where id = 7", into(b);
-            CHECK(b.get_len() == 0);
-
-            b.write_from_start(buf, sizeof(buf));
-            CHECK(b.get_len() == sizeof(buf));
-
-            b.append(buf, sizeof(buf));
-            CHECK(b.get_len() == 2 * sizeof(buf));
-        }
-        {
-            blob b(sql);
-            sql << "select img from soci_test where id = 7", into(b);
-            CHECK(b.get_len() == 2 * sizeof(buf));
-            char buf2[100];
-            b.read_from_start(buf2, 10);
-            CHECK(std::strncmp(buf2, "abcdefghij", 10) == 0);
-        }
-
-        unsigned long oid;
-        sql << "select img from soci_test where id = 7", into(oid);
-        sql << "select lo_unlink(" << oid << ")";
-    }
-}
-
 struct longlong_table_creator : table_creator_base
 {
     longlong_table_creator(soci::session & sql)
@@ -319,13 +341,18 @@ TEST_CASE("PostgreSQL boolean", "[postgresql][boolean]")
     sql << "insert into soci_test(val) values(:val)", use(i1);
 
     int i2 = 7;
+    row r;
     sql << "select val from soci_test", into(i2);
+    sql << "select val from soci_test", into(r);
 
     CHECK(i2 == i1);
+    CHECK(r.get<int8_t>(0) == i1);
 
     sql << "update soci_test set val = true";
     sql << "select val from soci_test", into(i2);
+    sql << "select val from soci_test", into(r);
     CHECK(i2 == 1);
+    CHECK(r.get<int8_t>(0) == 1);
 }
 
 struct uuid_table_creator : table_creator_base
@@ -404,9 +431,7 @@ TEST_CASE("PostgreSQL literals", "[postgresql][into]")
     }
     catch (soci_error const & e)
     {
-        char const * expectedPrefix = "Cannot convert data";
-        CAPTURE(e.what());
-        CHECK(strncmp(e.what(), expectedPrefix, strlen(expectedPrefix)) == 0);
+        CHECK_THAT( e.what(), Catch::StartsWith("Cannot convert data") );
     }
 }
 
@@ -582,6 +607,7 @@ TEST_CASE("PostgreSQL bytea", "[postgresql][bytea]")
         REQUIRE(r.size() == 1);
         column_properties const& props = r.get_properties(0);
         CHECK(props.get_data_type() == soci::dt_string);
+        CHECK(props.get_db_type() == soci::db_string);
         std::string bin2 = r.get<std::string>(0);
         CHECK(bin2 == expectedBytea);
     }
@@ -632,7 +658,7 @@ TEST_CASE("PostgreSQL JSON", "[postgresql][json]")
 
         CHECK_THROWS_AS((
             sql << "insert into soci_json_test (data) values(:data)",use(invalid_input)),
-            soci_error&
+            soci_error
         );
     }
     else
@@ -666,14 +692,11 @@ TEST_CASE("PostgreSQL statement prepare failure", "[postgresql][prepare]")
                 << "select * from soci_test where name=9999");
         FAIL("expected exception not thrown");
     }
-    catch(soci_error const& e)
+    catch (postgresql_soci_error const& e)
     {
-        std::string const msg(e.what());
-        CAPTURE(msg);
+        CHECK( e.get_error_category() == soci_error::invalid_statement );
 
-        // poor-man heuristics
-        CHECK(msg.find("prepared statement") == std::string::npos);
-        CHECK(msg.find("operator does not exist") != std::string::npos);
+        CHECK_THAT( e.what(), Catch::Contains("operator does not exist") );
     }
 }
 
@@ -686,6 +709,34 @@ TEST_CASE("PostgreSQL ORM cast", "[postgresql][orm]")
     sql << "select :a::int", use(v); // Must not throw an exception!
 }
 
+std::string get_table_name_without_schema(const std::string& table_name_with_schema)
+{
+    // Find the first occurrence of "."
+    size_t dotPos = table_name_with_schema.find('.');
+
+    // Check if the "." exists and there's exactly one "."
+    if (dotPos == std::string::npos || table_name_with_schema.find('.', dotPos + 1) != std::string::npos) {
+        return table_name_with_schema;
+    }
+
+    // Extract the substring after the "."
+    return table_name_with_schema.substr(dotPos + 1);
+}
+
+std::string get_schema_from_table_name(const std::string& table_name_with_schema)
+{
+    // Find the first occurrence of "."
+    size_t dotPos = table_name_with_schema.find('.');
+
+    // Check if the "." exists and there's exactly one "."
+    if (dotPos == std::string::npos || table_name_with_schema.find('.', dotPos + 1) != std::string::npos) {
+        return "";
+    }
+
+    // Extract the substring before the "."
+    return table_name_with_schema.substr(0, dotPos);
+}
+
 // Test the DDL and metadata functionality
 TEST_CASE("PostgreSQL DDL with metadata", "[postgresql][ddl]")
 {
@@ -709,9 +760,9 @@ TEST_CASE("PostgreSQL DDL with metadata", "[postgresql][ddl]")
     st.execute();
     while (st.fetch())
     {
-        if (table_name == ddl_t1) { ddl_t1_found = true; }
-        if (table_name == ddl_t2) { ddl_t2_found = true; }
-        if (table_name == ddl_t3) { ddl_t3_found = true; }
+        if (get_table_name_without_schema(table_name) == ddl_t1) { ddl_t1_found = true; }
+        if (get_table_name_without_schema(table_name) == ddl_t2) { ddl_t2_found = true; }
+        if (get_table_name_without_schema(table_name) == ddl_t3) { ddl_t3_found = true; }
     }
 
     CHECK(ddl_t1_found);
@@ -731,12 +782,14 @@ TEST_CASE("PostgreSQL DDL with metadata", "[postgresql][ddl]")
         if (ci.name == "i")
         {
             CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
             CHECK(ci.nullable);
             i_found = true;
         }
         else if (ci.name == "j")
         {
             CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
             CHECK(ci.nullable);
             j_found = true;
         }
@@ -786,9 +839,9 @@ TEST_CASE("PostgreSQL DDL with metadata", "[postgresql][ddl]")
     st2.execute();
     while (st2.fetch())
     {
-        if (table_name == ddl_t1) { ddl_t1_found = true; }
-        if (table_name == ddl_t2) { ddl_t2_found = true; }
-        if (table_name == ddl_t3) { ddl_t3_found = true; }
+        if (get_table_name_without_schema(table_name) == ddl_t1) { ddl_t1_found = true; }
+        if (get_table_name_without_schema(table_name) == ddl_t2) { ddl_t2_found = true; }
+        if (get_table_name_without_schema(table_name) == ddl_t3) { ddl_t3_found = true; }
     }
 
     CHECK(ddl_t1_found);
@@ -809,18 +862,21 @@ TEST_CASE("PostgreSQL DDL with metadata", "[postgresql][ddl]")
         if (ci.name == "j")
         {
             CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
             CHECK(ci.nullable);
             j_found = true;
         }
         else if (ci.name == "k")
         {
             CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
             CHECK(ci.nullable);
             k_found = true;
         }
         else if (ci.name == "big")
         {
             CHECK(ci.type == soci::dt_string);
+            CHECK(ci.dataType == soci::db_string);
             CHECK(ci.precision == 0); // "unlimited" for strings
             big_found = true;
         }
@@ -850,24 +906,28 @@ TEST_CASE("PostgreSQL DDL with metadata", "[postgresql][ddl]")
         if (ci.name == "i")
         {
             CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
             CHECK(ci.nullable);
             i_found = true;
         }
         else if (ci.name == "j")
         {
             CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
             CHECK(ci.nullable == false); // primary key
             j_found = true;
         }
         else if (ci.name == "k")
         {
             CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
             CHECK(ci.nullable == false);
             k_found = true;
         }
         else if (ci.name == "m")
         {
             CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
             CHECK(ci.nullable == false);
             m_found = true;
         }
@@ -896,9 +956,9 @@ TEST_CASE("PostgreSQL DDL with metadata", "[postgresql][ddl]")
     st2.execute();
     while (st2.fetch())
     {
-        if (table_name == ddl_t1) { ddl_t1_found = true; }
-        if (table_name == ddl_t2) { ddl_t2_found = true; }
-        if (table_name == ddl_t3) { ddl_t3_found = true; }
+        if (get_table_name_without_schema(table_name) == ddl_t1) { ddl_t1_found = true; }
+        if (get_table_name_without_schema(table_name) == ddl_t2) { ddl_t2_found = true; }
+        if (get_table_name_without_schema(table_name) == ddl_t3) { ddl_t3_found = true; }
     }
 
     CHECK(ddl_t1_found == false);
@@ -914,6 +974,108 @@ TEST_CASE("PostgreSQL DDL with metadata", "[postgresql][ddl]")
     CHECK(i == 2);
 }
 
+// Test cross-schema metadata
+TEST_CASE("Cross-schema metadata", "[postgresql][cross-schema]")
+{
+    soci::session sql(backEnd, connectString);
+
+    // note: prepare_column_descriptions expects l-value
+    std::string tables = "tables";
+    std::string column_name = "table_name";
+
+    // single-expression variant:
+    sql.create_table(tables).column(column_name, soci::dt_integer);
+
+    // check whether this table was created:
+
+    bool tables_found = false;
+    std::string schema;
+    std::string table_name;
+    soci::statement st = (sql.prepare_table_names(), into(table_name));
+    st.execute();
+    while (st.fetch())
+    {
+        if (get_table_name_without_schema(table_name) == tables) 
+        {
+            tables_found = true;
+            schema = get_schema_from_table_name(table_name);
+        }
+    }
+
+    CHECK(tables_found);
+    CHECK(!schema.empty());
+
+    // Get information for the tables table we just created and not
+    // the tables table in information_schema which isn't in our path.
+    int  records = 0;
+    soci::column_info ci;
+    soci::statement st1 = (sql.prepare_column_descriptions(tables), into(ci));
+    st1.execute();
+    while (st1.fetch())
+    {
+        if (ci.name == column_name)
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable);
+            records++;
+        }
+    }
+
+    CHECK(records == 1);
+
+    // Run the same query but this time specific with the schema.
+    std::string schemaTables = schema + "." + tables;
+    records = 0;
+    soci::statement st2 = (sql.prepare_column_descriptions(schemaTables), into(ci));
+    st2.execute();
+    while (st2.fetch())
+    {
+        if (ci.name == column_name)
+        {
+            CHECK(ci.type == soci::dt_integer);
+            CHECK(ci.dataType == soci::db_int32);
+            CHECK(ci.nullable);
+            records++;
+        }
+    }
+
+    CHECK(records == 1);
+
+    // Finally run the query with the information_schema.
+    std::string information_schemaTables = "information_schema." + tables;
+    records = 0;
+    soci::statement st3 = (sql.prepare_column_descriptions(information_schemaTables), into(ci));
+    st3.execute();
+    while (st3.fetch())
+    {
+        if (ci.name == column_name)
+        {
+            CHECK(ci.type == soci::dt_string);
+            CHECK(ci.dataType == soci::db_string);
+            CHECK(ci.nullable);
+            records++;
+        }
+    }
+
+    CHECK(records == 1);
+
+    // Delete table and check that it is gone
+    sql.drop_table(tables);
+    tables_found = false;
+    st3 = (sql.prepare_table_names(), into(table_name));
+    st3.execute();
+    while (st.fetch())
+    {
+        if (table_name == tables)
+        {
+            tables_found = true;
+        }
+    }
+
+    CHECK(tables_found == false);
+}
+
 // Test the bulk iterators functionality
 TEST_CASE("Bulk iterators", "[postgresql][bulkiters]")
 {
@@ -1057,6 +1219,170 @@ TEST_CASE("false_bind_variable_inside_identifier", "[postgresql][bind-variables]
     CHECK(type_value.compare("en_one")==0);
 }
 
+// test_enum_with_explicit_custom_type_string_rowset
+struct test_enum_with_explicit_custom_type_string_rowset : table_creator_base
+{
+    test_enum_with_explicit_custom_type_string_rowset(soci::session & sql)
+        : table_creator_base(sql)
+        , msession(sql)
+    {
+        try
+        {
+            sql << "CREATE TYPE EnumType AS ENUM ('A','B','C');";
+            sql << "CREATE TABLE soci_test (Type EnumType NOT NULL DEFAULT 'A');";
+        }
+        catch (...)
+        {
+            drop();
+        }
+
+    }
+    ~test_enum_with_explicit_custom_type_string_rowset()
+    {
+        drop();
+    }
+
+private:
+    void drop()
+    {
+        try
+        {
+            msession << "drop table if exists soci_test;";
+            msession << "DROP TYPE IF EXISTS EnumType ;";
+        }
+        catch (soci_error const& e){
+            std::cerr << e.what() << std::endl;
+        }
+    }
+    soci::session& msession;
+};
+
+TEST_CASE("test_enum_with_explicit_custom_type_string_rowset", "[postgresql][bind-variables]")
+{
+    TestStringEnum test_value = TestStringEnum::VALUE_STR_2;
+    TestStringEnum type_value;
+
+    {
+        soci::session sql(backEnd, connectString);
+        test_enum_with_explicit_custom_type_string_rowset tableCreator(sql);
+
+        statement s1 = (sql.prepare << "insert into soci_test values(:val);", use(test_value, "val"));
+        statement s2 = (sql.prepare << "SELECT Type FROM soci_test;");
+
+        s1.execute(false);
+
+        soci::row result;
+        s2.define_and_bind();
+        s2.exchange_for_rowset(soci::into(result));
+        s2.execute(true);
+
+        type_value = result.get<TestStringEnum>("type");
+    }
+
+    CHECK(type_value==TestStringEnum::VALUE_STR_2);
+}
+
+TEST_CASE("test_enum_with_explicit_custom_type_string_into", "[postgresql][bind-variables]")
+{
+    TestStringEnum test_value = TestStringEnum::VALUE_STR_2;
+    TestStringEnum type_value;
+
+    {
+        soci::session sql(backEnd, connectString);
+        test_enum_with_explicit_custom_type_string_rowset tableCreator(sql);
+
+        statement s1 = (sql.prepare << "insert into soci_test values(:val);", use(test_value, "val"));
+        statement s2 = (sql.prepare << "SELECT Type FROM soci_test;", into(type_value));
+
+        s1.execute(false);
+        s2.execute(true);
+    }
+
+    CHECK(type_value==TestStringEnum::VALUE_STR_2);
+}
+
+// test_enum_with_explicit_custom_type_int_rowset
+struct test_enum_with_explicit_custom_type_int_rowset : table_creator_base
+{
+    test_enum_with_explicit_custom_type_int_rowset(soci::session & sql)
+        : table_creator_base(sql)
+        , msession(sql)
+    {
+
+        try
+        {
+            sql << "CREATE TABLE soci_test( Type smallint)";
+            ;
+        }
+        catch (...)
+        {
+            drop();
+        }
+
+    }
+    ~test_enum_with_explicit_custom_type_int_rowset()
+    {
+        drop();
+    }
+
+private:
+    void drop()
+    {
+        try
+        {
+            msession << "drop table if exists soci_test;";
+        }
+        catch (soci_error const& e){
+            std::cerr << e.what() << std::endl;
+        }
+    }
+    soci::session& msession;
+};
+
+TEST_CASE("test_enum_with_explicit_custom_type_int_rowset", "[postgresql][bind-variables]")
+{
+    TestIntEnum test_value = TestIntEnum::VALUE_INT_2;
+    TestIntEnum type_value;
+
+    {
+        soci::session sql(backEnd, connectString);
+        test_enum_with_explicit_custom_type_int_rowset tableCreator(sql);
+
+        statement s1 = (sql.prepare << "insert into soci_test(Type) values(:val)", use(test_value, "val"));
+        statement s2 = (sql.prepare << "SELECT Type FROM soci_test ;");
+
+        s1.execute(false);
+
+        soci::row result;
+        s2.define_and_bind();
+        s2.exchange_for_rowset(soci::into(result));
+        s2.execute(true);
+
+        type_value = result.get<TestIntEnum>("type");
+    }
+
+    CHECK(type_value==TestIntEnum::VALUE_INT_2);
+}
+
+TEST_CASE("test_enum_with_explicit_custom_type_int_into", "[postgresql][bind-variables]")
+{
+    TestIntEnum test_value = TestIntEnum::VALUE_INT_2;
+    TestIntEnum type_value;
+
+    {
+        soci::session sql(backEnd, connectString);
+        test_enum_with_explicit_custom_type_int_rowset tableCreator(sql);
+
+        statement s1 = (sql.prepare << "insert into soci_test(Type) values(:val)", use(test_value, "val"));
+        statement s2 = (sql.prepare << "SELECT Type FROM soci_test ;", into(type_value));
+
+        s1.execute(false);
+        s2.execute(true);
+    }
+
+    CHECK(type_value==TestIntEnum::VALUE_INT_2);
+}
+
 // false_bind_variable_inside_identifier
 struct table_creator_colon_in_double_quotes_in_single_quotes :
     table_creator_base
@@ -1098,8 +1424,8 @@ struct table_creator_one : public table_creator_base
         : table_creator_base(sql)
     {
         sql << "create table soci_test(id integer, val integer, c char, "
-                 "str varchar(20), sh int2, ul numeric(20), d float8, "
-                 "num76 numeric(7,6), "
+                 "str varchar(20), sh int2, ll bigint, ul numeric(20), "
+                 "d float8, num76 numeric(7,6), "
                  "tm timestamp, i1 integer, i2 integer, i3 integer, "
                  "name varchar(20))";
     }
@@ -1152,98 +1478,85 @@ struct table_creator_for_clob : table_creator_base
     }
 };
 
-// Common tests context
-class test_context : public test_context_base
+struct table_creator_for_blob : public tests::table_creator_base
+{
+    table_creator_for_blob(soci::session & sql)
+		: tests::table_creator_base(sql)
+    {
+        sql << "create table soci_test(id integer, b oid)";
+    }
+};
+
+
+class test_context : public test_context_common
 {
 public:
-    test_context(backend_factory const &backend, std::string const &connstr)
-        : test_context_base(backend, connstr)
-    {}
+    test_context() = default;
 
-    table_creator_base* table_creator_1(soci::session& s) const SOCI_OVERRIDE
+    std::string get_example_connection_string() const override
+    {
+        return "host=localhost port=5432 dbname=test user=postgres password=postgres";
+    }
+
+    std::string get_backend_name() const override
+    {
+        return "postgresql";
+    }
+
+    table_creator_base* table_creator_1(soci::session& s) const override
     {
         return new table_creator_one(s);
     }
 
-    table_creator_base* table_creator_2(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_2(soci::session& s) const override
     {
         return new table_creator_two(s);
     }
 
-    table_creator_base* table_creator_3(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_3(soci::session& s) const override
     {
         return new table_creator_three(s);
     }
 
-    table_creator_base* table_creator_4(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_4(soci::session& s) const override
     {
         return new table_creator_for_get_affected_rows(s);
     }
 
-    table_creator_base* table_creator_xml(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_xml(soci::session& s) const override
     {
         return new table_creator_for_xml(s);
     }
 
-    table_creator_base* table_creator_clob(soci::session& s) const SOCI_OVERRIDE
+    table_creator_base* table_creator_clob(soci::session& s) const override
     {
         return new table_creator_for_clob(s);
     }
 
-    bool has_real_xml_support() const SOCI_OVERRIDE
+    table_creator_base* table_creator_blob(soci::session& s) const override
+    {
+        return new table_creator_for_blob(s);
+    }
+
+    bool has_real_xml_support() const override
     {
         return true;
     }
 
-    std::string to_date_time(std::string const &datdt_string) const SOCI_OVERRIDE
+    std::string to_date_time(std::string const &datdt_string) const override
     {
         return "timestamptz(\'" + datdt_string + "\')";
     }
 
-    bool has_fp_bug() const SOCI_OVERRIDE
+    bool has_fp_bug() const override
     {
         return false;
     }
 
-    std::string sql_length(std::string const& s) const SOCI_OVERRIDE
+    std::string sql_length(std::string const& s) const override
     {
         return "char_length(" + s + ")";
     }
 };
 
-int main(int argc, char** argv)
-{
-
-#ifdef _MSC_VER
-    // Redirect errors, unrecoverable problems, and assert() failures to STDERR,
-    // instead of debug message window.
-    // This hack is required to run assert()-driven tests by Buildbot.
-    // NOTE: Comment this 2 lines for debugging with Visual C++ debugger to catch assertions inside.
-    _CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE);
-    _CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDERR);
-#endif //_MSC_VER
-
-    if (argc >= 2)
-    {
-        connectString = argv[1];
-
-        // Replace the connect string with the process name to ensure that
-        // CATCH uses the correct name in its messages.
-        argv[1] = argv[0];
-
-        argc--;
-        argv++;
-    }
-    else
-    {
-        std::cout << "usage: " << argv[0]
-            << " connectstring [test-arguments...]\n"
-            << "example: " << argv[0]
-            << " \'connect_string_for_PostgreSQL\'\n";
-        return EXIT_FAILURE;
-    }
-
-    test_context tc(backEnd, connectString);
-
-    return Catch::Session().run(argc, argv);
-}
+test_context tc_postgresql;
